@@ -2,6 +2,10 @@
 #undef _DEBUG
 #undef CHECK_RANGE
 
+/* other config flags */
+#undef __STL_SOLTHREADS
+#undef __USE_MALLOC
+
 /* Define if the C++ compiler supports BOOL */
 #undef HAVE_BOOL
 
