@@ -254,7 +254,8 @@ void MAFThread::run()
 	{
 		// request mutex is locked here
 		if (m_queueRequest.size()==0)
-			m_waitRequest.wait(&m_mtxRequest);
+			if (!m_waitRequest.wait(&m_mtxRequest))
+				continue;
 		// request mutex is still locked here
 		pRequest = m_queueRequest.front();
 		m_queueRequest.pop_front();
@@ -392,6 +393,10 @@ void MAFThread::run()
 				break;
 			case AFR_CLOSE:
 			case AFR_STOP:
+				//if (!(pRequest->mode & AFM_WRITE))
+				//{
+				//	TRACE4("Closing file '", state.sPath, "'  fd = ", state.hFile);
+				//}
 				if (state.hFile>=0)
 					state.nReturn = close(state.hFile);
 				else
@@ -509,6 +514,7 @@ MAFDescriptor::MAFDescriptor(MAsyncFile* pAsyncFile) : MRCObject(true), m_pAsync
 MAFDescriptor::~MAFDescriptor()
 {
 	ASSERT(s_pContext);
+	ASSERT(m_actual.hFile < 0);
 	s_pContext->mutex.lock();
 	// here we have to notify the MAsyncFile object about completion
 	if (m_nMode == AFM_READ)
@@ -624,6 +630,7 @@ MAsyncFile::MAsyncFile(int mode /*=AFM_READWRITE*/)
 
 MAsyncFile::~MAsyncFile()
 {
+	Close();
 	m_pDesc->Detach();
 }
 
