@@ -54,6 +54,7 @@ MAsyncSocket::MAsyncSocket()
 	m_bSelectActive = false;
 	m_dwActivity = 0;
 	m_nLastError = errno;
+	m_nAsyncDnsReqID = -1;
 }
 
 MAsyncSocket::~MAsyncSocket()
@@ -67,10 +68,10 @@ BOOL MAsyncSocket::Create(UINT nSocketPort /*= 0*/, int nSocketType /*=SOCK_STRE
 		long lEvent /*= FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE*/,
 		LPCTSTR lpszSocketAddress /*=NULL*/)
 {
-#ifdef _DEBUG
-	if (lEvent & FD_WRITE)
-		TRACE2("WARNING: creating a socket with FD_WRITE on for ", typeid(*this).name());
-#endif
+//#ifdef _DEBUG
+//	if (lEvent & FD_WRITE)
+//		TRACE2("WARNING: creating a socket with FD_WRITE on for ", typeid(*this).name());
+//#endif
 	if (Socket(nSocketType, lEvent))
 	{
 		if (Bind(nSocketPort,lpszSocketAddress))
@@ -85,10 +86,10 @@ BOOL MAsyncSocket::Create(UINT nSocketPort /*= 0*/, int nSocketType /*=SOCK_STRE
 BOOL MAsyncSocket::Attach(SOCKET hSocket,
 		long lEvent /*= FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE*/)
 {
-#ifdef _DEBUG
-	if (lEvent & FD_WRITE)
-		TRACE2("WARNING: attaching a socket with FD_WRITE on for ", typeid(*this).name());
-#endif
+//#ifdef _DEBUG
+//	if (lEvent & FD_WRITE)
+//		TRACE2("WARNING: attaching a socket with FD_WRITE on for ", typeid(*this).name());
+//#endif
 	ASSERT(hSocket != INVALID_SOCKET);
 
 	m_hSocket = hSocket;
@@ -295,16 +296,40 @@ public:
 	}
 	void OnErrorDelayed() {
 		MAsyncSocket* pS = MAsyncSocket::FromHandle(m_hSocket);
-		if (!pS)
+		if (!pS || pS->m_nAsyncDnsReqID != m_nRequestID)
+		{
+			if (pS)
+			{
+				TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " is around, but ID is wrong");
+				TRACE4("OnResolveDelayed() : (pS->m_nAsyncDnsReqID != m_nRequestID) : ", pS->m_nAsyncDnsReqID, " != ", m_nRequestID);
+			}
+			else
+			{
+				TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " has gone");
+			}
 			return;
+		}
+		//TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " is still around");
 		pS->Close();
 		pS->SetLastError(GetLastError());
 		pS->OnClose(GetLastError());
 	}
 	void OnResolveDelayed() {
 		MAsyncSocket* pS = MAsyncSocket::FromHandle(m_hSocket);
-		if (!pS)
+		if (!pS || pS->m_nAsyncDnsReqID != m_nRequestID)
+		{
+			if (pS)
+			{
+				TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " is around, but ID is wrong");
+				TRACE4("OnResolveDelayed() : (pS->m_nAsyncDnsReqID != m_nRequestID) : ", pS->m_nAsyncDnsReqID, " != ", m_nRequestID);
+			}
+			else
+			{
+				TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " has gone");
+			}
 			return;
+		}
+		//TRACE3("OnResolveDelayed() : socket object for fd ", m_hSocket, " is still around");
 		SOCKADDR_IN sockAddr;
 		memset(&sockAddr,0,sizeof(sockAddr));
 		sockAddr.sin_family = AF_INET;
@@ -343,6 +368,7 @@ BOOL MAsyncSocket::Connect(LPCTSTR lpszHostAddress, UINT nHostPort)
 		lock.unlock();
 		SetLastError(EINPROGRESS);
 		MAsyncConnectByName* pACN = new MAsyncConnectByName(lpszHostAddress, nHostPort, m_hSocket);
+		m_nAsyncDnsReqID = pACN->GetRequestID();
 		pACN->Release();
 		return FALSE;
 	}
@@ -457,10 +483,10 @@ BOOL MAsyncSocket::Socket(int nSocketType /*=SOCK_STREAM*/,
 		int nProtocolType /*=0*/,
 		int nAddressFormat /*=PF_INET*/)
 {
-#ifdef _DEBUG
-	if (lEvent & FD_WRITE)
-		TRACE2("WARNING: calling 'socket' with FD_WRITE on for ",typeid(*this).name());
-#endif
+//#ifdef _DEBUG
+//	if (lEvent & FD_WRITE)
+//		TRACE2("WARNING: calling 'socket' with FD_WRITE on for ",typeid(*this).name());
+//#endif
 	ASSERT(m_hSocket == INVALID_SOCKET);
 
 	m_hSocket = socket(nAddressFormat,nSocketType,nProtocolType);
