@@ -21,17 +21,23 @@
 #ifndef _BYTEORDER_H_INCLUDED_
 #define _BYTEORDER_H_INCLUDED_
 
+#include "config.h"
+
 // byte order independent template classes-wrappers
 #define XOR(a,b) ( ((!(a)) && (b)) || ((a) && (!(b))) )
 
-typedef union { short s; char b[2]; } byteOrderTest;
 class MByteOrderTest {
+private:
+#ifdef WORDS_BIGENDIAN
+	static const bool bigEndian = true;
+#else
+	static const bool bigEndian = false;
+#endif 
+
 public:
 	MByteOrderTest(){};
-	bool inline IsLittleEndian() const {return 0 != ms_ByteOrderTest.b[0];}
-	bool inline IsBigEndian() const {return 0 == ms_ByteOrderTest.b[0];}
-private:
-	static byteOrderTest ms_ByteOrderTest;
+	bool inline IsLittleEndian() const {return !bigEndian;}
+	bool inline IsBigEndian() const { return bigEndian; }
 };
 
 #pragma pack(1)
@@ -61,7 +67,7 @@ public:
 	T_type data;
 protected:
 	void ByteReorder(const T_type& src, T_type& dest) const{
-		if(XOR(B_bigendian, IsBigEndian()))
+		if(XOR(B_bigendian, bigEndian))
 		{
 			for(int i=0; i<sizeof(T_type); i++)
 				((char*)(&dest))[sizeof(T_type)-i-1] = ((char*)(&src))[i];
@@ -72,7 +78,7 @@ protected:
 		}
 	}
 	void ByteReorder(T_type& value) const{
-		if(XOR(B_bigendian, IsBigEndian()))
+		if(XOR(B_bigendian, bigEndian))
 		{
 			char chTmp;
 			for(int i=0; i<sizeof(T_type)/2; i++)
@@ -83,16 +89,14 @@ protected:
 			}
 		}
 	}
-	
-	bool inline IsLittleEndian() const {return 0 != ms_ByteOrderTest.b[0];}
-	bool inline IsBigEndian() const {return 0 == ms_ByteOrderTest.b[0];}
 private:
-	static byteOrderTest ms_ByteOrderTest;
+#ifdef WORDS_BIGENDIAN
+	static const bool bigEndian = true;
+#else 
+	static const bool bigEndian = false;
+#endif
 };
 
-template<class T_type, bool B_bigendian>
-byteOrderTest TByteOrderWrapper<T_type, B_bigendian>::ms_ByteOrderTest = {1};
-
 #pragma pack()
 
 #define BE(_TYPE) TByteOrderWrapper<_TYPE, true>
