@@ -211,18 +211,57 @@ void MakeUpper(char * szStr)
 	}
 }
 
-char ToLower(char c)
+int CompareNoCase(LPCSTR sz1, LPCSTR sz2, int nChars /*=INT_MAX*/)
 {
-	if (c>='A' && c<='Z')
-		c += 'a' - 'A';
-	return c;
+	// sanity check
+	if (!sz1)
+		if (!sz2)
+			return 0;
+		else
+			return -1;
+	if (!sz2)
+		return 1;
+	//
+	register char cDiff;
+	for (int i=0; i<nChars; ++i, ++sz1, ++sz2)
+	{
+		if (!*sz1)
+			if (!*sz2)
+				return 0;
+			else
+				return -1;
+		if (!*sz2)
+			return 1;
+		cDiff = ToLower(*sz1) - ToLower(*sz2);
+		if (cDiff)
+			return (cDiff > 0) ? 1 : -1;
+	}
+	return 0;
 }
 
-char ToUpper(char c)
+int CompareNoCase(const CString& s1, const CString& s2, int nChars /*=INT_MAX*/)
 {
-	if (c>='a' && c<='z')
-		c -= 'a' - 'A';
-	return c;
+	return CompareNoCase(s1.c_str(), s2.c_str(), nChars);
+}
+
+char* SubstringNoCase(LPSTR szHaystack, LPCSTR szNeedle)
+{
+	char * szHaystackLow = strdup (szHaystack);
+	char * szNeedleLow = strdup (szNeedle);
+	char * szRetval;
+
+	MakeLower(szHaystackLow);
+	MakeLower(szNeedleLow);
+
+	szRetval = strstr (szHaystack, szNeedle);
+
+	free(szNeedleLow);
+	free(szHaystackLow);
+
+	if (szRetval == NULL)
+		return (NULL);
+
+	return szHaystack + (int) (szRetval - szHaystackLow);
 }
 
 inline bool IsWhite(char c)
@@ -922,10 +961,40 @@ CString find_header(CString Name, const CString& sHahdshake, const CString& sLow
 	return szDefault ? szDefault : "";
 }
 
+#define CONV_HEX_DIGIT(_d) \
+	if (_d>='0'&&_d<='9')\
+		_d -= '0';\
+	else if (_d>='a'&&_d<='f')\
+		_d -= 'a'-10;\
+	else if (_d>='A'&&_d<='F')\
+		_d -= 'A'-10;
 
-
-
-
-
-
+CString restore_string(CString o, bool bFormMode /*=true*/)
+{
+	int nPos=0;
+	if (bFormMode)
+	{
+		// change '+' to space
+		while ((nPos=o.find('+',nPos))>=0)
+			o[nPos++]=' ';
+	}
+	
+	// recover %xx hex values
+	nPos = 0;
+	while ((nPos=o.find('%',nPos))>=0 && nPos < o.length()-2)
+	{
+		u_char c1 = o[nPos+1],
+			   c2 = o[nPos+2];
+		if ( ((c1>='0'&&c1<='9')||(c1>='a'&&c1<='f')||(c1>='A'&&c1<='F')) &&
+			 ((c2>='0'&&c2<='9')||(c2>='a'&&c2<='f')||(c2>='A'&&c2<='F')) )
+		{
+			o = o.substr(0,nPos+1) + o.substr(nPos+3);
+			CONV_HEX_DIGIT(c1)
+			CONV_HEX_DIGIT(c2)
+			o[nPos] = (c1<<4) | c2;
+		}
+		++nPos;
+	}
+	return o;
+}
 
