@@ -42,6 +42,7 @@
 #include "mui.h"
 #include "uiterminal.h"
 #include "uiremote.h"
+#include "uilocalsocket.h"
 #include "inifile.h"
 #include "dir.h"
 #include "asyncfile.h"
@@ -49,6 +50,7 @@
 #include "conversions.h"
 #include "gnulogcentre.h"
 #include "sha1thread.h"
+#include "gnumarkedfiles.h"
 
 ///////////////////////////////////////////
 // MController handles resource persistance
@@ -57,6 +59,8 @@
 #define MUTELLARC 			"~/.mutella/mutellarc"
 #define MUTELLARC_SEARCH	"~/.mutella/search.save"
 #define MUTELLARC_HOSTS		"~/.mutella/hosts.save"
+#define MUTELLARC_KNOWNFILES "~/.mutella/knownfiles.save"
+#define MUTELLARC_WEBCACHES	"~/.mutella/webcaches.save"
 #define MUTELLARC_OLD 		"~/.mutellarc"
 #define MUTELLATERMRC 		"~/.mutella/termrc"
 #define MUTELLATERMRC_OLD 	"~/.mutella_termrc"
@@ -144,6 +148,49 @@ void MUIThread::run()
 	TRACE("Leaving MUIThread::run()...");
 }
 
+/////////////////////////////////////
+// MPrivPropertyContainer declaration
+
+class MPrivPropertyContainer : public MPropertyContainer {
+public:
+	MPrivPropertyContainer();
+	virtual bool DoMigration(LPCSTR szOldVersion, LPCSTR szCurrentVersion, MIniFile* pInifile);
+};
+
+///////////////////////////////////////
+// MPrivPropertyContainer imlementation
+
+MPrivPropertyContainer::MPrivPropertyContainer()
+{
+}
+
+bool MPrivPropertyContainer::DoMigration(LPCSTR szOldVersion, LPCSTR szCurrentVersion, MIniFile* pInifile)
+{
+	//cout << "Migrating from version `" << szOldVersion << "' to `" << szCurrentVersion << endl;
+	char tmp[1024];
+	char tmp1[1024];
+	bool bModified = false;
+	if (strcmp(szOldVersion, "0.4.4")<=0)
+	{
+		pInifile->SetSection("Bandwidth");
+		if (pInifile->ReadStr("BandwidthTransfer", tmp, NULL, true))
+		{
+			pInifile->WriteStr("BandwidthUpload", tmp);
+			pInifile->EraseKey("BandwidthTransfer");
+			bModified = true;
+		}
+		pInifile->SetSection("RemoteUI");
+		if (pInifile->ReadStr("AllowedRemoteNet", tmp, NULL, true))
+		{
+			sprintf(tmp1, "{%s}", tmp);
+			pInifile->WriteStr("AllowedRemoteNets", tmp1);
+			pInifile->EraseKey("AllowedRemoteNet");
+			bModified = true;
+		}
+	}
+	return bModified;
+}
+
 /////////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////////////
 // MControler
@@ -161,7 +208,7 @@ MController::MController()
 	m_pNetworkThread = new MGnutellaThread(this);
 	ASSERT(m_pNetworkThread);
 	//
-	m_pPropCont =  new MPropertyContainer;
+	m_pPropCont =  new MPrivPropertyContainer;
 	ASSERT(m_pPropCont);
 	//
 	m_pLogCentre = new MGnuLogCentre(this);
@@ -237,6 +284,11 @@ bool MController::GetSearchByID(DWORD dwID, SGnuSearch& gs, vector<Result>& rv,
 	return m_pDirector->GetSearchByID(dwID, gs, rv, gv);
 }
 
+int MController::GetSearchesCount()
+{
+	return m_pDirector->GetSearchesCount();
+}
+
 bool MController::GetResultsByID(DWORD dwID, vector<Result>& rv)
 {
 	return m_pDirector->GetResultsByID(dwID, rv);
@@ -355,52 +407,62 @@ void MController::GetNodeCache(list<Node>& list)
 
 void MController::LoadPartials(const CString& partDir)
 {
-    DirEntryVec partials;
-    if (!ScanDir(partials, ExpandPath(partDir), "*.[[*]]"))
-    {
-    	POST_ERROR(ES_IMPORTANT, CString("LoadPartials: error scanning the dir `") + partDir + "'");
-    	return;
-    }
-    if (!partials.size())
-    {
-    	return;
-    }
-    for (int i = 0; i<partials.size(); ++i)
-    {
-    	CString path = partials[i].Path;
-    	CString name = partials[i].Name;
-    	int nB1 = name.find(".[[");
-    	int nB2 = name.find("]]");
-    	int nSize = atoi(name.substr(nB1+3, nB2-(nB1+3)).c_str());
-    	//
-    	name = name.substr(0,nB1);
+	DirEntryVec partials;
+	if (!ScanDir(partials, ExpandPath(partDir), "*.[[*]]"))
+	{
+		POST_ERROR(ES_IMPORTANT, CString("LoadPartials: error scanning the dir `") + partDir + "'");
+		return;
+	}
+	if (!partials.size())
+		return;
+	for (int i = 0; i<partials.size(); ++i)
+	{
+		CString path = partials[i].Path;
+		CString name = partials[i].Name;
+		int nB1 = name.find(".[[");
+		int nB2 = name.find("]]");
+		int nSize = atoi(name.substr(nB1+3, nB2-(nB1+3)).c_str());
+		//
+		name = name.substr(0,nB1);
 
-        // repair stuck downloads
-        if (nSize <= partials[i].Size)
-        {
-            //BuildDownloadPaths(CString sFileName, int nFileSize, CString* psFileName, CString* psFilePath, CString* psPartPath)(CString sFileName, int nFileSize, CString* psFileName, CString* psFilePath, CString* psPartPath)
-            CString sComplPath;
-            m_pDirector->BuildDownloadPaths(name, nSize, NULL, &sComplPath, NULL);
-            if (MoveFile(path, sComplPath))
-            {
-                POST_MESSAGE(ES_GOODTOKNOW, CString("Sucessfully repaired stuck download `") + name + "'");
-            }
-            else
-            {
-                POST_ERROR(ES_IMPORTANT, CString("Failed to repair stuck download `") + name + "'");
-            }
-            continue;
-        }
-     
-    	CString search = MakeSearchOfFilename(name);
-    	//printf("%s %d\n", search.c_str(), nSize);
-    	MGnuSearch* pSearch = m_pDirector->AddSearch(search.c_str(), ST_PARTIAL, nSize, LIMIT_EXACTLY);
-    	if (pSearch)
-    	{
-    		pSearch->m_bAutoget = true;
-			pSearch->m_Filename = name;
+		// repair stuck downloads
+		if (nSize <= partials[i].Size)
+		{
+			CString sComplPath;
+			m_pDirector->BuildDownloadPaths(name, nSize, NULL, &sComplPath, NULL);
+			if (MoveFile(path, sComplPath))
+			{
+				POST_MESSAGE(ES_GOODTOKNOW, CString("Sucessfully repaired stuck download `") + name + "'");
+			}
+			else
+			{
+				POST_ERROR(ES_IMPORTANT, CString("Failed to repair stuck download `") + name + "'");
+			}
+			continue;
 		}
-    }
+     
+		CString search = MakeSearchOfFilename(name);
+		//
+		MGnuSearch* pSearch = m_pDirector->GetSearchByFile(name, nSize);
+		if (!pSearch)
+		{
+			TRACE4("LoadPartials() : creating a new auto-search because we could not find one for ", name, ", ", nSize);
+			pSearch = m_pDirector->AddSearch(search.c_str(), ST_ALTERNATIVE, nSize, LIMIT_EXACTLY);
+		}
+		if (pSearch)
+		{
+			pSearch->SetAutoget(true);
+			pSearch->SetFilename(name);
+			//pSearch->SetFileHandle(nFileHandle);
+			pSearch->SetType(ST_ALTERNATIVE);
+		}
+		// register an entry in the "known files" DB
+		#warning add a proper knownDB entry
+		m_pDirector->GetMarkedFiles()->AddFile(MFT_Partial, nSize, SHA1Hash(), search);
+	}
+	// clean up the search list -- erase all the searches which are "partial",
+	// that are for which we did not find files
+	m_pDirector->RemoveSearchesByType(ST_PARTIAL);
 }
 
 //struct SGnuSearch
@@ -409,10 +471,6 @@ void MController::LoadPartials(const CString& partDir)
 //??	// autoget support
 //??	bool    m_bAutoget;
 //??	CString m_Filename;
-//	// filters
-//	DWORD   m_MinSpeed;
-//	int     m_SpeedFilterMode;
-//	int     m_SpeedFilterValue;
 //	// size filters
 //	int     m_SizeFilterMode;
 //	int     m_SizeFilterValue;
@@ -424,23 +482,28 @@ void MController::LoadPartials(const CString& partDir)
 
 bool MController::AddSearch(const CString& sSearch, const CString& sOptions)
 {
-	// parse filters
-	DWORD   MinSpeed = 0;
-	int     SpeedFilterMode = 0;
-	int     SpeedFilterValue = 0;
 	// size filters
 	int     SizeFilterMode = 0;
 	int     SizeFilterValue = 0;
 	// autoget
 	int     Autoget = 0;
 	//
+	long    Handle = 0;
+	//
 	int n;
-	PARSE_PARAM(MinSpeed);
-	PARSE_PARAM(SpeedFilterMode);
-	PARSE_PARAM(SpeedFilterValue);
 	PARSE_PARAM(SizeFilterMode);
 	PARSE_PARAM(SizeFilterValue);
 	PARSE_PARAM(Autoget);
+	PARSE_PARAM(Handle);
+	// parse filename. if it's there we are dealing with a "partial" search
+	int nType = ST_USER;
+	CString sFilename;
+	if (0<=(n=sOptions.find("Filename:")))
+	{
+		sFilename = sOptions.substr(n+9);
+		if (!sFilename.empty())
+			nType = ST_PARTIAL;
+	}
 
 	// check for Sha1 searches and split the search line if needed
 	list<CString> listSha1;
@@ -459,17 +522,20 @@ bool MController::AddSearch(const CString& sSearch, const CString& sOptions)
 	if (sText.empty() && listSha1.empty())
 		return false;
 
-	MGnuSearch* pSearch = new MGnuSearch(m_pDirector, sText, "", ST_USER, SizeFilterValue, SizeFilterMode);
+	MGnuSearch* pSearch = new MGnuSearch(m_pDirector, sText, SHA1Hash(), nType, SizeFilterValue, SizeFilterMode);
 	ASSERT(pSearch);
-	pSearch->m_MinSpeed = MinSpeed;
-	pSearch->m_SpeedFilterMode = SpeedFilterMode;
-	pSearch->m_SpeedFilterValue = SpeedFilterValue;
-	pSearch->m_bAutoget = Autoget;
+	pSearch->SetAutoget(Autoget);
+	if (!sFilename.empty())
+		pSearch->SetFilename(sFilename);
+	if (Handle)
+		pSearch->SetFileHandle(Handle);
 	// sha1
+	SHA1Hash sha1;
 	for (list<CString>::iterator it = listSha1.begin(); it != listSha1.end(); ++it)
 	{
 		ASSERT(it->length()==32);
-		pSearch->AddSha1Hash(*it);
+		if (sha1.fromStr(*it))
+			pSearch->AddSha1Hash(sha1);
 	}
 	//
 	if (m_pDirector->AddSearch(pSearch))
@@ -537,8 +603,6 @@ void MController::LoadSearches()
 bool save_search_callback(void* pData, SGnuSearch* pSearch)
 {
 	FILE* f = (FILE*) pData;
-	if (pSearch->IsAutomatic())
-		return true;
 	// search string
 	if (!pSearch->m_Search.empty())
 		fprintf(f,"%s", pSearch->m_Search.c_str());
@@ -547,20 +611,26 @@ bool save_search_callback(void* pData, SGnuSearch* pSearch)
 	{
 		if (!pSearch->m_Search.empty())
 			fprintf(f," Urn:Sha1:");
-		fprintf(f,"Urn:Sha1:");
-		for (set<CString>::iterator it = pSearch->m_setSha1.begin(); it != pSearch->m_setSha1.end(); ++it)
+		else
+			fprintf(f,"Urn:Sha1:");
+		for (set<SHA1Hash>::iterator it = pSearch->m_setSha1.begin(); it != pSearch->m_setSha1.end(); ++it)
 		{
-			fprintf(f," %s",         it->c_str());
+			fprintf(f," %s", it->toStr().c_str());
 		}
 	}
 	fprintf(f,"\n");
 	// options
-	fprintf(f,"MinSpeed:%d  ",         pSearch->m_MinSpeed);
-	fprintf(f,"SpeedFilterMode:%d  ",  pSearch->m_SpeedFilterMode);
-	fprintf(f,"SpeedFilterValue:%d  ", pSearch->m_SpeedFilterValue);
 	fprintf(f,"SizeFilterMode:%d  ",   pSearch->m_SizeFilterMode);
 	fprintf(f,"SizeFilterValue:%d  ",  pSearch->m_SizeFilterValue);
 	fprintf(f,"Autoget:%d  ",          pSearch->m_bAutoget ? 1 : 0);
+	// save filename for automatic searches
+	if (pSearch->IsAutomatic())
+	{
+		if (pSearch->m_FileHandle)
+			fprintf(f,"Handle:%d  ",pSearch->m_FileHandle);
+		if (pSearch->m_Filename.size())
+			fprintf(f,"Filename:%s  ",   pSearch->m_Filename.c_str());
+	}
 	fprintf(f,"\n");
 	// empty line
 	fprintf(f,"\n");
@@ -625,14 +695,21 @@ bool MController::OnStopUI(MUI* pUI) // called by the UI thread before its termi
 	return m_listStoppedUI.end()!=find(m_listStoppedUI.begin(),m_listStoppedUI.end(),pUI); // it was already in the stopped list -- fine
 }
 
-bool MController::Init()
+bool MController::Init(CString sConfigFile, bool bStartInteractive)
 {
 	// init property environment
 	m_pPropCont->Transfer(m_pPrefs->m_pContainer);
 	ASSERT(m_pPrefs->m_pContainer->Count()==0);
 	// create UIs
-	AddUI(new MUITerminal());
+	if (bStartInteractive)
+		AddUI(new MUITerminal());
 	AddUI(new MUIRemote());
+	AddUI(new MUILocalSocket());
+	//
+	if (!sConfigFile.empty())
+		m_sConfigFile = ExpandPath(sConfigFile);
+	else
+		m_sConfigFile = ExpandPath(MUTELLARC);
 	//
 	// now we are about to read the configuration
 	// so lets check if we have '.mutella' dir
@@ -658,7 +735,7 @@ bool MController::Init()
 				MProperty* pP = m_pPropCont->FindProperty("StartupScript");
 				if (pP && 0==strcmp(MUTELLATERMRC_OLD, pP->GetStringValue()))
 					pP->SetStringValue(MUTELLATERMRC);
-				m_pPropCont->Write(MUTELLARC);
+				m_pPropCont->Write(m_sConfigFile.c_str());
 			}
 			if (FileExists(MUTELLATERMRC_OLD))
 				MoveFile(MUTELLATERMRC_OLD,MUTELLATERMRC);
@@ -667,7 +744,7 @@ bool MController::Init()
 	}
 
 	// read config
-	m_pPropCont->Read(MUTELLARC);
+	m_pPropCont->Read(m_sConfigFile.c_str(), VERSION);
 	// let UIs initialise and load their private settings
 	m_mutex.lock();
 	for (list<SUIEntry>::iterator it = m_listUI.begin();it != m_listUI.end(); ++it)
@@ -687,18 +764,25 @@ bool MController::Init()
 		POST_MESSAGE(ES_IMPORTANT, CString("Directory \'") + PartialDir + "\' does not exist and will be created");
 		CreateDirectory(PartialDir);
 	}
-	//
-	LoadPartials(PartialDir);
+
+	// load marked files database
+	m_pDirector->GetMarkedFiles()->LoadList(MUTELLARC_KNOWNFILES);
 
 	//
 	if (m_pPrefs->m_bSaveSearches)
 		LoadSearches();
 
+	// load partial files (re-generate auto-gets)
+	LoadPartials(PartialDir);
+
 	// Start sharing and load files
 	m_pDirector->GetShare()->InitShare();
 
 	// load the 'good' host list
 	m_pDirector->GetHostStore()->LoadNodes(MUTELLARC_HOSTS);
+	
+	// load web caches
+	m_pDirector->LoadWebCaches(MUTELLARC_WEBCACHES);
 
 	//
 	m_pDirector->StartListening();
@@ -810,6 +894,7 @@ void MController::Do()
 	m_pDirector->GetShare()->CloseShare();
 
 	// stop AsyncDns thread
+	TRACE("MController::Do: stopping the AsyncDNS thread...");
 	MAsyncDnsThread::StopThread();
 
 	// wait for the network thread to stop
@@ -834,13 +919,17 @@ void MController::SaveState()
 {
 	// we are closing, save props
 	//TRACE("MController::SaveState()...");
-	if (!m_pPropCont->Write(MUTELLARC))
+	if (!m_pPropCont->Write(m_sConfigFile.c_str(), VERSION))
 		POST_ERROR(ES_IMPORTANT, "Failed to save settings");
 	// save searches
 	if (m_pPrefs->m_bSaveSearches)
 		SaveSearches();
 	// store the 'good' host list
 	m_pDirector->GetHostStore()->SaveNodes(MUTELLARC_HOSTS);
+	// store known files database
+	m_pDirector->GetMarkedFiles()->StoreList(MUTELLARC_KNOWNFILES);
+	
+	m_pDirector->SaveWebCaches(MUTELLARC_WEBCACHES);
 }
 
 void MController::ForceStop()
