@@ -49,7 +49,7 @@ public:
 	MController();
 	~MController();
 	// called from main()
-	bool Init();
+	bool Init(CString sConfigFile, bool bStartInteractive);
 	void Do();
 	// interface to the network core
 	MGnuPreferences* GetPrefs();
@@ -78,6 +78,7 @@ public:
 	bool ModifySearch(DWORD dwID, LPCSTR szSearch);
 	void ForEachSearch(void* pData, tEachSearch callback);
 	bool GetSearchByID(DWORD dwID, SGnuSearch& gs, ResultVec& rv, ResultGroupVec& gv); /* C: problem */
+	int  GetSearchesCount();
 	bool GetResultsByID(DWORD dwID, ResultVec& rv); /* C: problem */
 	bool RemoveSearchByID(DWORD dwID, bool bDeletePartial);
 	bool ClearSearchByID(DWORD dwID);
@@ -118,6 +119,7 @@ protected:
 	MWaitCondition       m_waitForChanges;
 	MMutex               m_mutex;
 	bool                 m_bExplisitStop;
+	CString              m_sConfigFile;
 	// protected methods
 	void LoadPartials(const CString& partDir);
 	void LoadSearches();
