@@ -58,8 +58,8 @@ bool Str2Ip(const CString& sIn, IP& ip)
 {
 	CString s = StripWhite(sIn);
 	struct in_addr addr;
-	addr.s_addr == INADDR_NONE;
-	#warning "we also use g_LibcMutex here"
+	addr.s_addr = INADDR_NONE;
+	// we lock mutex here for any case
 	MLock lock(g_LibcMutex);
 	addr.s_addr = inet_addr(s.c_str());
 	if (addr.s_addr != INADDR_NONE)
@@ -75,19 +75,20 @@ bool Str2Ip_lookup(const CString& sIn, IP& ip)
 {
 	CString s = StripWhite(sIn);
 	struct in_addr addr;
-	#warning "we temporarily place g_LibcMutex where I dont want to have it"
-	MLock lock(g_LibcMutex);
+	// we lock mutex here for any case
+	MLock lockG(g_LibcMutex);
 	addr.s_addr = inet_addr(s.c_str());
 	if (addr.s_addr != INADDR_NONE)
 	{
 		ip.S_addr = addr.s_addr;
 		return true;
 	}
+	lockG.unlock();
 #ifdef _DEBUG
 	time_t t;
 #endif
 	IP out;
-	//MLock lock(g_GetHostByNameMutex);
+	MLock lock(g_GetHostByNameMutex);
 	out.S_addr = 0;
 #ifdef _DEBUG
 	time(&t);
@@ -111,7 +112,7 @@ bool Str2Ip_lookup(const CString& sIn, IP& ip)
 CString IpNet2Str(const IPNET& ipn)
 {
 	char buffer[48];
-	::sprintf(buffer,"%d.%d.%d.%d/%d",ipn.S_ip.a,ipn.S_ip.b,ipn.S_ip.c,ipn.S_ip.d,ipn.net_bits);
+	::sprintf(buffer,"%d.%d.%d.%d/%ld",ipn.S_ip.a,ipn.S_ip.b,ipn.S_ip.c,ipn.S_ip.d,ipn.net_bits);
 	return buffer;
 }
 
@@ -238,6 +239,7 @@ CString GetVendor(CString VendorID)
 	if(VendorID == "LIME") return "LimeWire";
 	if(VendorID == "MACT") return "Mactella";
 	if(VendorID == "MRPH") return "Morpheus";
+	if(VendorID == "MMMM") return "Morpheus2";
 	if(VendorID == "MUTE") return "Mutella";
 	if(VendorID == "NAPS") return "NapShare";
 	if(VendorID == "OCFG") return "OpenCola";
@@ -266,8 +268,6 @@ static hostent *internal_fakehostent;
 /*MZ: __private_extern__*/ struct hostent *
 fake_hostent(const char *name, struct in_addr addr)
 {
-	int addr_len;
-
 	if (name == NULL) return NULL;
 
 	// If not created, calloc our internal hostent.
@@ -323,15 +323,17 @@ hostent *GetHostByName(const char *host)
 	{
 		// MZ: inet_aton does not link on Solaris
 		struct in_addr addr;
-		#warning "we also use g_LibcMutex here"
-		MLock lock(g_LibcMutex);
+		// we use g_LibcMutex here just in case...
+		g_LibcMutex.lock();
 		addr.s_addr = inet_addr(host);
 		if (addr.s_addr == INADDR_NONE)
 		{
 			h_errno = NO_DATA;
 			res = NULL;
 		}
-		else res = fake_hostent(host, addr);
+		else
+			res = fake_hostent(host, addr);
+		g_LibcMutex.unlock();
 	}
 	else
 	{
@@ -468,7 +470,7 @@ CString Base64::decode(CString data)
  *         7 H            16 Q            25 Z
  *         8 I            17 R            26 2
  */
-static char values[256] = {
+static signed char values[256] = {
 /*  0  1  2  3  4  5  6  7  8  9  */	/* 0123456789              */
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,		/*            -  00 ->  09 */
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,		/*            -  10 ->  19 */
@@ -507,7 +509,7 @@ static char *b32_alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
  * Older base32 alphabet: "ABCDEFGHIJK MN PQRSTUVWXYZ  23456789"
  * We decode it only.
  */
-static char old_values[256] = {
+static signed char old_values[256] = {
 /*  0  1  2  3  4  5  6  7  8  9  */	/* 0123456789              */
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,		/*            -  00 ->  09 */
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,		/*            -  10 ->  19 */
@@ -714,14 +716,14 @@ char *base32_encode(const BYTE *buf, int len, int *retpad)
  *
  * Return TRUE if successful, FALSE if the input was not valid base32.
  */
-static bool base32_decode_alphabet(char valmap[256],
+static bool base32_decode_alphabet(signed char valmap[256],
 	const char *buf, int len, char *decbuf, int declen)
 {
 	long i = 0;					/* Input accumulator, 0 for trailing pad */
 	char const *ip = buf + len;	/* Input pointer, one byte off end */
 	int dlen = (len >> 3) * 5;		/* Exact decoded lenth */
 	char *op;						/* Output pointer, one byte off end */
-	char v;
+	signed char v;
 
 	ASSERT(buf);
 	ASSERT(decbuf);
@@ -848,9 +850,9 @@ static bool base32_decode_alphabet(char valmap[256],
  * Return TRUE if successful, FALSE if the input was not valid base32.
  */
 bool base32_decode_into(const char *buf, int len,
-	char *decbuf, int declen)
+	BYTE *decbuf, int declen)
 {
-	return base32_decode_alphabet(values, buf, len, decbuf, declen);
+	return base32_decode_alphabet(values, buf, len, (char*) decbuf, declen);
 }
 
 /*
@@ -888,7 +890,7 @@ char *base32_decode(const char *buf, int len)
 	declen = (len >> 3) * 5;
 	decbuf = (char*) malloc(declen);
 
-	if (!base32_decode_into(buf, len, decbuf, declen)) {
+	if (!base32_decode_into(buf, len, (BYTE*) decbuf, declen)) {
 		free(decbuf);
 		return NULL;
 	}
@@ -896,6 +898,3 @@ char *base32_decode(const char *buf, int len)
 	return decbuf;
 }
 
-
-
-
