@@ -28,11 +28,11 @@
 #define FALSE false
 #endif
 
-#define BOOL  bool
-#define BYTE  u_char
-#define UINT  u_int
-#define WORD  u_short
-#define DWORD u_long
+typedef bool           BOOL;
+typedef unsigned char  BYTE;
+typedef unsigned int   UINT;
+typedef unsigned short WORD;
+typedef unsigned long  DWORD;
 
 typedef char * LPSTR;
 typedef const char * LPCSTR;
@@ -45,7 +45,7 @@ typedef const char * LPCSTR;
 
 typedef void* LPVOID;
 
-#define HANDLE DWORD
+typedef DWORD HANDLE;
 typedef HANDLE* LPHANDLE;
 
 #ifndef MIN
@@ -66,6 +66,7 @@ using std::endl;
 // debugging macros
 #ifdef _DEBUG
 
+#include <assert.h>
 #define ASSERT(cond) assert(cond)
 #define VERIFY(cond) assert(cond)
 #define TRACE1(message) cout << message << endl << flush
