@@ -116,7 +116,7 @@ bool MEventDispatcher::SendEvent(MEvent* pEvent)
 	{
 		// we still have plenty of references
 		pEvent->Release();
-		en.wc.wait(&en.mutex);
+		while (!en.wc.wait(&en.mutex)) {}
 		en.mutex.unlock();
 	}
 	else
@@ -275,7 +275,10 @@ TSmartPtr<MEvent> MAsyncEventReceiver::SafeFront()
 {
 	MLock lock(m_mutex, m_hThreadLock!=MThread::currentThread());
 	if (m_queue.size())
+	{
+		ASSERT(m_queue.front());
 		return m_queue.front();
+	}
 	return NULL;
 }
 
