@@ -50,6 +50,7 @@
 #include "preferences.h"
 #include "asyncfile.h"
 #include "asyncdns.h"
+#include "gnumarkedfiles.h"
 
 /////////////////////////////////////////////////////////////////////////////
 // MGnuDirector
@@ -63,12 +64,15 @@ MGnuDirector::MGnuDirector(MController* pController) : m_listmutex(true) // resu
 	m_pHostCatcher  = new MGnuCache(this);
 	m_pUltraCatcher = new MGnuCache(this);
 	m_pHostStore    = new MGnuCache(this);
+
+	m_pKnownFiles = new MGnuMarkedFiles();
+	
 	m_pShare = NULL;
 	VERIFY(new MGnuShare(this));	
 
 	CreateGuid(&m_ClientID);
-	m_nSecsAboveLimitConn = 0;
 	m_dCurrentReceiveLimit = 0;
+	m_dDownloadRateLimit = 0;
 	m_dwReSearchID = 0;
 	m_nReSearchTimer = 0;
 
@@ -100,6 +104,9 @@ MGnuDirector::MGnuDirector(MController* pController) : m_listmutex(true) // resu
 	m_nUltraPongBytes = 0;
 	m_nStartTime = xtime();
 	m_bReceivedIncomingConnections = false;
+	
+	m_anInitialWebCacheRequests[0] = m_anInitialWebCacheRequests[1] = m_anInitialWebCacheRequests[2] = 0;
+	m_bNoValidWebCacheUrlsMessageSent = false;
 
 	m_pAsyncDNS = NULL;
 }
@@ -154,6 +161,9 @@ MGnuDirector::~MGnuDirector()
 	if (m_pWordHashTable)
 		delete m_pWordHashTable;
 
+	if (m_pKnownFiles)
+		delete m_pKnownFiles;
+
 	if (m_pAsyncDNS)
 		m_pAsyncDNS->Release();
 	
@@ -412,8 +422,9 @@ void MGnuDirector::Broadcast_Ping(packet_Ping* Ping, int length, MGnuNode* excep
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 	{
 		pNode = itNode->second;
+		ASSERT(pNode);
 		if( m_nHostMode == CM_ULTRAPEER     &&
-			pNode->GetPeerMode() == CM_LEAF )
+			pNode->GetPeerMode() == CM_LEAF ) // it could have crashed here
 				continue;
 		if( pNode != exception   &&
 			pNode->IsConnected() )
@@ -449,8 +460,9 @@ void MGnuDirector::Broadcast_Query(packet_Query* Query, int length, MGnuNode* ex
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 	{
 		pNode = itNode->second;
+		ASSERT(pNode);
 		if( m_nHostMode == CM_ULTRAPEER     &&
-			pNode->GetPeerMode() == CM_LEAF )
+			pNode->GetPeerMode() == CM_LEAF ) // it could have crashed here
 				continue;
 		if( pNode != exception   &&
 			pNode->IsConnected() )
@@ -458,7 +470,7 @@ void MGnuDirector::Broadcast_Query(packet_Query* Query, int length, MGnuNode* ex
 	}
 }
 
-GUID MGnuDirector::Broadcast_LocalQuery(BYTE* Packet, WORD Speed, int length)
+GUID MGnuDirector::Broadcast_LocalQuery(BYTE* Packet, int length)
 {
 	GUID Guid = GUID_NULL;
 	CreateGuid(&Guid);
@@ -475,7 +487,7 @@ GUID MGnuDirector::Broadcast_LocalQuery(BYTE* Packet, WORD Speed, int length)
 	Query->Header.Hops = 0;
 	Query->Header.TTL = MAX_TTL_SELF;
 	Query->Header.le_Payload = length - 23;
-	Query->le_Speed = (WORD) Speed;
+	Query->le_Speed = 0;
 
 	m_TableLocal.Insert(&Guid, 0ul);
 
@@ -714,13 +726,7 @@ void MGnuDirector::RemoveNode(MGnuNode* pNode)
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 		if(itNode->second == pNode)
 		{
-			// Make sure socket is closed
-			if(pNode->m_hSocket != INVALID_SOCKET)
-			{
-				pNode->AsyncSelect(FD_CLOSE);
-				pNode->ShutDown(2);
-			}
-			pNode->Close();
+			pNode->ForceDisconnect();
 			m_NodeMap.erase(itNode);
 			delete pNode;
 			return;
@@ -735,12 +741,7 @@ void MGnuDirector::RemoveNode(DWORD dwID)
 	if (itNode == m_NodeMap.end())
 		return;
 	// Make sure socket is closed
-	if(itNode->second->m_hSocket != INVALID_SOCKET)
-	{
-		itNode->second->AsyncSelect(FD_CLOSE);
-		itNode->second->ShutDown(2);
-	}
-	itNode->second->Close();
+	itNode->second->ForceDisconnect();
 	m_NodeMap.erase(itNode);
 	delete itNode->second;
 }
@@ -892,7 +893,6 @@ MGnuDownload* MGnuDirector::LookUpDownload(IP ipHost, CString sFileName, int nIn
 {
 	MLock lock(m_listmutex);
 	//
-	sFileName.make_lower();
 	for(int i = 0; i < m_DownloadList.size(); i++)
 	{
 		MGnuDownload *p = m_DownloadList[i];
@@ -946,6 +946,9 @@ bool MGnuDirector::AddDownload(const ResultVec& results)
 {
 	if (!results.size())
 		return false;
+	// check whether we are downloading any of those already and
+	// at the same time get the shortest filename
+	int nShortestNameIndex = 0;
 	for (int i=0; i<results.size();++i)
 	{
 		// check if we already downloading this
@@ -953,11 +956,25 @@ bool MGnuDirector::AddDownload(const ResultVec& results)
 		{
 			return false; // todo: something more appropriate
 		}
+		if (results[i].Name.length()<results[nShortestNameIndex].Name.length())
+			nShortestNameIndex = i;
 	}
-	MGnuDownload* pD =  new MGnuDownload(this, results);
+	// get free file handle 
+	long nFileHandle = m_pKnownFiles->GetFreeFileHandle(results[0].Size);
+	// create the download object
+	MGnuDownload* pD =  new MGnuDownload(this, results, results[nShortestNameIndex].Name, nFileHandle);
 	ASSERT(pD);
+	// add the file to the known files DB
+	CString sCommonSearch = MakeSearchOfFilename(results[nShortestNameIndex].Name);
+	for (int i=0; i<results.size();++i)
+	{
+		CString s = MakeSearchOfFilename(results[i].Name);
+		m_pKnownFiles->AddFile(MFT_Attempted, results[i].Size, results[i].Sha1, s, nFileHandle);
+		if (s != sCommonSearch)
+			m_pKnownFiles->AddFile(MFT_Attempted, results[i].Size, results[i].Sha1, sCommonSearch, nFileHandle);
+	}
 	// add to download queue
-	AddAndStartDownload(pD);
+	AddAndStartDownload(pD);	
 	return true;
 }
 
@@ -991,9 +1008,23 @@ MGnuSearch* MGnuDirector::LookUpSearch(const CString& search, int size)
 	MLock lock(m_listmutex);
 	map<DWORD, MGnuSearch*>::iterator itS;
 	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
-		if ( LIMIT_EXACTLY == itS->second->m_SizeFilterMode &&
-		     size == itS->second->m_SizeFilterValue         &&
-		     search == itS->second->m_Search                )
+		if ( LIMIT_EXACTLY == itS->second->GetSizeFilterMode() &&
+		     size == itS->second->GetSizeFilterValue()         &&
+		     search == itS->second->GetSearchString()          )
+		{
+			return itS->second;
+		}
+	return NULL;
+}
+
+MGnuSearch* MGnuDirector::GetSearchByFile(const CString& name, int size)
+{
+	MLock lock(m_listmutex);
+	map<DWORD, MGnuSearch*>::iterator itS;
+	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
+		if ( LIMIT_EXACTLY == itS->second->GetSizeFilterMode() &&
+		     size == itS->second->GetSizeFilterValue()         &&
+		     name == itS->second->GetFilename()                )
 		{
 			return itS->second;
 		}
@@ -1011,39 +1042,41 @@ MGnuSearch* MGnuDirector::AddSearch(LPCSTR szSearch, int type, int size, int siz
 		0 == memcmp(szSearch, "sha1:", 4))
 	{
 		// we seem to get SHA1 search
-		// TODO: check for Base32 decoding of the hash
-		CString sSha1 = szSearch+5;
-		map<DWORD, MGnuSearch*>::iterator itS;
-		for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
+		SHA1Hash sha1;
+		// check for Base32 decoding of the hash
+		if (sha1.fromStr(szSearch+5))
 		{
-			MLock lock(itS->second->m_mutex);
-			if ( itS->second->m_setSha1.find(sSha1) != itS->second->m_setSha1.end() )
-				return NULL;
+			map<DWORD, MGnuSearch*>::iterator itS;
+			for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
+			{
+				if ( itS->second->IsInList(sha1) )
+					return NULL;
+			}
+			pS = new MGnuSearch(this, "", sha1, type, size, sizeMode);
 		}
-		pS = new MGnuSearch(this, "", sSha1, type, size, sizeMode);
 	}
 	else
 	{
 		// ordinary string search
 		map<DWORD, MGnuSearch*>::iterator itS;
 		for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
-			if ( sizeMode == itS->second->m_SizeFilterMode   &&
-				 size == itS->second->m_SizeFilterValue      &&
-				 QueryMatch(szSearch, itS->second->m_Search) &&
-				 QueryMatch(itS->second->m_Search, szSearch) )
+			if ( sizeMode == itS->second->GetSizeFilterMode()         &&
+				 size == itS->second->GetSizeFilterValue()            &&
+				 QueryMatch(szSearch, itS->second->GetSearchString()) &&
+				 QueryMatch(itS->second->GetSearchString(), szSearch) )
 				 return NULL; // mutualy matching searches are redundant
 		 // TODO: above check conflicts slightly with exclusive searches -- fix it
 
-		pS = new MGnuSearch(this, szSearch, "", type, size, sizeMode);
+		pS = new MGnuSearch(this, szSearch, SHA1Hash(), type, size, sizeMode);
 	}
 	ASSERT(pS);
-	pS->m_bAutoget = bAutoGet;
+	pS->SetAutoget(bAutoGet);
 	// add to the list
-	pS->m_dwID = GenID();
-	ASSERT(m_SearchMap.find(pS->m_dwID) == m_SearchMap.end());
-	m_SearchMap[pS->m_dwID] = pS;
+	pS->SetID(GenID());
+	ASSERT(m_SearchMap.find(pS->GetID()) == m_SearchMap.end());
+	m_SearchMap[pS->GetID()] = pS;
 	// queue sends and do them on timer
-	m_PendingQueries.push_back(pS->m_dwID);
+	m_PendingQueries.push_back(pS->GetID());
 	return pS;
 }
 
@@ -1054,11 +1087,11 @@ bool MGnuDirector::AddSearch(MGnuSearch* pSearch)
 	if (m_SearchMap.size()>=m_pPrefs->m_nMaxSearches)
 		return false;
 	// add to the list
-	pSearch->m_dwID = GenID();
-	ASSERT(m_SearchMap.find(pSearch->m_dwID) == m_SearchMap.end());
-	m_SearchMap[pSearch->m_dwID] = pSearch;
+	pSearch->SetID(GenID());
+	ASSERT(m_SearchMap.find(pSearch->GetID()) == m_SearchMap.end());
+	m_SearchMap[pSearch->GetID()] = pSearch;
 	// queue sends and do them on timer
-	m_PendingQueries.push_back(pSearch->m_dwID);
+	m_PendingQueries.push_back(pSearch->GetID());
 	return true;
 }
 
@@ -1072,7 +1105,7 @@ bool MGnuDirector::ModifySearch(DWORD dwID, LPCSTR szSearch)
 	map<DWORD, MGnuSearch*>::iterator itS = m_SearchMap.find(dwID);
 	if (itS != m_SearchMap.end())
 	{
-		ASSERT(dwID == itS->second->m_dwID);
+		ASSERT(dwID == itS->second->GetID());
 		// so now we have the search, let's modify it!
 		itS->second->SetSearchString(szSearch);
 		// TODO: check if this may produce some interference with Download
@@ -1090,12 +1123,18 @@ void MGnuDirector::ForEachSearch(void* pData, tEachSearch callback)
 	map<DWORD, MGnuSearch*>::iterator itS;
 	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
 	{
-		gs = *itS->second;
+		itS->second->GetSGnuSearch(gs);
 		if (!(*callback)(pData, &gs))
 			return;
 	}
 }
 
+int MGnuDirector::GetSearchesCount()
+{
+	MLock lock(m_listmutex);
+	return m_SearchMap.size();
+}
+
 MGnuSearch* MGnuDirector::GetSearchByID(DWORD dwID)
 {
 	if (dwID == 0)
@@ -1121,6 +1160,8 @@ bool MGnuDirector::GetSearchByID(DWORD dwID, SGnuSearch& gs, vector<Result>& rv,
 	if (itS != m_SearchMap.end())
 	{
 		MGnuSearch* pSearch = itS->second;
+		pSearch->GetAllResults(&gs, &rv, &gv);
+		/*
 		pSearch->m_mutex.lock();
 		gs = *pSearch;
 		// now it's time to package results into a vector and create a map if IDs to indeces
@@ -1145,6 +1186,7 @@ bool MGnuDirector::GetSearchByID(DWORD dwID, SGnuSearch& gs, vector<Result>& rv,
 		}
 		//
 		pSearch->m_mutex.unlock();
+		*/
 		m_listmutex.unlock();
 		return true;
 	}
@@ -1161,7 +1203,39 @@ bool MGnuDirector::GetResultsByID(DWORD dwID, vector<Result>& rv)
 	map<DWORD, MGnuSearch*>::iterator itS;
 	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
 	{
+		vector<Result> rvl;
+		vector<ResultGroup> gvl;
 		MGnuSearch* pSearch = itS->second;
+		pSearch->GetAllResults(NULL, &rvl, &gvl);
+		for (vector<ResultGroup>::iterator itg = gvl.begin() ; itg!=gvl.end(); ++itg)
+		{
+			if ( (*itg).dwID == dwID )
+			{
+				if ( 0 == (*itg).ResultSet.size() )
+				{
+					// should never happen
+					m_listmutex.unlock();
+					return false;
+				}
+				rv.reserve((*itg).ResultSet.size());
+				for (set<int>::iterator iti = (*itg).ResultSet.begin(); iti!=(*itg).ResultSet.end(); ++iti)
+					rv.push_back(rvl[*iti]);
+				//
+				m_listmutex.unlock();
+				return true;
+			}
+		}
+		for (vector<Result>::iterator itr = rvl.begin() ; itr!=rvl.end(); ++itr)
+		{
+			if ( (*itr).dwID == dwID )
+			{
+				rv.push_back(*itr);
+				//
+				m_listmutex.unlock();
+				return true;
+			}
+		}
+		/*
 		pSearch->m_mutex.lock();
 		for (list<ResultGroup>::iterator itg = pSearch->m_listGroups.begin() ; itg!=pSearch->m_listGroups.end(); ++itg)
 		{
@@ -1194,6 +1268,7 @@ bool MGnuDirector::GetResultsByID(DWORD dwID, vector<Result>& rv)
 			return true;
 		}
 		pSearch->m_mutex.unlock();
+		*/
 	}
 	m_listmutex.unlock();
 	return false;
@@ -1227,11 +1302,11 @@ bool MGnuDirector::RemoveSearchByID(DWORD dwID, bool bRemovePartial)
 		m_SearchMap.erase(itS);
 		m_listmutex.unlock();
 		if (bRemovePartial &&
-			pSearch->m_Filename.length() &&
-			pSearch->m_SizeFilterMode == LIMIT_EXACTLY )
+			pSearch->GetFilename().length() &&
+			pSearch->GetSizeFilterMode() == LIMIT_EXACTLY )
 		{
 			CString sPartPath;
-			BuildDownloadPaths(pSearch->m_Filename, pSearch->m_SizeFilterValue, NULL, NULL, &sPartPath);
+			BuildDownloadPaths(pSearch->GetFilename(), pSearch->GetSizeFilterValue(), NULL, NULL, &sPartPath);
 			if (!IsDownloading(sPartPath))
 			{
 				// non-blocking file deletion
@@ -1250,6 +1325,20 @@ bool MGnuDirector::RemoveSearchByID(DWORD dwID, bool bRemovePartial)
 	return false;
 }
 
+void MGnuDirector::RemoveSearchesByType(int nType)
+{
+	m_listmutex.lock();
+	map<DWORD, MGnuSearch*>::iterator itS;
+	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); )
+	{
+		if (itS->second->GetType()==nType)
+			m_SearchMap.erase(itS++);
+		else
+			++itS;
+	}
+	m_listmutex.unlock();
+}
+
 bool MGnuDirector::IsDownloading(const CString& sPartPath)
 {
 	MLock lock(m_listmutex);
@@ -1271,11 +1360,12 @@ void MGnuDirector::BuildDownloadPaths(CString sFileName, int nFileSize, CString*
 	ReplaceSubStr(sFileName,"\"", " ");
 	// remove path
 	sFileName = sFileName.substr( sFileName.rfind("/") + 1);
+	// makes life much easier
+	MakeLower(sFileName);
 	//
 	if (psFileName)
 		*psFileName = sFileName;
-	// makes life much easier
-	MakeLower(sFileName);
+	
 	// Set where the file will go upon completion
 	if (psFilePath)
 	{
@@ -1373,6 +1463,118 @@ bool MGnuDirector::RemoveTransferByID(DWORD dwID, bool bDelPart)
 	return false;
 }
 
+
+// original decode
+static CString
+GGEP_decode(BYTE *Packet, int Length)
+{
+	char Flags;
+	char ID[16], Buffer[Length + 256];
+	int IDLen, BufferLen = 0, i = 0, j;
+
+	BufferLen = sprintf(Buffer, "GGEP Block: ");
+	while (i < Length)
+	{
+		Flags = Packet[i++];
+		IDLen = Flags & 0x0f;
+		for (j = 0; j < IDLen && i < Length; ++j)
+			ID[j] = Packet[i++];
+		ID[j] = '\0';
+		BufferLen += sprintf(&Buffer[BufferLen], "ID='%s' ", ID);
+
+		BYTE Byte;
+		int Amount = 0;
+		for (j = 0; j < 3 && i < Length; ++j)
+		{
+			Byte = Packet[i++];
+			Amount |= Byte & 0x3f;
+			if (Byte & 0x40)
+				break;
+			Amount <<= 6;
+		}
+
+		if (Flags & 0x20)
+			BufferLen += sprintf(&Buffer[BufferLen], "DATA=<compressed (not yet supported)> ");
+		else
+		{
+			if (Flags & 0x40)
+				BufferLen += sprintf(&Buffer[BufferLen], "DATA=<COBS encoded (not yet supported)> ");
+			else
+			{
+				if (IDLen == 1 && ID[0] == 'H')
+					BufferLen += sprintf(&Buffer[BufferLen], "DATA=<sha1 moved>ggep:sha1:%s", &Packet[i]);
+				else
+					BufferLen += sprintf(&Buffer[BufferLen], "DATA=<discarded (unknown extension)> ");
+			}
+		}
+		i += Amount;
+
+		if (Flags & 0x80)
+			break;
+	}
+	Buffer[BufferLen++] = '\0';
+	return CString(Buffer);
+}
+
+// modified by MZ /* crashes */
+/*
+static CString
+GGEP_decode(BYTE *Packet, int Length)
+{
+	char Flags;
+	char ID[16];
+	CString sBuffer, sTmp;
+	sBuffer.reserve(Length + 256); // why 256?
+	int IDLen, i = 0, j;
+
+	sBuffer = "GGEP Block: ";
+	while (i < Length)
+	{
+		Flags = Packet[i++];
+		IDLen = Flags & 0x0f;
+		for (j = 0; j < IDLen && i < Length; ++j)
+			ID[j] = Packet[i++];
+		ID[j] = '\0';
+		sTmp.format("ID='%s' ", ID);
+		sBuffer += sTmp;
+
+		BYTE Byte;
+		int Amount = 0;
+		for (j = 0; j < 3 && i < Length; ++j)
+		{
+			Byte = Packet[i++];
+			Amount |= Byte & 0x3f;
+			if (Byte & 0x40)
+				break;
+			Amount <<= 6;
+		}
+
+		if (Flags & 0x20)
+			sBuffer += "DATA=<compressed (not yet supported)> ";
+		else
+		{
+			if (Flags & 0x40)
+				sBuffer += "DATA=<COBS encoded (not yet supported)> ";
+			else
+			{
+				if (IDLen == 1 && ID[0] == 'H')
+				{
+					sTmp.format("DATA=<sha1 moved>ggep:sha1:%s", &Packet[i]);
+					sBuffer += sTmp;
+				}
+				else
+					sBuffer += "DATA=<discarded (unknown extension)> ";
+			}
+		}
+		i += Amount;
+
+		if (Flags & 0x80)
+			break;
+	}
+	return sBuffer;
+}*/
+
+
 void MGnuDirector::OnQueryHit(packet_QueryHit* QueryHit, MGnuNode* pOrigin, bool bOur)
 {
 	if (!bOur && m_pPrefs->m_bStrictSearch)
@@ -1471,18 +1673,48 @@ void MGnuDirector::OnQueryHit(packet_QueryHit* QueryHit, MGnuNode* pOrigin, bool
 			for(i = NextPos + 8; Packet[i] != '\0' && i < Length - 16; ++i);
 
 			Item.Name = CString(((char*)Packet)+NextPos+8, 0, i-NextPos-8);
-			if ( i<Length-16 && Packet[i+1] != '\0')
+			for (int idx = 0; Packet[i+1] != '\0' && i<Length-16; idx++)
 			{
 				// we have extra info at the hit tail
-				Item.Extra = CString(((char*)Packet)+i+1, 0, Length-17-i);
-				for( ; Packet[i+1] != '\0' && i < Length - 16; ++i);
-				// try to get Sha1 out of Extra
-				int nShaPos = Item.Extra.find("urn:sha1:");
-				if (nShaPos >= 0)
+				if (Packet[i+1] == 0xC3)
 				{
-					Item.Sha1 = Item.Extra.substr(nShaPos+9, 32);
-					Item.Extra = StripWhite(Item.Extra.substr(0, nShaPos) + Item.Extra.substr(nShaPos+42));
+					// GGEP extension...
+					int j = i;
+					for( ; Packet[j+1] != '\0' && j < Length - 16; ++j);
+					Item.Extra.push_back(GGEP_decode(&Packet[i+2], j-i-1));
+					i = j;
+					// try to get Sha1 out of Extra
+					int nShaPos = Item.Extra[idx].find("ggep:sha1:");
+					if (nShaPos >= 0)
+					{
+						Item.Sha1.fromStr(Item.Extra[idx].substr(nShaPos+10, 32));
+						Item.Extra[idx] = StripWhite(Item.Extra[idx].substr(0, nShaPos) + Item.Extra[idx].substr(nShaPos+43));
+					}
 				}
+				else
+				{
+					// HUGE or plain text extension...
+					int j = i;
+					bool NonPrintable = false;
+					for( ; Packet[j+1] != 0x1C && Packet[j+1] != '\0' && j < Length - 16; ++j)
+					{
+						if (!isprint(Packet[j+1]))
+							NonPrintable = true;
+					}
+					Item.Extra.push_back(CString(((char*)Packet)+i+1, 0, j-i));
+					i = j;
+					// try to get Sha1 out of Extra
+					int nShaPos = Item.Extra[idx].find("urn:sha1:");
+					if (nShaPos >= 0)
+					{
+						Item.Sha1.fromStr(Item.Extra[idx].substr(nShaPos+9, 32));
+						Item.Extra[idx] = StripWhite(Item.Extra[idx].substr(0, nShaPos) + Item.Extra[idx].substr(nShaPos+42));
+					}
+					if (NonPrintable)
+						Item.Extra[idx] = CString("data contains non-printable characters");
+				}
+				if (Packet[i+1] == 0x1C)
+					i++;
 			}
 			/*// TODO: get rid of this rubbush
 			for(i = NextPos + 8; !(Packet[i] == '\0' && Packet[i + 1] == '\0'); i++)
@@ -1491,8 +1723,6 @@ void MGnuDirector::OnQueryHit(packet_QueryHit* QueryHit, MGnuNode* pOrigin, bool
 				else
 					break;*/
 
-			Item.NameLower = Item.Name;
-			MakeLower(Item.NameLower);
 			// now check if this result matches some of the queries
 			//cout << "query hit: " << Item.NameLower << endl;
 			m_listmutex.lock();
@@ -1521,7 +1751,7 @@ void MGnuDirector::OnQueryHit(packet_QueryHit* QueryHit, MGnuNode* pOrigin, bool
 
 void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 {
-	POST_DMESSAGE("MGnuDirector::OnTimer()");
+	//POST_DMESSAGE("MGnuDirector::OnTimer()");
 	//double BytesReceived = 0,  BytesSent = 0;
 	int    nNumUploads   = CountUploads();
 
@@ -1674,23 +1904,62 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 
 	// Downloads
 	m_dDownloadRate = 0;
+	int nReceivingDownloadSocks = 0;
+	int nRC;
 	for(i = 0; i < m_DownloadList.size(); i++)
 	{
 		MGnuDownload *pDown = m_DownloadList[i];
 		pDown->OnTimer(nTimeStamp);
-		if(pDown->CountActiveConnections(NULL))
+		nRC = pDown->CountReceivingConnections(NULL);
+		if(nRC)
+		{
 			m_dDownloadRate += pDown->GetRate();
+			nReceivingDownloadSocks += nRC;
+		}
 	}
-	// Bandwidth
+	// recalculate download rate limits
+	if (m_pPrefs->m_dBandwidthDownl > 0)
+	{
+		if (nReceivingDownloadSocks)
+		{
+			register double dBand = m_pPrefs->m_dBandwidthDownl * 1024.0;
+			if (m_dDownloadRate > dBand)
+			{
+				m_dDownloadRateLimit -= (m_dDownloadRate - dBand)*0.7; // 0.9 is here to make it all more smooth
+				if (m_dDownloadRateLimit < dBand * 0.1)
+					m_dDownloadRateLimit = dBand * 0.1; // dont let it drop to 0
+			}
+			else
+			{
+				// limit is possibly too high we shall make it more relaxed
+				m_dDownloadRateLimit += (dBand - m_dDownloadRateLimit) * 0.5; // 0.5 reflects our intention not to make sudden moves
+				if (m_dDownloadRateLimit > 3.0*dBand)
+					m_dDownloadRateLimit = 3.0*dBand; // dont let it grow really high
+			}
+			//cout << "m_dDownloadRateLimit = " << m_dDownloadRateLimit << endl;
+		}
+		else
+			m_dDownloadRateLimit =  m_pPrefs->m_dBandwidthDownl * 1024.0;
+	}
+	else
+		m_dDownloadRateLimit = 0; // means 'no limit'
+	// update downloads rate limits
+	for(i = 0; i < m_DownloadList.size(); i++)
+	{
+		MGnuDownload *pDown = m_DownloadList[i];
+		pDown->SetRatePerSocket((int) (m_dDownloadRateLimit / (nReceivingDownloadSocks>=1 ? nReceivingDownloadSocks : 1)));
+	}
+	
+	// track total bandwidth
 	int BitSpeed = (int) ((m_dwConnRecvRate+m_dwConnSendRate+m_dDownloadRate+m_dUploadRate) / 1024 * 8);
 	if(BitSpeed > m_pPrefs->m_dwSpeedDyn)
 		m_pPrefs->m_dwSpeedDyn = BitSpeed;
 
 	// renew searches
-	if (m_pPrefs->m_nResubmitSearches && m_SearchMap.size())
+	if (m_pPrefs->m_nResubmitSearches > 0 && m_SearchMap.size())
 	{
 		m_nReSearchTimer++;
-		if ( m_nReSearchTimer >= (2 /*let the min time be 2 sec*/ + m_pPrefs->m_nResubmitSearches/m_SearchMap.size()) )
+		if ( m_nReSearchTimer >= (2 /*let the min time be 2 sec*/ + m_pPrefs->m_nResubmitSearches * 60 / m_SearchMap.size()) )
 		{
 			m_nReSearchTimer = 0;
 			map<DWORD, MGnuSearch*>::iterator itS = m_SearchMap.upper_bound(m_dwReSearchID);
@@ -1700,6 +1969,9 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 			m_dwReSearchID = itS->first;
 			if (!itS->second->IsFull()) //food protection for searches like ".mp3"
 				m_PendingQueries.push_back(m_dwReSearchID);
+			// sanity check against "non-cooperative users"
+			if (m_pPrefs->m_nResubmitSearches < 5)
+				m_pPrefs->m_nResubmitSearches = 30;
 		}
 	}
 
@@ -1744,54 +2016,48 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 	for (itS = m_SearchMap.begin(); itS != m_SearchMap.end(); ++itS)
 	{
 		MGnuSearch* pS = itS->second;
-		pS->m_mutex.lock();
-		if (pS->m_bAutoget && pS->m_bUpdated && pS->m_mapResults.size())
+		if (pS->IsAutoget() && pS->IsUpdated() && pS->GetResultsCount())
 		{
-			//ASSERT(pS->m_SizeFilterMode == LIMIT_EXACTLY);
 			POST_EVENT( MStringEvent(
 				ET_MESSAGE,
 				ES_JUSTINCASE,
-				pS->m_Filename.size() ? ("`" + pS->m_Filename + "'") : ("for user autoget search `" + pS->m_Search + "'"),
+				pS->GetFilename().size() ? ("`" + pS->GetFilename() + "'") : ("for user autoget search `" + pS->GetSearchString() + "'"),
 				MSG_AUTOGET,
 				MSGSRC_DIRECTOR
 			));
 			//
-			pS->m_bUpdated = false;
+			pS->SetUpdated(false);
 			bool bAlready = false;
 			// build up the vector of results
 			ResultVec rv;
-			rv.reserve(pS->m_mapResults.size());
-			for (map<DWORD,Result>::iterator itr = pS->m_mapResults.begin(); itr != pS->m_mapResults.end(); ++itr )
+			pS->GetResults(rv);
+			for (vector<Result>::iterator itr = rv.begin(); itr != rv.end(); ++itr )
 			{
 				// check if we already downloading this
-				if ( LookUpDownload(itr->second.Host, itr->second.NameLower, itr->second.FileIndex) )
+				if ( LookUpDownload(itr->Host, itr->Name, itr->FileIndex) )
 				{
 					bAlready = true;
 					break;
 				}
-				rv.push_back(itr->second);
 			}
 			if (!bAlready)
 			{
-				MGnuDownload* pD =  new MGnuDownload(this, rv);
+				MGnuDownload* pD = new MGnuDownload(this, rv, pS->GetFilename(), pS->GetFileHandle());
 				ASSERT(pD);
-				if (pS->m_Filename.size())
-					pD->SetFileName(pS->m_Filename);
-				if (pS->m_SizeFilterMode == LIMIT_EXACTLY)
-					pD->SetSearchID(pS->m_dwID);
+				if (pS->GetSizeFilterMode() == LIMIT_EXACTLY)
+					pD->SetSearchID(pS->GetID());
 				// add to download queue
 				AddAndStartDownload(pD);
 			}
 			//
-			pS->m_bAutoget = false;
+			pS->SetAutoget(false);
 		}
-		pS->m_mutex.unlock();
 	}
 
 	// Gnutella Web Cache
 	if (!m_WebCachesUrlQueue.empty())
 	{
-		if (nTimeStamp % 300 == 0) // once in 5 minutes
+		if (nTimeStamp % 900 == 0) // once in 15 minutes
 		{
 			// update hosts list
 			// TODO: get a random URL
@@ -1801,7 +2067,7 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 			// Add client connection
 			OpenWebCacheConnection(sURL, MGnuWebCache::HostFile, false);
 		}
-		else if (nTimeStamp % 300 == 150) // once in 5 minutes
+		else if (nTimeStamp % 900 == 450) // once in 15 minutes
 		{
 			// update URL list
 			// TODO: get a random URL
@@ -1811,7 +2077,7 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 			// Add client connection
 			OpenWebCacheConnection(sURL, MGnuWebCache::UrlFile, false);
 		}
-		else if ( false /*nTimeStamp % 600 == 599*/ ) // once in 10 minutes
+		else if ( IsUltraPeer() && nTimeStamp % 1800 == 1799 ) // once in 30 minutes
 		{
 			// m_pPrefs->m_nMaxConnects < 0 || GetLastConnCount() <= m_pPrefs->m_nMaxConnects * 0.8
 			// inform the world about us
@@ -1827,13 +2093,49 @@ void MGnuDirector::OnTimer(int nTimeStamp) // called once per second
 	{
 		if (nTimeStamp % 30 == 0) // maximum once in 30 secs
 		{
+			// check if our initial servers have not been updated
+			if (m_asInitialWebCaches[0] != m_pPrefs->m_szGWebCache1)
+			{
+				m_asInitialWebCaches[0] = m_pPrefs->m_szGWebCache1;
+				m_anInitialWebCacheRequests[0] = 0;
+				m_bNoValidWebCacheUrlsMessageSent = false; // we reset the flag if user changes something
+			}
+			if (m_asInitialWebCaches[1] != m_pPrefs->m_szGWebCache2)
+			{
+				m_asInitialWebCaches[1] = m_pPrefs->m_szGWebCache2;
+				m_anInitialWebCacheRequests[1] = 0;
+				m_bNoValidWebCacheUrlsMessageSent = false; // we reset the flag if user changes something
+			}
+			if (m_asInitialWebCaches[2] != m_pPrefs->m_szGWebCache3)
+			{
+				m_asInitialWebCaches[2] = m_pPrefs->m_szGWebCache3;
+				m_anInitialWebCacheRequests[2] = 0;
+				m_bNoValidWebCacheUrlsMessageSent = false; // we reset the flag if user changes something
+			}
+			// check if our initial servers are valid
+			bool bHaveValidUrls = false;
 			// request urls from the initial connect servers
-			if (strlen(m_pPrefs->m_szGWebCache1))
-				OpenWebCacheConnection( m_pPrefs->m_szGWebCache1, MGnuWebCache::UrlFile, false);
-			if (strlen(m_pPrefs->m_szGWebCache2))
-				OpenWebCacheConnection( m_pPrefs->m_szGWebCache2, MGnuWebCache::UrlFile, false);
-			if (strlen(m_pPrefs->m_szGWebCache3))
-				OpenWebCacheConnection( m_pPrefs->m_szGWebCache3, MGnuWebCache::UrlFile, false);
+			for (int i=0; i<3; ++i)
+				if (m_anInitialWebCacheRequests[i] < 5 && m_asInitialWebCaches[i].length())
+				{
+					OpenWebCacheConnection( m_asInitialWebCaches[i], MGnuWebCache::UrlFile, false);
+					++m_anInitialWebCacheRequests[i];
+					bHaveValidUrls = true;
+				}
+			// inform user if we have no valid urls	
+			if (!bHaveValidUrls)
+			{
+				if (!m_bNoValidWebCacheUrlsMessageSent)
+				{
+					POST_ERROR(
+						ES_IMPORTANT,
+						"All initial GWebCaches appear to be invalid or unreacheable. Supply working Gnutella web cache URLs or verify your internet connection. Currently it is imposible to initiate connection to the Gnutella network."
+					);
+					m_bNoValidWebCacheUrlsMessageSent = true;
+				}
+			}
+			else
+				m_bNoValidWebCacheUrlsMessageSent = false; // we reset the flag if there were valid URLs
 		}
 	}
 
@@ -1934,8 +2236,12 @@ void MGnuDirector::ManageConnects(int nTimeStamp)
 		MLock lock (m_listmutex);
 		map<DWORD, MGnuNode*>::iterator itNode;
 		for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+		{
+			ASSERT(itNode->second);
+			ASSERT(itNode->second->GetID() == itNode->first);
 			if(itNode->second->GetStatus() == SOCK_CLOSED)
 				queueDeadIDs.push(itNode->first);
+		}
 		// TODO: move this to the very end... to collect all the deads
 		while (queueDeadIDs.size())
 		{
@@ -1952,11 +2258,14 @@ void MGnuDirector::ManageConnects(int nTimeStamp)
 	int nNormalConnects = 0;
 	int nLeafConnects = 0;
 	int nUltraConnects = 0;
+	int nNodeMapSize = 0;
 	{
 		MLock lock (m_listmutex);
+		nNodeMapSize = m_NodeMap.size();
 		map<DWORD, MGnuNode*>::iterator itNode;
 		for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 		{
+			ASSERT(itNode->second);
 			if(itNode->second->GetStatus() == SOCK_CONNECTED)
 			{
 				if( m_nHostMode == CM_ULTRAPEER )
@@ -1978,7 +2287,7 @@ void MGnuDirector::ManageConnects(int nTimeStamp)
 
 		for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 		{
-			itNode->second->SetMaxInRate(m_dCurrentReceiveLimit / (nNormalConnects + nLeafConnects));
+			itNode->second->SetMaxInRate( (DWORD) (m_dCurrentReceiveLimit / (nNormalConnects + nLeafConnects)) );
 			itNode->second->OnTimer();
 		}
 	}
@@ -2012,8 +2321,13 @@ void MGnuDirector::ManageConnects(int nTimeStamp)
 	{
 		if(IsLeaf())
 		{
-			if(nNormalConnects < m_pPrefs->m_nLeafModeConnects)
+			if(nNormalConnects < m_pPrefs->m_nLeafModeConnects && nNodeMapSize < m_pPrefs->m_nLeafModeConnects * 2)
+			{
 				AddConnect(true);
+				// MZ: add more connects at once -- we dont seem to find
+				//     proper ultrapeers quickly
+				AddConnect(true, true);
+			}
 		}
 		else
 		{
@@ -2030,12 +2344,15 @@ void MGnuDirector::ManageConnects(int nTimeStamp)
 }
 
 // Node management functions
-void MGnuDirector::AddConnect(bool bSuperpeer)
+void MGnuDirector::AddConnect(bool bSuperpeer, bool bUseStore /*=false*/)
 {
+	if (bUseStore)
+		bUseStore = !m_pHostStore->IsHalfEmpty();
+	//
 	Node node;
-	if ( (bSuperpeer && m_pUltraCatcher->GetRandomNode(node)) ||
-		 m_pHostCatcher->GetRandomNode(node)                  ||
-		 m_pHostStore->GetRandomNode(node)                    )
+	if ( !bUseStore && (bSuperpeer && m_pUltraCatcher->GetRandomNode(node)) ||
+		 !bUseStore && m_pHostCatcher->GetRandomNode(node)                  ||
+		               m_pHostStore->GetRandomNode(node)                    )
 	{
 		if (IsOkForDirectConnect(node.Host) && NotLocal(node))
 			AddNode( node.Host, node.Port );
@@ -2055,7 +2372,7 @@ void MGnuDirector::AddConnect(bool bSuperpeer)
 	}
 }
 
-void MGnuDirector::AddWebCacheUrl(const CString& sURL)
+void MGnuDirector::AddWebCacheUrl(const CString& sURL, bool bChecked /*=false*/)
 {
 	if (sURL.empty())
 		return;
@@ -2083,6 +2400,58 @@ void MGnuDirector::AddWebCacheUrl(const CString& sURL)
 		m_WebCachesUrlSet.erase(m_WebCachesUrlQueue.front());
 		m_WebCachesUrlQueue.pop_front();
 	}
+	// keep track of the last good URL
+	if (bChecked)
+		m_sLastGoodWebCacheUrl = sURL;
+}
+
+void MGnuDirector::OnWebCacheReply(const CString& sURL)
+{
+	AddWebCacheUrl(sURL, true);
+	if (sURL == m_pPrefs->m_szGWebCache1)
+		m_anInitialWebCacheRequests[0] = 0;
+	else if (sURL == m_pPrefs->m_szGWebCache2)
+		m_anInitialWebCacheRequests[1] = 0;
+	else if (sURL == m_pPrefs->m_szGWebCache3)
+		m_anInitialWebCacheRequests[2] = 0;
+}
+
+bool MGnuDirector::SaveWebCaches(const CString sPath)
+{
+	if( m_WebCachesUrlSet.empty() )
+		return true; // all OK, we just have nothing to write
+	FILE * pFile;
+	if ( (pFile = fopen(ExpandPath(sPath).c_str(), "w")) )
+	{
+		fprintf(pFile, "# this is mutella's GWebCache list file\n");
+		fprintf(pFile, "# it is created automatically at the end\n");
+		fprintf(pFile, "# of mutella session and updated periodically\n");
+		fprintf(pFile, "# during the session\n");
+		fprintf(pFile, "# \n");
+		for ( set<CString>::iterator it = m_WebCachesUrlSet.begin(); it != m_WebCachesUrlSet.end(); ++it)
+			fprintf(pFile, "%s\n", it->c_str());
+		fclose(pFile);
+	}
+	return NULL!=pFile;
+}
+
+bool MGnuDirector::LoadWebCaches(const CString sPath)
+{
+	FILE * pFile;
+	if ( (pFile = fopen(ExpandPath(sPath).c_str(), "r")) )
+	{
+		CString sURL;
+		char buf[1024];
+		while ( !feof(pFile) )
+		{
+			sURL = fgets(buf,1024,pFile);
+			if ( sURL.length()==0 || buf[0]=='#' )
+				continue;
+			AddWebCacheUrl(sURL);
+		}
+		fclose(pFile);
+	}
+	return NULL!=pFile;
 }
 
 void MGnuDirector::RemoveWebCacheUrl(const CString& sURL)
@@ -2122,6 +2491,7 @@ void MGnuDirector::DropSomeNode(int nTimeStamp)
 	map<DWORD, MGnuNode*>::iterator itNode, itRemove;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 	{
+		ASSERT(itNode->second);
 		if( itNode->second->GetStatus() == SOCK_CONNECTED )
 		{
 			if( itNode->second->GetPeerMode() == CM_NORMAL        &&
@@ -2154,6 +2524,7 @@ void MGnuDirector::DropSomeLeaf(int nTimeStamp)
 	map<DWORD, MGnuNode*>::iterator itNode, itRemove;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected()                     &&
 			itNode->second->GetPeerMode() == CM_LEAF          &&
 			itNode->second->GetEfficeincy() <= dwLowestRating )
@@ -2218,10 +2589,13 @@ void MGnuDirector::RemoveNode_LeastReplyRate(int nTimeStamp)
 	nodes.reserve(m_NodeMap.size());
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected()                                &&
 			itNode->second->GetPeerMode() != CM_LEAF                     &&  // do not kill leaves, they are not supposed to be too good anyway
 			(nTimeStamp - itNode->second->GetSGnuNode().m_nUptime) > 180 )   // be kind to new connections, give them 3 minutes
 				nodes.push_back(itNode->second);
+	}
 	//
 	if(nodes.size()<3)
 		return;
@@ -2408,9 +2782,12 @@ int MGnuDirector::CountUltrapeerLeaves()
 	MLock lock(m_listmutex);
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected()            &&
 			itNode->second->GetPeerMode() == CM_LEAF )
 				nChildCount++;
+	}
 	return nChildCount;
 }
 
@@ -2420,6 +2797,8 @@ int MGnuDirector::CountNormalConnects()
 	MLock lock(m_listmutex);
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected() )
 		{
 			if(m_nHostMode == CM_ULTRAPEER)
@@ -2430,6 +2809,7 @@ int MGnuDirector::CountNormalConnects()
 			else
 				nNormalConnects++;
 		}
+	}
 	m_nLastNormalConnects = nNormalConnects;
 	return nNormalConnects;
 }
@@ -2445,7 +2825,7 @@ void MGnuDirector::CloseAllOtherNodes(MGnuNode* pExcept)
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
 		if( itNode->second != pExcept )
-			itNode->second->Close();
+			itNode->second->ForceDisconnect();
 }
 
 void MGnuDirector::Route_Pong(packet_Pong* pPong, int nLength, DWORD dwRouteID)
@@ -2472,7 +2852,8 @@ void MGnuDirector::Route_UltraPong(packet_Pong* pPong, int nLength, DWORD dwRout
 	if(itNode != m_NodeMap.end())
 	{
 		MGnuNode* p = itNode->second;
-		if( p->GetPeerMode() == CM_LEAF &&
+		ASSERT(p);
+		if( p->GetPeerMode() == CM_LEAF &&  // it could have crashed here
 			p->IsConnected()            &&
 			!p->m_UltraPongSent         )
 		{
@@ -2509,7 +2890,7 @@ void MGnuDirector::Route_QueryHit(packet_QueryHit* pQueryHit, int nLength, DWORD
 void MGnuDirector::EnableSupernodePromotion(bool bForce /*=false*/ )
 {
 	// this function analyses the situation and changes the node capability if appropriate
-	if (m_nHostMode == CM_ULTRAPEER || m_nHostMode == CM_ULTRALEAF)
+	if (m_nHostMode == CM_ULTRAPEER || (!bForce && m_nHostMode == CM_ULTRALEAF))
 		return;
 	// Cant be behind firewall
 	if (!bForce                                                                     &&
@@ -2527,9 +2908,12 @@ void MGnuDirector::EnableSupernodePromotion(bool bForce /*=false*/ )
 		MLock lock(m_listmutex);
 		map<DWORD, MGnuNode*>::iterator itNode;
 		for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+		{
+			ASSERT(itNode->second);
 			if( itNode->second->IsConnected()            &&
 				itNode->second->GetPeerMode() == CM_LEAF )
-					itNode->second->Close();
+					itNode->second->ForceDisconnect();
+		}
 	}
 	else
 		// upgrade to utrapeer-capable leaf
@@ -2552,8 +2936,11 @@ void MGnuDirector::DisableSupernodePromotion()
 	MLock lock(m_listmutex);
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected() && itNode->second->GetPeerMode() == CM_LEAF )
-			itNode->second->Close();
+			itNode->second->ForceDisconnect();
+	}
 }
 
 bool MGnuDirector::IsLeaf()
@@ -2573,8 +2960,11 @@ bool MGnuDirector::IsShieldedLeaf()
 	MLock lock();
 	map<DWORD, MGnuNode*>::iterator itNode;
 	for(itNode = m_NodeMap.begin(); itNode != m_NodeMap.end(); itNode++)
+	{
+		ASSERT(itNode->second);
 		if( itNode->second->IsConnected() && itNode->second->GetPeerMode() == CM_ULTRAPEER )
 			return true;
+	}
 	return false;
 }
 
