@@ -38,6 +38,7 @@ class MGnuShare;
 class MGnuSearch;
 class MGnuWebCache;
 class MGnuWordHash;
+class MGnuMarkedFiles;
 class MAsyncDns;
 class MUI;
 
@@ -86,10 +87,13 @@ public:
 	bool ModifySearch(DWORD dwID, LPCSTR szSearch);
 	MGnuSearch* GetSearchByID(DWORD dwID);
 	bool GetSearchByID(DWORD dwID, SGnuSearch& gs, vector<Result>& rv, vector<ResultGroup>& gv);
+	int  GetSearchesCount();
 	bool GetResultsByID(DWORD dwID, vector<Result>& rv);
 	bool ClearSearchByID(DWORD dwID);
 	bool RemoveSearchByID(DWORD dwID, bool bRemovePartial = false);
 	MGnuSearch* LookUpSearch(const CString& search, int size);
+	MGnuSearch* GetSearchByFile(const CString& name, int size);
+	void RemoveSearchesByType(int nType);
 	//
 	void ForEachConnection(void* pData, tEachConnection callback);
 	bool CloseConnectionByID(DWORD dwID);
@@ -148,6 +152,8 @@ public:
 	DWORD GetUltrapeerSizeMarker();
 	void  OnShareChanged();
 
+	MGnuMarkedFiles* GetMarkedFiles() { return m_pKnownFiles; }
+
 	double FreeUltraCapacity(int nRemoteTotal);
 	double RunningUltraCapacity(int nRemoteTotal);
 
@@ -185,7 +191,7 @@ public:
 	void Broadcast_Ping(packet_Ping*,   int, MGnuNode*);
 	void Broadcast_Query(packet_Query*, int, MGnuNode*);
 	void Send_ForwardQuery(const QueryComp&, const list<DWORD>& NodeIDs );
-	GUID Broadcast_LocalQuery(BYTE*, WORD, int);
+	GUID Broadcast_LocalQuery(BYTE*, int);
 	void SendAllLocalQueries(MGnuNode* pNode);
 
 	void Route_Pong(packet_Pong*, int, key_Value*);
@@ -233,14 +239,19 @@ public:
 	
 	void RemoveNode_LeastReplyRate(int nTimeStamp);
 	void RemoveNode_MaxDrop(int nTimeStamp);
-	void AddConnect(bool bSuperpeer);
+	void AddConnect(bool bSuperpeer, bool bUseStore = false);
 	void ManageConnects(int nTimeStamp);
 	
 	bool IsInTheList(MGnuNode* pNode);
 
-	void AddWebCacheUrl(const CString& sURL);
+	void AddWebCacheUrl(const CString& sURL, bool bChecked = false);
+	void OnWebCacheReply(const CString& sURL);
 	void RemoveWebCacheUrl(const CString& sURL);
 	void OpenWebCacheConnection(const CString& sURL, int nRequestType, bool bLock = true);
+	const CString& GetLastGoodWebCacheUrl() { return m_sLastGoodWebCacheUrl; }
+	
+	bool SaveWebCaches(const CString sPath);
+	bool LoadWebCaches(const CString sPath);
 
 	IP    GetPublicIP();
 	DWORD GetPublicPort();
@@ -260,8 +271,12 @@ protected:
 	GUID    m_ClientID;
 	DWORD   m_dwFiles;
 	DWORD   m_dwFilesSize;
-	int		m_nSecsAboveLimitConn; // Seconds over max conn bandwidth
+
+	//int		m_nSecsAboveLimitConn; // Seconds over max conn bandwidth
+	
 	double	m_dCurrentReceiveLimit;
+	double m_dDownloadRateLimit;
+
 	int     m_nPort;
 	// Used to get hostname from an IP, (one per thread)
 	hostent* m_HostEnt;
@@ -278,6 +293,10 @@ protected:
 	vector<MGnuWebCache*>   m_WebCacheList;
 	set<CString>            m_WebCachesUrlSet;
 	std::deque<CString>     m_WebCachesUrlQueue;
+	CString                 m_sLastGoodWebCacheUrl;
+	int                     m_anInitialWebCacheRequests[3];
+	CString                 m_asInitialWebCaches[3];
+	bool                    m_bNoValidWebCacheUrlsMessageSent;
 	// custom web folders
 	map<CString, MUI*>    m_mapWebFolders;
 	// access control to the lists above
@@ -285,6 +304,8 @@ protected:
 	//
 	std::queue<QueuedHit> m_hitQueue;
 	MMutex m_hitqueuemutex;
+
+	MGnuMarkedFiles* m_pKnownFiles;
 	//
 	int m_nLastConnCount;
 	// download queuing
