@@ -36,6 +36,7 @@
 #include "gnudirector.h"
 #include "gnusearch.h"
 #include "gnudownload.h"
+#include "gnumarkedfiles.h"
 
 #include "asyncfile.h"
 #include "messages.h"
@@ -258,6 +259,8 @@ public:
 	void Enqueue(); // changes status to "TRANSFER_QUEUED"
 	bool OnSetStartPoint(int nFilePos); // return true if start point was accepted
 	void OnDataWritten(DWORD dwPos, DWORD dwSize, MDownloadBuffer* pB);
+	//
+	void SetMaxRate(long nBytesPerSec) { m_nMaxReceiveRate = nBytesPerSec; }
 protected:
 	Result m_result;
 	//
@@ -276,6 +279,10 @@ protected:
 	int m_nSecBytesReceived; // bytes received in the last second
 	int m_dwRate;
 	int m_nSecsUnderLimit;
+	// rate limiting
+	int m_nMaxReceiveRate;
+	bool m_bReceiveIsBlocked_Rate;
+	bool m_bReceiveIsBlocked_Buff;
 	// double buffering
 	MDownloadBuffer* m_pReceiveBuff;
 	MDownloadBuffer* m_pStoreBuff;
@@ -665,6 +672,10 @@ MDownloader::MDownloader(MGnuDownload* pDownl, MGnuDirector* pDirector, const Re
 	m_nSecsUnderLimit = 0;
 	m_bServerIsUp = false;
 	m_bForceUriRes = false;
+	//
+	m_nMaxReceiveRate = 0;
+	m_bReceiveIsBlocked_Rate = false;
+	m_bReceiveIsBlocked_Buff = false;
 	// double buffering
 	m_pReceiveBuff = NULL;
 	m_pStoreBuff = NULL;
@@ -719,9 +730,22 @@ void MDownloader::OnTimer(int nTimeStamp)
 				ForceDisconnect(REASON_NO_RESPONSE, m_nPushTimeouts>=20 ? TRANSFER_CLOSED : TRANSFER_FAILED);
 			}
 			break;
-		case TRANSFER_RECEIVING: // check for min speed limit
-			if(pPrefs->m_dMinDownSpeed > 0 && nSecStable>pPrefs->m_dwSpeedTimeout)
+		case TRANSFER_RECEIVING:
+			// check for bandwidth limits
+			if (m_bReceiveIsBlocked_Rate && m_nMaxReceiveRate > m_dwRate)
 			{
+				if (!m_bReceiveIsBlocked_Buff)
+				{
+					TRACE("Enabling OnReceive in OnTimer");
+					ModifySelectFlags(FD_READ, 0);
+				}
+				m_bReceiveIsBlocked_Rate = false;
+			}
+	
+			// check for min speed limit
+			if(!m_bReceiveIsBlocked_Rate && pPrefs->m_dMinDownSpeed > 0 && nSecStable>pPrefs->m_dwSpeedTimeout)
+			{
+				// TODO: resole conflicts between MinDownloadSpeed and RateLimit
 				// Check if its under the bandwidth limit
 				if(m_dwRate < pPrefs->m_dMinDownSpeed &&
 				   m_pDownload->GetNumAlternatives() > 1)
@@ -781,6 +805,7 @@ void MDownloader::ForceDisconnect(int nReason, int nStatus)
 		m_pReceiveBuff &&
 		m_pReceiveBuff->contains())
 	{
+		TRACE3("Storing remains of receive buffer ( ", m_pReceiveBuff->contains(), " bytes )");
 		ASSERT(m_pStoreBuff);
 		if (REASON_STORAGE_DENIED != nReason &&
 			REASON_WRITE_ERROR != nReason)
@@ -789,7 +814,8 @@ void MDownloader::ForceDisconnect(int nReason, int nStatus)
 			// against repeated saves
 
 			// TODO: do it more properly!
-			if ( SR_AsyncError == m_pDownload->StoreDataAt(this, m_nFilePos+m_pStoreBuff->contains(), m_pReceiveBuff, m_pReceiveBuff->contains()) )
+			StoreResult sr = m_pDownload->StoreDataAt(this, m_nFilePos+m_pStoreBuff->contains(), m_pReceiveBuff, m_pReceiveBuff->contains());
+			if ( SR_AsyncError == sr )
 			{
 				// this error may occur when the previous store rquest is not yet processed by the disk-write thread
 				// the only way to fix thi without rewriting the whole of MAsyncFile is to sleep for a second and try again
@@ -809,6 +835,21 @@ void MDownloader::ForceDisconnect(int nReason, int nStatus)
 					if (nStatus == TRANSFER_COMPLETED)
 						nStatus = TRANSFER_FAILED;
 				}
+			} else if ( sr == SR_NoMatch)
+			{
+				nStatus = TRANSFER_CLOSED;
+				nReason = REASON_FILES_ARE_DIFFERENT;
+				POST_EVENT( MStringEvent(
+					ET_ERROR,
+					ES_JUSTINCASE,
+					"file:`" + m_result.Name + "' host: " + Ip2Str(m_result.Host),
+					MSG_DOWNLOAD_FILE_MISSMATCH,
+					MSGSRC_DOWNLOAD
+				));
+			} else if ( sr != SR_Success )
+			{
+				nStatus = TRANSFER_CLOSED;
+				TRACE("MDownloader::ForceDisconnect() : updated status due to unknown error");
 			}
 			
 			m_pReceiveBuff->Release();
@@ -889,9 +930,14 @@ bool MDownloader::OnIncomingPush(const CString& sFileName, int nIndex, SOCKET hP
 		return false;
 	if (nIndex != m_result.FileIndex)
 		return false;
-	if (sFileName != m_result.NameLower) // make more advanced checks to increase success rate
-	                                     // like remove path or whatever...
-		return false;
+	if (0 != CompareNoCase(sFileName, m_result.Name))
+	{
+		// make more advanced checks to increase success rate
+		// like remove path or whatever...
+		int nMinLen = min(sFileName.length(), m_result.Name.length());
+		if (nMinLen<5 || 0 != CompareNoCase(sFileName.substr(0,nMinLen), m_result.Name.substr(0,nMinLen)))
+			return false;
+	}
 	// we have right incoming push!
 	ASSERT(m_hSocket==INVALID_SOCKET);
 	Attach(hPushSock, FD_READ|FD_CLOSE);
@@ -944,7 +990,7 @@ void MDownloader::OnDataWritten(DWORD dwPos, DWORD dwSize, MDownloadBuffer* pB)
 		ForceDisconnect(REASON_WRITE_ERROR, TRANSFER_CLOSED);
 		return;
 	}
-	m_nFilePos += dwSize;	
+	m_nFilePos += dwSize;
 	// if we are storing something else than m_pStoreBuff => it is called from ForceDisconnect();
 	if ( m_pStoreBuff == NULL ||
 		 m_pStoreBuff != pB   )
@@ -955,11 +1001,15 @@ void MDownloader::OnDataWritten(DWORD dwPos, DWORD dwSize, MDownloadBuffer* pB)
 	}
 	// all seems to be OK
 	m_pStoreBuff->clear();
-	
+
 	if (m_pReceiveBuff == NULL || !m_pReceiveBuff->isFull())
 		return;
-	TRACE("Enabling OnReceive");
-	ModifySelectFlags(FD_READ, 0);
+	if (!m_bReceiveIsBlocked_Rate)
+	{
+		TRACE("Enabling OnReceive in OnDataWritten");
+		ModifySelectFlags(FD_READ, 0);
+	}
+	m_bReceiveIsBlocked_Buff = false;
 	swap(m_pReceiveBuff, m_pStoreBuff);
 	StoreStoreBuff();
 }
@@ -1018,7 +1068,7 @@ void MDownloader::OnReceive(int nErrorCode)
 	ASSERT(m_pStoreBuff);
 	//
 	m_nLastActive = xtime();
-	
+
 	int nReceived = Receive(m_pReceiveBuff->space(), m_pReceiveBuff->remains());
     //printf("recieved %d bytes, errno = %d\n", BuffLength, errno);
 	// Check for errors
@@ -1035,14 +1085,23 @@ void MDownloader::OnReceive(int nErrorCode)
 	}
 	m_pReceiveBuff->increment(nReceived);
 	m_nSecBytesReceived += nReceived;
-	
+
 	// Connected and downloading
 	if(TRANSFER_RECEIVING == m_nStatus)
 	{
+		// rate limiting
+		if (m_nMaxReceiveRate > 0 &&
+			m_nSecBytesReceived >= m_nMaxReceiveRate )
+		{
+			// block OnReceive
+			TRACE("Disabling OnReceive because of the rate limit");
+			ModifySelectFlags(0, FD_READ);
+			m_bReceiveIsBlocked_Rate = true;
+		}
 		// do something about the buffers and if the receive buffer is full
 		// and store buffer is empty -- store current receive buffer and swap those
 		// otherwise, if the receive buffer is full -- just block further receives
-		
+
 		// check for compltetion
 		if (m_result.Size <= m_pReceiveBuff->contains() + m_pStoreBuff->contains() + m_nFilePos)
 		{
@@ -1063,8 +1122,9 @@ void MDownloader::OnReceive(int nErrorCode)
 		if (!m_pStoreBuff->isEmpty())
 		{
 			// block OnReceive
-			TRACE("Disabling OnReceive");
+			TRACE("Disabling OnReceive because of buffers");
 			ModifySelectFlags(0, FD_READ);
+			m_bReceiveIsBlocked_Buff = true;
 			return;
 		}
 		swap(m_pReceiveBuff, m_pStoreBuff);
@@ -1207,7 +1267,7 @@ void MDownloader::OnReceive(int nErrorCode)
 		// Server error code
 		else if(400 <= nCode && nCode < 500)
 		{
-			if (m_bForceUriRes || m_result.Sha1.empty())
+			if (m_bForceUriRes || !m_result.Sha1.isValid())
 				ForceDisconnect(REASON_FILE_NOT_FOUND, TRANSFER_CLOSED);
 			else
 			{
@@ -1250,7 +1310,7 @@ void MDownloader::SendRequest()
 	// create the request
 	CString sGetRequest;
 	CString sGetFormat;
-	if (m_result.Vendor == "BEAR" && m_result.Sha1.size()==32)
+	if (m_result.Vendor == "BEAR" && m_result.Sha1.isValid())
 		m_bForceUriRes = true;
 	if (m_bForceUriRes)
 	{
@@ -1258,7 +1318,7 @@ void MDownloader::SendRequest()
 							 "User-Agent: Mutella-" + VERSION + "\r\n" +
 							 "Range: bytes=%d-\r\n" +
 							 "X-Gnutella-Content-URN: urn:sha1:%s\r\n\r\n";
-		sGetRequest.format(sGetFormat.c_str(), m_result.Sha1.c_str(), m_nFilePos, m_result.Sha1.c_str());
+		sGetRequest.format(sGetFormat.c_str(), m_result.Sha1.toStr().c_str(), m_nFilePos, m_result.Sha1.toStr().c_str());
 	}
 	else
 	{
@@ -1267,9 +1327,9 @@ void MDownloader::SendRequest()
 							 "Range: bytes=%d-\r\n\r\n";
 		sGetRequest.format(sGetFormat.c_str(), m_result.FileIndex, m_result.Name.c_str(), m_nFilePos);
 	}
-	
+
 	Send(sGetRequest.c_str(), sGetRequest.length());
-	
+
 	SetStatus(TRANSFER_NEGOTIATING);
 
 }
@@ -1307,29 +1367,32 @@ bool bySpeed(const Result* pr1, const Result* pr2)
 ////////////////////////////////////////////////////////////////////
 // MGnuDownload implementation
 
-MGnuDownload::MGnuDownload(MGnuDirector* pDir, const ResultVec& results) : m_mutex(true)
+MGnuDownload::MGnuDownload(MGnuDirector* pDir, const ResultVec& results, const CString& sFilename, long nFileHandle) : m_mutex(true)
 {
 	m_pDirector  = pDir;
 	m_pPrefs     = m_pDirector->GetPrefs();
-	
+
 	m_dwBytesCompleted = 0;
 	m_dwResumeStartPosition = 0;
 	m_dRate = 0;
-	
+
 	// timers
 	m_nCreationTime   = xtime();
 	//m_nSecInactive    = 0;
 	//
 	m_nLastDisconnectReason = REASON_UNDEFINED;
 	m_nStatus           = TRANSFER_NEW;
-	
+
 	m_dwSearchID = 0;
 	m_bDelSearch = false;
-	
+
 	m_dwID = 0;
-	
+
 	ASSERT(results.size());
-	m_sName        = results[0].Name;
+	if (sFilename.empty())
+		m_sName    = results[0].Name;
+	else
+		m_sName    = sFilename;
 	m_dwFileLength = results[0].Size;
 	// sort results vector so that we start connection to the fastest hosts
 	vector<const Result*> res_sorted;
@@ -1347,6 +1410,11 @@ MGnuDownload::MGnuDownload(MGnuDirector* pDir, const ResultVec& results) : m_mut
 		if ( m_vecDownloaders.size() >= MAX_DOWNLOADERS)
 			break;
 	}
+	//
+	m_nFileHandle = nFileHandle;
+	//
+	m_bExactSha1Match = false;
+	//
 	m_pFileMng = new MDownFileMng(this, m_pPrefs);
 	ASSERT(m_pFileMng);
 }
@@ -1371,6 +1439,10 @@ void MGnuDownload::Start()
 	MLock lock(m_mutex);
 	SetStatus(TRANSFER_OPENING);
 	m_pFileMng->RequestOpen(m_sName, m_dwFileLength);
+	
+	lock.unlock();
+	ASSERT(m_pDirector->GetMarkedFiles());
+	m_pDirector->GetMarkedFiles()->AddFileFlag(m_dwFileLength, m_nFileHandle, MFT_InDownload);
 }
 
 void MGnuDownload::Stop(bool bDeleteFile, bool bDeleteSearch)
@@ -1386,6 +1458,10 @@ void MGnuDownload::Stop(bool bDeleteFile, bool bDeleteSearch)
 	//if (bDeleteSearch && m_dwSearchID)
 	//	m_pDirector->RemoveSearchByID(m_dwSearchID);
 	m_bDelSearch = bDeleteSearch;
+	
+	lock.unlock();
+	ASSERT(m_pDirector->GetMarkedFiles());
+	m_pDirector->GetMarkedFiles()->RemoveFile(m_dwFileLength, m_nFileHandle, MFT_InDownload);
 }
 
 void MGnuDownload::ForceDisconnectAll()
@@ -1412,6 +1488,26 @@ int MGnuDownload::CountActiveConnections(const HOST* pHost)
 	return nCount;
 }
 
+int MGnuDownload::CountReceivingConnections(const HOST* pHost)
+{
+	MLock lock(m_mutex);
+	int nCount = 0;
+	for (vector<MDownloader*>::iterator itd = m_vecDownloaders.begin(); itd != m_vecDownloaders.end(); ++itd)
+		if (((*itd)->GetStatus()==TRANSFER_RECEIVING) &&
+			(pHost==NULL ||
+			((*itd)->GetResult().Host==pHost->ip      &&
+			 (*itd)->GetResult().Port==pHost->port    )))
+			++nCount;
+	return nCount;
+}
+
+void MGnuDownload::SetRatePerSocket(long nKbytesPerSec)
+{
+	MLock lock(m_mutex);
+	for (vector<MDownloader*>::iterator itd = m_vecDownloaders.begin(); itd != m_vecDownloaders.end(); ++itd)
+		(*itd)->SetMaxRate(nKbytesPerSec);
+}
+
 void MGnuDownload::OnTimer(int nTimeStamp)
 {
 	MLock lock(m_mutex);
@@ -1445,6 +1541,9 @@ void MGnuDownload::OnTimer(int nTimeStamp)
 			#warning status will be lost here
 			SetStatus(TRANSFER_CLOSING);
 			m_pFileMng->RequestClose();
+			// unregister in the known files DB
+			ASSERT(m_pDirector->GetMarkedFiles());
+			m_pDirector->GetMarkedFiles()->RemoveFile(m_dwFileLength, m_nFileHandle, MFT_InDownload);
 			return;
 		}
 	}
@@ -1482,8 +1581,7 @@ int  MGnuDownload::LookUp(IP ipHost, const CString& sFileName, int nIndex)
 	{
 		if ((*itd)->GetResult().Host == ipHost      &&
 			(*itd)->GetResult().FileIndex == nIndex &&
-			((*itd)->GetResult().NameLower == sFileName ||
-			 (*itd)->GetResult().Name == sFileName      ))
+			0==CompareNoCase((*itd)->GetResult().Name, sFileName))
 			return i;
 		++i;
 	}
@@ -1643,10 +1741,7 @@ bool MGnuDownload::Rename(const CString& sNewName)
 		NULL != (pSearch = m_pDirector->GetSearchByID(m_dwSearchID)) )
 	{
 		CString sSearch = MakeSearchOfFilename(m_sName);
-		pSearch->m_mutex.lock();
-		pSearch->SetSearchString(sSearch, false);
-		pSearch->m_Filename = m_sName;
-		pSearch->m_mutex.unlock();
+		pSearch->SetSearchStringAndFilename(sSearch, m_pFileMng->GetName());
 	}
 	return true;
 }
@@ -1668,14 +1763,14 @@ bool MGnuDownload::AddSearch(bool bAutoget)
 		//
 		if (pSearch)
 		{
-			MLock lock(pSearch->m_mutex);
-			pSearch->m_Filename = m_sName;
-			m_dwSearchID = pSearch->m_dwID;
+			pSearch->SetFilename(m_pFileMng->GetName());
+			pSearch->SetFileHandle(m_nFileHandle);
+			m_dwSearchID = pSearch->GetID();
 		}
 	}
 	if (bAutoget && pSearch)
 	{
-		pSearch->m_bAutoget = true;
+		pSearch->SetAutoget(true);
 		// clear the search for any case
 		pSearch->Clear();
 	}
@@ -1690,22 +1785,22 @@ void MGnuDownload::UpdateResults()
 	{
 		// first we have to look up the appropriate search
 		MGnuSearch* pSearch = pSearch = m_pDirector->GetSearchByID(m_dwSearchID);
-		if (!pSearch || !pSearch->m_bUpdated)
+		if (!pSearch || !pSearch->IsUpdated())
 		{
 			if (!pSearch)
 				AddSearch(false);
 			return;
 		}
-		// lock search mutex for any case
-		MLock lock(pSearch->m_mutex);
 		// now we have to match and update
 		// but first reset the 'updated' flag on the search
-		pSearch->m_bUpdated = false;
+		pSearch->SetUpdated(false);
+		vector<Result> res;
+		pSearch->GetResults(res);
 		// sort results vector so that we start updating from the fastest hosts
 		vector<Result*> res_sorted;
-		res_sorted.reserve(pSearch->m_mapResults.size());
-		for (map<DWORD,Result>::iterator it = pSearch->m_mapResults.begin(); it != pSearch->m_mapResults.end(); ++it)
-			res_sorted.push_back(&(it->second));
+		res_sorted.reserve(res.size());
+		for (vector<Result>::iterator it = res.begin(); it != res.end(); ++it)
+			res_sorted.push_back(&*it);
 		sort(res_sorted.begin(), res_sorted.end(), bySpeed);
 		//
 		for (vector<Result*>::iterator its = res_sorted.begin(); its != res_sorted.end(); ++its)
@@ -1714,10 +1809,10 @@ void MGnuDownload::UpdateResults()
 			bool bFound = false;
 			for (vector<MDownloader*>::iterator itd = m_vecDownloaders.begin(); itd != m_vecDownloaders.end(); ++itd)
 			{
-				if (SR.Port        == (*itd)->GetResult().Port        &&
-					SR.FileIndex   == (*itd)->GetResult().FileIndex   &&
-					SR.Host.S_addr == (*itd)->GetResult().Host.S_addr &&
-					SR.NameLower   == (*itd)->GetResult().NameLower   )
+				if (SR.Port        == (*itd)->GetResult().Port            &&
+					SR.FileIndex   == (*itd)->GetResult().FileIndex       &&
+					SR.Host.S_addr == (*itd)->GetResult().Host.S_addr     &&
+					0 == CompareNoCase(SR.Name, (*itd)->GetResult().Name) )
 				{
 					// update current result
 					//TRACE("UpdateResults: updating existing");
@@ -1733,10 +1828,10 @@ void MGnuDownload::UpdateResults()
 				//
 				for (vector<Result>::iterator itBR = m_vecBadResults.begin(); itBR != m_vecBadResults.end(); ++itBR )
 				{
-					if (SR.Port        == itBR->Port        &&
-					    SR.FileIndex   == itBR->FileIndex   &&
-					    SR.Host.S_addr == itBR->Host.S_addr &&
-					    SR.NameLower   == itBR->NameLower   )
+					if (SR.Port        == itBR->Port            &&
+					    SR.FileIndex   == itBR->FileIndex       &&
+					    SR.Host.S_addr == itBR->Host.S_addr     &&
+					    0 == CompareNoCase(SR.Name, itBR->Name) )
 					{
 						// check if it was updated
 						if (itBR->ChangeTime != SR.ChangeTime)
@@ -1818,10 +1913,14 @@ void MGnuDownload::OnDataWritten(MDownloader* pDownloader, DWORD dwPos, DWORD dw
 		if (*itd == pDownloader)
 		{
 			// if it's the 'first success' than we should add the Sha1 to the search
-			const CString& sha1 = pDownloader->GetResult().Sha1;
-			if (sha1.length() == 32 && m_setSha1.find(sha1) == m_setSha1.end())
+			// NO! it might be just few bytes from the failed downloader, which just by chance overlapped!
+			// that's why we need this addititonal check for status
+			const SHA1Hash& sha1 = pDownloader->GetResult().Sha1;
+			if (pDownloader->GetStatus()==TRANSFER_RECEIVING &&
+				sha1.isValid()                               &&
+				m_setSha1.find(sha1) == m_setSha1.end()      )
 			{
-				// TODO: modify the file name or somehow make this set persistent
+				// the set is de facto persistent via search
 				m_setSha1.insert(sha1);
 				// modfy the auto-search
 				MGnuSearch* pSearch = NULL;
@@ -1830,6 +1929,9 @@ void MGnuDownload::OnDataWritten(MDownloader* pDownloader, DWORD dwPos, DWORD dw
 				{
 					pSearch->AddSha1Hash(sha1);
 				}
+				// register this SHA1 in the known files database
+				m_pDirector->GetMarkedFiles()->AddFile(MFT_Attempted, m_dwFileLength, sha1, pSearch ? pSearch->GetSearchString() : "", m_nFileHandle);
+				m_pDirector->GetMarkedFiles()->AddFileFlag(m_dwFileLength, m_nFileHandle, MFT_InDownload);
 			}
 			pDownloader->OnDataWritten(dwPos, dwSize, pB);
 			return;
@@ -1894,13 +1996,17 @@ void MGnuDownload::OnFileCompleted()
 		MSG_DOWNLOAD_COMPLETED,
 		MSGSRC_DOWNLOAD
 	));
+	
+	lock.unlock();
+	ASSERT(m_pDirector->GetMarkedFiles());
+	m_pDirector->GetMarkedFiles()->RemoveFile(m_dwFileLength, m_nFileHandle, MFT_Attempted|MFT_InDownload);
 }
 
 void MGnuDownload::OnSegmentCompleted(MDownloader* pDownloader, bool bSuccess)
 {
 	MLock lock(m_mutex);
 	#warning MGnuDownload::OnSegmentCompleted(MDownloader* pDownloader, bool bSuccess) is not implemented
-	assert(0);
+	ASSERT(0);
 	// Downloader should stop here
 	// shell we give it a new task?
 }
