@@ -59,7 +59,7 @@ private:
 class MGnuDownload : protected SGnuDownload
 {
 public:
-	MGnuDownload(MGnuDirector*, const ResultVec& results);
+	MGnuDownload(MGnuDirector*, const ResultVec& results, const CString& sFilename, long nFileHandle);
 	virtual ~MGnuDownload();
 	MGnuDirector* GetDirector(){return m_pDirector;}
 	
@@ -69,6 +69,7 @@ public:
 	
 	int     LookUp(IP ipHost, const CString& sFileName, int nIndex); // searches the results list and returns the index or -1 if fails
 	int     CountActiveConnections(const HOST* pHost);
+	int     CountReceivingConnections(const HOST* pHost);
 	int     GetNumAlternatives();
 	bool    IsPartOf(const CString& path);// returns true if the path points to the partial file of the download
 	
@@ -93,6 +94,10 @@ public:
 	bool    ShouldDeleteSearch(){return m_bDelSearch;} // internaly used by Director
 	
 	bool    OnIncomingPush(const CString& sFileName, int nIndex, SOCKET hPushSock); // returns 'true' if the push was accepted
+
+	// rate limiting
+	void SetRatePerSocket(long nKbytesPerSec);
+	
 	// internal use functions
 	bool RequestStartPoint(MDownloader* pDownloader); // if returns false caller should close and be ready to go home
 	StoreResult StoreDataAt(MDownloader* pDownloader, int nPos, MDownloadBuffer* pBuff, int nSize); // if returns not SR_Success caller should close and be ready to go home
@@ -118,7 +123,10 @@ protected:
 	//
 	vector<Result> m_vecBadResults;
 	// good sha1-s, which passed the overlap test
-	set<CString> m_setSha1;
+	set<SHA1Hash> m_setSha1;
+	bool m_bExactSha1Match;
+	// KnownFile database support
+	long m_nFileHandle;
 	//
 	DWORD m_dwSearchID;
 	bool  m_bDelSearch;
