@@ -39,6 +39,7 @@
 struct packet_Header;
 struct packet_Ping;
 struct packet_Pong;
+struct packet_Bye;
 struct packet_Push;
 struct packet_Query;
 struct packet_QueryHit;
@@ -63,8 +64,10 @@ struct RecentQuery
 	}
 };
 
-struct PriorityPacket
+class PriorityPacket
 {
+friend class MGnuNode;
+public:
 	PriorityPacket(void* pData, int nLen) : Length(nLen) {
 		Packet = new BYTE[nLen];
 		memcpy(Packet, pData, nLen);
@@ -73,16 +76,34 @@ struct PriorityPacket
 		if (Packet)
 			delete [] Packet;
 	}
-	PriorityPacket(const PriorityPacket& orig) : Packet(orig.Packet), Length(orig.Length) {
+	PriorityPacket(const PriorityPacket& orig) {
+		orig.mutex.lock();
+		Packet = orig.Packet;
+		Length = orig.Length;
 		const_cast<PriorityPacket*>(&orig)->Packet = NULL;
 		const_cast<PriorityPacket*>(&orig)->Length = 0;
+		orig.mutex.unlock();
+	}
+	const PriorityPacket& operator=(const PriorityPacket& orig) {
+		mutex.lock();
+		orig.mutex.lock();
+		if (Packet)
+			delete Packet;
+		Packet = orig.Packet;
+		Length = orig.Length;
+		const_cast<PriorityPacket*>(&orig)->Packet = NULL;
+		const_cast<PriorityPacket*>(&orig)->Length = 0;
+		orig.mutex.unlock();
+		mutex.unlock();
+		return *this;
 	}
 	//
-	int   Length;
-	BYTE* Packet;
-	private:
+protected:
+	int    Length;
+	BYTE*  Packet;
+	mutable MMutex mutex;
+private:
 	PriorityPacket();
-	const PriorityPacket& operator=(const PriorityPacket&);
 };
 
 enum SendPacket {
@@ -252,7 +273,7 @@ public:
 
 protected:
 	// Packet handlers
-	DWORD SplitBundle(BYTE*, DWORD);
+	bool SplitBundle(BYTE*, DWORD, DWORD*);
 	void  HandlePacket(packet_Header*, DWORD);
 	bool  InspectPacket(packet_Header*);
 
@@ -260,6 +281,7 @@ protected:
 	void Receive_Ping(packet_Ping*,   int);
 	void Receive_Pong(packet_Pong*,   int);
 	void Receive_Push(packet_Push*,   int);
+	void Receive_Bye (packet_Bye*,    int);
 	void Receive_Query(packet_Query*, int);
 	void Receive_QueryHit(packet_QueryHit*, DWORD);
 
