@@ -43,6 +43,7 @@
 #include "preferences.h"
 #include "gnuwordhash.h"
 #include "sha1thread.h"
+#include "gnumarkedfiles.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -102,6 +103,8 @@ public:
 		m_pWordHash = m_pShare->m_pDirector->GetWordTable();
 		ASSERT(m_pWordHash);
 
+		m_pKnownFiles = m_pShare->m_pDirector->GetMarkedFiles();
+
 		m_dwTotalFiles = 0;
 		m_dwTotalSize  = 0;
 		ED().AddReceiver(&m_EventQueue);
@@ -138,6 +141,8 @@ private:
 
 	MGnuShare*    m_pShare;
 	MGnuWordHash* m_pWordHash;
+
+	MGnuMarkedFiles* m_pKnownFiles;
 	
 	MShareThread();
 	void run();
@@ -217,7 +222,7 @@ void MGnuShare::Rescan()
 void MGnuShare::ResetDirectories(DWORD &EventCount, LPHANDLE EventList)
 {
 	// Close wait events on all directories
-	assert(0);
+	ASSERT(0);
 	/*for(int i = 1; i < EventCount; i++)
 		FindCloseChangeNotification( EventList[i]);
 
@@ -394,9 +399,20 @@ int MShareThread::GetIndexBySha1(const CString& sSha1)
 	MLock FilesAccess(m_filesmutex);
 	for ( ; Indexes.size(); Indexes.pop_front())
 	{
-		ASSERT(m_SharedFiles[Indexes.front()].Sha1Hash == sSha1);
-		if (!m_SharedFiles[Indexes.front()].Hidden)
-			return Indexes.front();
+		if (m_SharedFiles[Indexes.front()].Sha1Hash == sSha1)
+		{
+			if (!m_SharedFiles[Indexes.front()].Hidden)
+				return Indexes.front();
+		}
+#ifdef _DEBUG
+		else
+		{
+			TRACE("MShareThread::GetIndexBySha1 : Index list contains wrong entry");
+			TRACE2("    index = ", Indexes.front());
+			TRACE2("    sSha1 = ", sSha1);
+			TRACE2("    .Sha1Hash = ", m_SharedFiles[Indexes.front()].Sha1Hash);
+		}
+#endif
 	}
 	return -1;
 }
@@ -438,6 +454,9 @@ void MShareThread::run()
 	ASSERT(m_pShare);
 	MGnuDirector*  pComm  = m_pShare->m_pDirector;;
 	ASSERT(pComm);
+
+	MGnuPreferences* pPrefs = pComm->GetPrefs();
+	ASSERT(pPrefs);
 	
 	BYTE* QueryReply = new BYTE[65000]; // 1000 bytes extra for overrun
 	ASSERT(QueryReply);
@@ -472,7 +491,8 @@ void MShareThread::run()
 	
 	for(;;)
 	{
-		m_EventQueue.Poll();
+		if (!m_EventQueue.Poll())
+			continue;
 		// now we do have an event!
 		TSmartPtr<MEvent> spEvent = m_EventQueue.SafeFront();
 		m_EventQueue.Pop();
@@ -509,7 +529,7 @@ void MShareThread::run()
 			}
 			break;
 			case MSG_DOWNLOAD_FILE_MOVED:
-			if (pComm->GetPrefs()->m_bShareDownloadDir)
+			if (pPrefs->m_bShareDownloadDir)
 			{
 				MEvFileMoved* pEFM = (MEvFileMoved*) (MEvent*) spEvent;
 				CString sPath = pEFM->GetValue();
@@ -556,6 +576,11 @@ void MShareThread::run()
 				{
 					m_SharedFiles[pEHR->m_nShareIndex].Sha1Hash = pEHR->m_sSha1;
 					m_pWordHash->InsertString("sha1:" + pEHR->m_sSha1, pEHR->m_nShareIndex, false);
+					// add file to "known" database
+					ASSERT(m_pKnownFiles);
+					SHA1Hash sha1;
+					sha1.fromStr(pEHR->m_sSha1);
+					m_pKnownFiles->AddFile(MFT_LibraryActual, m_SharedFiles[pEHR->m_nShareIndex].Size, sha1, MakeSearchOfFilename(m_SharedFiles[pEHR->m_nShareIndex].Name));
 				}
 				m_filesmutex.unlock();
 			}
@@ -571,6 +596,13 @@ void MShareThread::run()
                 // A four space query is an index query
                 bIndexingQuery = ( strcmp(SearchQuery.Text, "    ") == 0 && SearchQuery.nHops == 1);
 
+				if (pPrefs->m_bSendOnlyAvail   &&
+					!bIndexingQuery            &&
+					pPrefs->m_nMaxUploads >= 0 &&
+					pComm->CountUploads() >= pPrefs->m_nMaxUploads)
+						break;
+				
+
                 if (!bIndexingQuery)
                 {
 					// Look up query in shared files and remote indexes
@@ -583,7 +615,7 @@ void MShareThread::run()
                 QueryReplyLength        = 0;
                 TotalReplyCount         = 0;
                 ReplyCount              = 0;
-                MaxReplies              = pComm->GetPrefs()->m_nMaxReplies;
+                MaxReplies              = pPrefs->m_nMaxReplies;
                 MetaTail                = "";
 
                 m_filesmutex.lock();
@@ -707,8 +739,8 @@ void MShareThread::run()
 						// if matched -- add to search reply
 						if(MatchWordList((*itFile).NameLower, QWords))
 						{
-							if(pComm->GetPrefs()->m_nMaxReplies >= 0)
-								if(pComm->GetPrefs()->m_nMaxReplies <= ReplyCount)	
+							if(pPrefs->m_nMaxReplies >= 0)
+								if(pPrefs->m_nMaxReplies <= ReplyCount)	
 									break;
 							if(QueryReplyLength > 64000)
 								break;
