@@ -310,8 +310,11 @@ void MGnuSock::ParseHandshake(char* pBuffRest, DWORD dwRestSize)
 				Http200 += "<BODY>\r\n";
 				Http200 += "<H1>Gnutella</H1>\r\n";
 				Http200 += "This is not a web server, it's a Gnutella-network node.\r\n";
+				Http200 += "<br><smaller>Requested URL: " + sPath + "</smaller>\r\n";
 				Http200 += "<HR NOSHADE SIZE=1>\r\n";
-				Http200 += "Powered by <I><B><A HREF=\"http://mutella.sourceforge.net\">Mutella</A></B></I>.\r\n";
+				Http200 += "Powered by <B><A HREF=\"http://mutella.sourceforge.net\">Mutella ";
+				Http200 += VERSION;
+				Http200 += "</A></B>.\r\n";
 				Http200 += "</BODY></HTML>\r\n\r\n";
 		
 		Send(Http200.c_str(), Http200.length());
