@@ -55,12 +55,12 @@ class MUploadFile : public MAsyncFile
 {
 public:
 	MUploadFile(MGnuUpload* pUpload);
+	virtual ~MUploadFile();
 	virtual void OnSuccessDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState);
 	virtual void OnErrorDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState);
+	void DetachUpload();
 protected:
 	MGnuUpload* m_pBuddy;
-private:
-	virtual ~MUploadFile();
 };
 
 class MUploadBuffer : public MAFBufferTrivial
@@ -98,6 +98,11 @@ MUploadFile::~MUploadFile()
 {
 }
 
+void MUploadFile::DetachUpload()
+{
+	m_pBuddy = NULL;
+}
+
 void MUploadFile::OnSuccessDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState)
 {
 	switch (nReqType)
@@ -133,24 +138,12 @@ MGnuUpload::MGnuUpload(MGnuDirector* pComm)
 	m_pShare = m_pDirector->GetShare();
 	m_pPrefs = m_pDirector->GetPrefs();
 
-	m_sFileName = "Unknown";
 	m_sUserAgent = "Unknown";
-	m_nFileLength = 0;
-	m_nRequestedRange = -1;
-	m_pAFile = new MUploadFile(this);
-	ASSERT(m_pAFile);
-	m_pBuffer = new MUploadBuffer(4096); // 4k
-	m_pBuffer->Release(); // SmartPtr takes responsibility
+	
 
-	m_nBytesCompleted = 0;
-	m_nFileIndex = -1;
-	m_nFileIndexPush = -1;
+	m_pAFile = NULL;
 
-	m_bUriRequest = false;
-
-	m_nStartOffset = 0;
-	m_nStartTime = xtime();
-	m_bHttpOK = false; // we have not yet accepted the transfer
+	m_bKeepAlive = false;
 
 	// Bandwidth
 	for(int i = 0; i < 60; i++)
@@ -173,28 +166,27 @@ MGnuUpload::MGnuUpload(MGnuDirector* pComm)
 	m_nSecNum = 0;
 	m_nSecPos = 0;
 
+	m_nRequestedRange = -1;
+
 	//
 	m_dwID = 0;
-
-	m_nError = UPLERR_NONE;
-	StatusUpdate(TRANSFER_CONNECTING);
 }
 
-MGnuUpload::~MGnuUpload()
-{
-	//
-	m_mutex.lock();
-	
-	ForceDisconnect();
-	
-	if( m_pAFile )
+void MGnuUpload::ResetState(bool bLock)
+{             
+	MLock lock(m_mutex, bLock);
+	// close the file
+	if (m_pAFile)
 	{
+		m_pAFile->DetachUpload();
 		m_pAFile->Close();
 		delete m_pAFile;
 		m_pAFile = NULL;
 	}
-	m_mutex.unlock();
+	// release the buffer
+	m_pBuffer=NULL;
 
+	// for the record...
 	if (m_bHttpOK)
 	{
 		// this message is only needed for logging
@@ -218,6 +210,42 @@ MGnuUpload::~MGnuUpload()
 			MSGSRC_UPLOAD
 		));
 	}
+
+	// indeed reset the state
+	if (m_sFileName.empty())
+		m_sFileName = "**not yet known**";
+	else
+		m_sFileName += " **completed**";
+	m_nFileLength = 0;
+	m_nRequestedRange = -1;
+
+	m_nBytesCompleted = 0;
+	m_nFileIndex = -1;
+	m_nFileIndexPush = -1;
+
+	m_bUriRequest = false;
+
+	m_nStartOffset = 0;
+	m_nStartTime = xtime();
+	m_bHttpOK = false; // we have not yet accepted the transfer
+
+	m_nSecsDead       = 0; // reset it here as well
+
+	m_sHandshake.clear();
+	m_sTmpHandshake.clear();
+
+	m_nError = UPLERR_NONE;
+	StatusUpdate(TRANSFER_CONNECTING);
+}
+
+MGnuUpload::~MGnuUpload()
+{
+	//
+	m_mutex.lock();
+	ForceDisconnect();
+
+	ResetState(false);
+	m_mutex.unlock();
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -287,7 +315,7 @@ void MGnuUpload::OnReceive(int nErrorCode)
 	//TRACE("MGnuUpload::OnReceive");
 	MLock lock(m_mutex);
 	
-	BYTE* pBuff = new BYTE[6000];
+	BYTE* pBuff = new BYTE[4096];
 	ASSERT(pBuff);
 	DWORD dwBuffLength = Receive(pBuff, 4096);
 	
@@ -305,7 +333,8 @@ void MGnuUpload::OnReceive(int nErrorCode)
 		return;
 	}
 	
-	if (m_nStatus != TRANSFER_PUSH_CONNECTED)
+	if (m_nStatus != TRANSFER_PUSH_CONNECTED &&
+		(m_nStatus != TRANSFER_CONNECTING || !m_bKeepAlive ))
 	{
 		delete [] pBuff;
 		MAsyncSocket::OnReceive(nErrorCode);
@@ -315,14 +344,13 @@ void MGnuUpload::OnReceive(int nErrorCode)
 	pBuff[dwBuffLength] = 0;
 	CString Header((char*)pBuff);
 	m_sHandshake += Header;
+	m_sTmpHandshake += Header;
 
 	// New Upload
 	if(Header.find("\r\n\r\n") >= 0 )
 	{
-		CString Handshake = Header;
-
-		if( Handshake.find("GET /get/") == 0                  ||
-			Handshake.find("GET /uri-res/N2R?urn:sha1:") == 0 )
+		if( m_sTmpHandshake.find("GET /get/") == 0                  ||
+			m_sTmpHandshake.find("GET /uri-res/N2R?urn:sha1:") == 0 )
 		{
 			// Get Node info
 			CString Host;
@@ -333,7 +361,8 @@ void MGnuUpload::OnReceive(int nErrorCode)
 			Str2Ip(Host,m_ipHost);
 			m_nPort = nPort;
 			// "false" means that the mutex is locked
-			UploadFile(Handshake,false);
+			UploadFile(m_sTmpHandshake,false);
+			m_sTmpHandshake.clear();
 		}
 		else
 		{
@@ -367,6 +396,9 @@ void MGnuUpload::Send_HttpOK()
 
 	HttpOK += "Content-Length: " + DWrdtoStr(m_nRequestedRange - m_nBytesCompleted) + "\r\n";
 
+	if (m_bKeepAlive)
+		HttpOK += "Connection: Keep-Alive\r\n";
+
 	// X-Filename or X-Gnutella-Content-URN
 	if(m_bUriRequest)
 		HttpOK += "X-Filename: " + m_sFileName + "\r\n";
@@ -382,13 +414,13 @@ void MGnuUpload::Send_HttpOK()
 
 void MGnuUpload::Send_HttpBusy()
 {
-	CString Http503 =  "HTTP/1.0 503\r\n";
+	CString Http503 =  "HTTP/1.0 503 Server Is Busy\r\n";
 			Http503 += "Server: Mutella\r\n";
 			Http503 += "Content-type:text/html\r\n\r\n";
 			Http503 += "<HTML>\r\n";
-			Http503 += "<HEAD><TITLE>503 Server Busy</TITLE></HEAD>\r\n";
+			Http503 += "<HEAD><TITLE>503 Server Is Busy</TITLE></HEAD>\r\n";
 			Http503 += "<BODY>\r\n";
-			Http503 += "<H1>Server Busy</H1>\r\n";
+			Http503 += "<H1>Server Is Busy</H1>\r\n";
 			Http503 += "This server's upload max has been met, try again later.\r\n";
 			Http503 += "<HR NOSHADE SIZE=1>\r\n";
 			Http503 += "Powered by <I><B><A HREF=\"http://mutella.sourceforge.net\">Mutella</A></B></I>.\r\n";
@@ -404,7 +436,7 @@ void MGnuUpload::Send_HttpBusy()
 
 void MGnuUpload::Send_HttpNotFound()
 {
-	CString Http404 =  "HTTP/1.0 404\r\n";
+	CString Http404 =  "HTTP/1.0 404 Not Found\r\n";
 			Http404 += "Server: Mutella\r\n";
 			Http404 += "Content-type:text/html\r\n\r\n";
 			Http404 += "<HTML>\r\n";
@@ -473,8 +505,8 @@ void MGnuUpload::PushFile()
 		return;
 	}
 	
-	ASSERT(m_pAFile);
-	ASSERT(!m_pAFile->IsOpen());
+	ASSERT(m_pAFile==NULL);
+	m_pAFile = new MUploadFile(this);
 	m_pAFile->Open(m_sFilePath.c_str());
 }
 
@@ -491,18 +523,23 @@ void MGnuUpload::UploadFile(const CString& sHandshake, bool bLock)
 		Send_HttpNotFound();
 		return;
 	}
-	// If there's an upload max
-	if(m_pPrefs->m_nMaxUploads >= 0 && m_pDirector->CountUploads() >= m_pPrefs->m_nMaxUploads)
+	
+	// white list is a list to which upload slots are always given...
+	if (!m_pPrefs->m_ipnetsetWhiteList.IsIn(m_ipHost))
 	{
-		Send_HttpBusy();
-		return;
-	}
+		// If there's an upload max
+		if(m_pPrefs->m_nMaxUploads >= 0 && m_pDirector->CountUploads() >= m_pPrefs->m_nMaxUploads)
+		{
+			Send_HttpBusy();
+			return;
+		}
 
-	// If we are already uploading to this host, give a busy signal
-	if ( m_pPrefs->m_nMaxPerHostUploads > 0 && m_pDirector->CountUploads(m_ipHost, this) >= m_pPrefs->m_nMaxPerHostUploads )
-	{
-		Send_HttpBusy();
-		return;
+		// If we are already uploading to this host, give a busy signal
+		if ( m_pPrefs->m_nMaxPerHostUploads > 0 && m_pDirector->CountUploads(m_ipHost, this) >= m_pPrefs->m_nMaxPerHostUploads )
+		{
+			Send_HttpBusy();
+			return;
+		}
 	}
 
 	CString sLowHandshake = sHandshake;
@@ -526,7 +563,10 @@ void MGnuUpload::UploadFile(const CString& sHandshake, bool bLock)
 			End = FirstLine.length();
 		m_sFileName = FirstLine.substr(Begin, End - Begin);
 		// Web browser compatibility
-		ReplaceSubStr(m_sFileName,"%20", " ");
+		m_sFileName = restore_string(m_sFileName, false);
+		//ReplaceSubStr(m_sFileName,"%28", "(");
+		//ReplaceSubStr(m_sFileName,"%29", ")");
+		//ReplaceSubStr(m_sFileName,"%20", " ");
 	}
 	else if (0 == strncmp(sHandshake.c_str(),"GET /uri-res/N2R?urn:sha1:",26))
 	{
@@ -580,15 +620,44 @@ void MGnuUpload::UploadFile(const CString& sHandshake, bool bLock)
 	CString sRange = find_header("Range",sHandshake,sLowHandshake);
 	if(!sRange.empty())
 	{
+		TRACE2("Range header : ", sRange);
 		sRange.make_lower();
-		sscanf( sRange.c_str(), "bytes=%ld-%ld", &m_nBytesCompleted, &m_nRequestedRange);
-		if (m_nRequestedRange>=0)
-			++m_nRequestedRange; // FIX: in HTTP/1.1 ranges are inclusive!
+		int nDashPos = sRange.find('-');
+		CString sRangeEnd;
+		if(nDashPos>0)
+		{
+			sRangeEnd = StripWhite(sRange.substr(nDashPos+1));
+			sRange.cut(nDashPos);
+			if (!sRangeEnd.empty())
+			{
+				if(0 >= sscanf(sRangeEnd.c_str(), "%ld", &m_nRequestedRange))
+				{
+                        		m_nError = UPLERR_BAD_REQUEST; // TODO: send http reply
+                        		StatusUpdate(TRANSFER_CLOSED);
+                        		Close();
+                        		return;
+                		}
+				if (m_nRequestedRange >= 0)
+                        		++m_nRequestedRange; // FIX: in HTTP/1.1 ranges are inclusive!
+			}
+		}
+		if (0 >= sscanf( sRange.c_str(), "bytes=%ld", &m_nBytesCompleted))
+		{
+			m_nError = UPLERR_BAD_REQUEST; // TODO: send http reply
+			StatusUpdate(TRANSFER_CLOSED);
+			Close();
+			return;
+		}
+		TRACE4("Detected range request : ", m_nBytesCompleted, " - ", m_nRequestedRange);
 	}
 	
+	// Connection: KeepAlive header
+	CString sConnection = find_header("Connection",sHandshake,sLowHandshake);
+	sConnection.make_lower();
+	if (sConnection.find("keep-alive") >= 0)
+		m_bKeepAlive = true;
 	// here compex part starts: if it is a push we have file already open, but it would make sence to verify if it is the right file.
-	ASSERT(m_pAFile);
-	if (m_nStatus != TRANSFER_PUSH_CONNECTING )
+	if (m_nStatus != TRANSFER_PUSH_CONNECTING && m_nStatus != TRANSFER_PUSH_CONNECTED)
 	{
 		// normal transfer
 		if(m_nFileIndex == -1 || !m_pShare->GetFileByIndex(m_nFileIndex, m_sFileName, m_sFilePath, m_sSha1.empty()))
@@ -597,13 +666,16 @@ void MGnuUpload::UploadFile(const CString& sHandshake, bool bLock)
 			Send_HttpNotFound();
 			return;
 		}
+		ASSERT(m_pAFile == NULL);
 		StatusUpdate(TRANSFER_CONNECTED);
+		m_pAFile = new MUploadFile(this);
 		m_pAFile->Open(m_sFilePath.c_str());
 		return;
 	}
 
 	// okm it's a push
 	// so the file is already open, lets check if it is the right one
+	ASSERT(m_pAFile);
 	if (m_nFileIndexPush != m_nFileIndex ||
 		!m_pAFile->IsOpen()              )
 	{
@@ -623,6 +695,9 @@ void MGnuUpload::OnFileOpen(bool bLock)
 	//TRACE("OnFileOpen");	
 	MLock lock(m_mutex, bLock);
 	ASSERT(m_mutex.locked());
+
+	if (!m_pAFile)
+		return;
 	
 	// called in two cases: push request and actual upload request.
 	// we keep file open after push request.
@@ -660,6 +735,10 @@ void MGnuUpload::OnFileOpen(bool bLock)
 	
 	Send_HttpOK();
 	m_bHttpOK = true; // we have accepted the transfer
+
+	// time to allocate the memory
+	m_pBuffer = new MUploadBuffer(4096); // 4k
+	m_pBuffer->Release(); // SmartPtr takes responsibility
 	
 	m_bBufferReady = false;
 	UpdateSelectFlags();
@@ -682,6 +761,9 @@ void MGnuUpload::OnFileRead(int Requested, int nRead, int nError)
 	
 	MLock lock(m_mutex);
 
+	if (!m_pAFile)
+		return;
+
     ASSERT(m_pBuffer->isEmpty());
     ASSERT(nRead > 0);
 	m_pBuffer->increment(nRead);
@@ -809,7 +891,9 @@ void MGnuUpload::BandwidthTimer()
 	else if(TRANSFER_COMPLETED == m_nStatus)
 	{
 		m_nSecsDead++;
-		if(m_nSecsDead > 15)
+		if(m_nSecsDead > 5 && m_bKeepAlive)
+			ResetState(false);
+		else if (m_nSecsDead > 15)
 		{
 			ForceDisconnect();
 			m_nStatus = TRANSFER_CLOSED;
