@@ -29,7 +29,7 @@
 class MGnuDirector;
 class MGnuShare;
 class MGnuPreferences;
-class MAsyncFile;
+class MUploadFile;
 class MUploadBuffer;
 
 #define UPLERR_NONE         0
@@ -66,11 +66,27 @@ public:
 	void ForceDisconnect();
 	void UpdateSelectFlags();
 	void SetMaxRate(double dMR){m_dMaxRate = dMR;}
+
+	void ResetState(bool bLock);
 public:
 	MMutex m_mutex;
 protected:
-	MAsyncFile* m_pAFile;
+	MUploadFile* m_pAFile;
 	TSmartPtr<MUploadBuffer> m_pBuffer;
+	//
+	int    m_nFileIndexPush;
+	CString m_sFilePath;
+	CString m_sSha1;
+	bool   m_bUriRequest;
+	//
+	int    m_nRequestedRange;
+	int    m_nStartOffset;
+	int    m_nStartTime;
+	bool   m_bHttpOK;
+	bool   m_bKeepAlive;
+
+	bool   m_bBelowRateLimit;
+	bool   m_bBufferReady;
 
 	// Bandwidth
 	DWORD  m_dwBytes60;      // Bytes received in past 60 seconds
@@ -79,24 +95,14 @@ protected:
 	int    m_nSecsUnderLimit;
 	int	   m_nSecsDead;
 	double m_dMaxRate;
-	//
-	int    m_nFileIndexPush;
-	CString m_sFilePath;
-	CString m_sSha1;
-	bool m_bUriRequest;
-
-	int    m_nRequestedRange;
-	
-	bool   m_bBelowRateLimit;
-	bool   m_bBufferReady;
+	DWORD  m_dwAvgBytes[60]; // Average bytes from last 60 seconds
+	int    m_nSecPos;        // Position of next element insert
 
 	//
-	int m_nStartOffset;
-	int m_nStartTime;
-	bool m_bHttpOK;
+	CString m_sHandshake;
+	CString m_sTmpHandshake;
 
-	MGnuDirector* m_pDirector;
-// Overrides
+	// Overrides
 public:
 	public:
 	virtual void OnClose(int nErrorCode);
@@ -115,15 +121,10 @@ protected:
 	MGnuUpload(const MGnuUpload& rSrc);  // no implementation
 	
 // Implementation
-protected:	
+protected:
+	MGnuDirector* m_pDirector;
 	MGnuShare*    m_pShare;
 	MGnuPreferences* m_pPrefs;
-	
-	// Bandwidth
-	DWORD m_dwAvgBytes[60]; // Average bytes from last 60 seconds
-	int	  m_nSecPos;        // Position of next element insert
-	//
-	CString m_sHandshake;
 };
 
 /////////////////////////////////////////////////////////////////////////////
