@@ -760,6 +760,34 @@ bool MIniFile::CreateSection(const char *_szSectionName)
   return(true);
 }
 
+/*-----EraseKey-----------------------------------------------------------
+ * Erases a key from the buffer. Returns true if the key existed and false
+   otherwise.
+ *------------------------------------------------------------------------*/
+bool MIniFile::EraseKey(const char *_szKey)
+{
+  char *sz, *szLine, szLineBuffer[MAX_LINE_LEN], szKeyBuffer[MAX_KEYxNAME_LEN];
+  int nCutStart;
+
+  ResetSection();
+
+  // Find the position of the key or the end of the section / file
+  do
+  {
+    nCutStart = m_nBufPos;
+    szLine = ReadLine(szLineBuffer);
+    sz = GetKeyFromLine(szLine, szKeyBuffer);
+  }
+  while (sz != NULL && strcmp(sz, _szKey) != 0);
+  int nCutEnd = m_nBufPos;
+
+  if (strcmp(sz, _szKey) == 0)
+  {
+    InsertStr("", nCutStart, nCutEnd);
+    return (true);
+  }
+  return (false);
+}
 
 /*-----WriteStr----------------------------------------------------------------
  * Writes a string to the buffer replacing the given key or creating a new
