@@ -223,11 +223,73 @@ void mutella_handle_sigchld(int s)
 } /*extern "C"*/
 
 MMutex g_LibcMutex;
-//MMutex g_GetHostByNameMutex;
+MMutex g_GetHostByNameMutex;
 
 // The program starts here
 int main(int argc, char *argv[])
 {
+	// analyse command-line options
+	bool bNoUI = false;
+	bool bFork = false;
+	CString sConfigFile;
+	if (argc > 1)
+	{
+		for (int i = 1; i < argc; ++i)
+		{
+			if (0==strcmp(argv[i], "-h") || 0==strcmp(argv[i], "--help"))
+			{
+				cout << endl;
+				cout << "Mutella : a command line and HTTP-based Gnutella client" << endl;
+				cout << "  options:" << endl;
+				cout << "  -n, --no-ui     Do not start libreadline-based interactive" << endl;
+				cout << "                  user interface. Note that 'termrc' script" << endl;
+				cout << "                  will not be executed." << endl;
+				cout << "  -d, --daemon    Start without interactive user interface" << endl;
+				cout << "                  and fork to the background. Note that 'termrc'" << endl;
+				cout << "                  script will not be executed." << endl;
+				cout << "  -c [file]," << endl;
+				cout << "  --config=[file] Use alternative configuration file." << endl;
+				cout << "  -h, --help      Print this message and exit." << endl;
+				cout << endl;
+				return EXIT_SUCCESS;
+			} else if (0==strcmp(argv[i], "-n") || 0==strcmp(argv[i], "--no-ui"))
+				bNoUI = true;
+			else if (0==strcmp(argv[i], "-d") || 0==strcmp(argv[i], "--daemon"))
+				bFork = bNoUI = true;
+			else if (0==strcmp(argv[i], "-c"))
+			{
+				if (i == argc - 1)
+				{
+					cout << "Syntax error with '-c' option";
+					return EXIT_FAILURE;
+				}
+				sConfigFile = argv[++i];
+			}
+			else if (0==strncmp(argv[i], "--config", 8))
+			{
+				// here we expect equal sign in this or next parameter
+				char * szRest = strchr(argv[i], '=');
+				if (szRest)
+					sConfigFile = StripWhite(szRest + 1);
+				if (sConfigFile.empty())
+				{
+					if (i == argc - 1)
+					{
+						cout << "Syntax error with '--config' option";
+						return EXIT_FAILURE;
+					}
+					sConfigFile = argv[++i];
+				}
+			}
+		}
+	}
+
+	// fork to background ...
+	if ( bFork && fork())
+	{
+		return EXIT_SUCCESS;
+	}
+
     // set ignore to the alarm clock
     struct sigaction sa;
     memset(&sa,0,sizeof(sa));
@@ -244,6 +306,7 @@ int main(int argc, char *argv[])
     //signal(SIGCHLD, &mutella_handle_sigchld);
     //signal(SIGSEGV, &libpublic_segv_handler);
     //
+
     // set the random seed to start with
     srand(xtime());
     //
@@ -251,7 +314,7 @@ int main(int argc, char *argv[])
     MAsyncFile::InitStaticStructures();
     //
     MController controller;
-	if (!controller.Init())
+	if (!controller.Init(sConfigFile, !bNoUI))
 		return EXIT_FAILURE;
 
 	// start the controller functionality
