@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: mthread_p.h,v 1.2 2001/12/16 20:10:24 maksik Exp $
+** $Id: mthread_p.h,v 1.10 2002/07/07 13:37:28 maksik Exp $
 **
 ** MThread class for Unix
 **
@@ -67,6 +67,13 @@
 #    define M_NORMAL_MUTEX_TYPE PTHREAD_MUTEX_DEFAULT
 #    define M_RECURSIVE_MUTEX_TYPE PTHREAD_MUTEX_RECURSIVE
 #  endif
+#elif defined (_OS_CYGWIN_)
+// CygWin
+#  define M_HAS_RECURSIVE_MUTEX
+#  undef  M_USE_PTHREAD_MUTEX_SETKIND
+#  define M_NORMAL_MUTEX_TYPE PTHREAD_MUTEX_DEFAULT
+#  define M_RECURSIVE_MUTEX_TYPE PTHREAD_MUTEX_RECURSIVE
+#  define M_MANUAL_LOCK_TRACKING
 #elif defined(_OS_OSF_)
 // Digital UNIX - 4.0 and later has a POSIX 1003.1c implementation
 //   - should we assume > 4.0?
@@ -92,6 +99,12 @@
 #  undef  M_USE_PTHREAD_MUTEX_SETKIND
 #  define M_NORMAL_MUTEX_TYPE PTHREAD_MUTEX_ERRORCHECK
 #  define M_RECURSIVE_MUTEX_TYPE PTHREAD_MUTEX_RECURSIVE
+#elif defined(_OS_DARWIN_)
+// Darwin in its current rev (xnu-201) has a pretty limited POSIX implementation
+#  undef  M_HAS_RECURSIVE_MUTEX
+#  undef  M_USE_PTHREAD_MUTEX_SETKIND
+#  undef  M_USE_PTHREAD_MUTEX_TYPE
+#  undef  M_RECURSIVE_MUTEX_TYPE
 #elif defined(_OS_SOLARIS_)
 // Solaris 2.7 and later - we use the native Solaris threads implementation
 #  undef  M_HAS_RECURSIVE_MUTEX
@@ -128,10 +141,10 @@ extern "C" { static void * start_thread(void * t); }
 // supported by Solaris 2.6 and better.
 // So we are trying to detect Solaris 2.5.1 using macro _XOPEN_UNIX which is
 // defined by <unistd.h> when XPG4v2 is supported.
-#if !defined(_XOPEN_UNIX)
-typedef unsigned int useconds_t;
-extern "C" int usleep(useconds_t);
-#endif
+//#if !defined(_XOPEN_UNIX)
+//typedef unsigned int useconds_t;
+//extern "C" int usleep(useconds_t);
+//#endif
 
 
 class MMutexPrivate {
@@ -147,7 +160,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: init failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: init failure: %s\n", strerror( ret ) );
 #endif
     }
 
@@ -160,7 +173,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if ( ret )
-	    fprintf(stderr, "MMutex::~MMutex: destroy failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::~MMutex: destroy failure: %s\n", strerror( ret ) );
 #endif
     }
 
@@ -173,7 +186,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if (ret)
-	    fprintf(stderr,"MMutex::lock: mutex lock failure: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::lock: mutex lock failure: %s\n", strerror(ret));
 #endif
     }
 
@@ -186,7 +199,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if (ret)
-	    fprintf(stderr,"MMutex::unlock: mutex unlock failure: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::unlock: mutex unlock failure: %s\n", strerror(ret));
 #endif
     }
 
@@ -198,7 +211,7 @@ public:
 	    return TRUE;
 	} else if (ret) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr,"MMutex::locked: try lock failed: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::locked: try lock failed: %s\n", strerror(ret));
 #endif
 	} else {
 	    mutex_unlock(&mutex);
@@ -231,7 +244,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: init failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: init failure: %s\n", strerror( ret ) );
 #endif
 
 	count = 0;
@@ -246,7 +259,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: destroy failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: destroy failure: %s\n", strerror( ret ) );
 #endif
     }
 
@@ -274,8 +287,8 @@ public:
 
 	if (owner != MThread::currentThread()) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr,"MMutex::unlock: unlock from different thread than locker");
-	    fprintf(stderr,"                was locked by %d, unlock attempt from %d",
+	    fprintf(stderr,"MMutex::unlock: unlock from different thread than locker\n");
+	    fprintf(stderr,"                was locked by %d, unlock attempt from %d\n",
 		     owner, MThread::currentThread());
 	    mutex_unlock(&mutex2);
 #endif
@@ -311,16 +324,26 @@ public:
 class MThreadPrivate {
 public:
     thread_t thread_id;
-    MWaitCondition thread_done;
-    bool finished, running;
+    MWaitCondition* thread_done;
+    bool finished, running, autodelete;
 
     MThreadPrivate()
 	: thread_id(0), finished(FALSE), running(FALSE)
     {
+	thread_done = NULL;
+	autodelete = false;
+    // this assumes that the first thread is created from
+    // yet single-threaded application
 	if (! dictMutex)
+	{
 	    dictMutex = new MMutex;
+	    ASSERT(dictMutex);
+	}
 	if (! thrDict)
+	{
 		thrDict = new MapHndlTrd;
+		ASSERT(thrDict);
+	}
     }
 
     ~MThreadPrivate()
@@ -337,6 +360,12 @@ public:
 		thread_id = 0;
     }
 
+    bool thread_wait(MMutex *mtx, unsigned long time){
+    	if (thread_done)
+    		return thread_done->wait(mtx, time);
+    	return true;
+    }
+
     void init(MThread *that)
     {
 	that->d->running = TRUE;
@@ -347,11 +376,11 @@ public:
 
 #ifdef CHECK_RANGE
 	if (ret)
-	    fprintf(stderr,"MThread::start: thread creation error: %s", strerror(ret));
+	    fprintf(stderr,"MThread::start: thread creation error: %s\n", strerror(ret));
 #endif
     }
 
-    static void internalRun(MThread *that)
+/*    static void internalRun(MThread *that)
     {
 	dictMutex->lock();
 	thrDict->insert(MapHndlTrd::value_type(MThread::currentThread(), that));
@@ -367,7 +396,7 @@ public:
     MapHndlTrd::iterator it = thrDict->find(MThread::currentThread());
 	if ( it != thrDict->end() ) {
         MThread *there = (*it).second;
-        assert(there);
+        ASSERT(there);
 	    there->d->running = FALSE;
 	    there->d->finished = TRUE;
 
@@ -377,6 +406,34 @@ public:
 	thrDict->erase(MThread::currentThread());
 	dictMutex->unlock();
 	}
+};*/
+
+	MapHndlTrd::iterator it = thrDict->find(MThread::currentThread());
+	if ( it != thrDict->end() ) {
+		MThread *there = (*it).second;
+		ASSERT(there);
+		there->d->running = FALSE;
+		there->d->finished = TRUE;
+		MWaitCondition* pWait = there->d->thread_done;
+		there->d->thread_done = NULL;
+		
+		if (there->d->autodelete)
+		{
+			thrDict->erase(it);
+			dictMutex->unlock();
+			// delete the object
+			delete there;
+			pWait->wakeAll();
+			delete pWait;
+			return;
+		}
+		pWait->wakeAll();
+		delete pWait;
+	}
+
+	thrDict->erase(MThread::currentThread());
+	dictMutex->unlock();
+    }
 };
 
 
@@ -384,9 +441,11 @@ class MWaitConditionPrivate {
 public:
     cond_t cond;
     MMutex mutex;
+    int nrefs;
 
     MWaitConditionPrivate()
     {
+    nrefs=1;
 #ifdef CHECK_RANGE
 	int ret =
 #endif
@@ -394,58 +453,82 @@ public:
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MWaitCondition::MWaitCondition: event init failure %s", strerror( ret ) );
+	    fprintf(stderr, "MWaitCondition::MWaitCondition: event init failure %s\n", strerror( ret ) );
 #endif
     }
-
+private:
     ~MWaitConditionPrivate()
     {
 	int ret = cond_destroy(&cond);
 	if( ret ) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr, "MWaitCondition::~MWaitCondition: event destroy failure %s", strerror( ret ) );
+	    fprintf(stderr, "MWaitCondition::~MWaitCondition: event destroy failure %s\n", strerror( ret ) );
 #endif
 
 	    // seems we have threads waiting on us, lets wake them up
 	    cond_broadcast(&cond);
 	}
     }
-
+public:
+	// references
+	inline void addref(){
+	mutex.lock();
+	++nrefs;
+	mutex.unlock();
+	}
+	inline void addref_lock(){
+	mutex.lock();
+	++nrefs;
+	}
+	inline void release(){
+	mutex.lock();
+	ASSERT(nrefs>0);
+	if (--nrefs<=0)
+	{
+		mutex.unlock();
+		delete this;
+		return;
+	}
+	mutex.unlock();
+	}
+	inline void release_unlock(){
+	ASSERT(nrefs>0);
+	if (--nrefs<=0)
+	{
+		mutex.unlock();
+		delete this;
+		return;
+	}
+	mutex.unlock();
+	}
+	// functionality
     void wakeOne()
     {
-	mutex.lock();
-
 #ifdef CHECK_RANGE
 	int ret =
 #endif
 	    cond_signal(&(cond));
 
 #ifdef CHECK_RANGE
-	if ( ret ) fprintf(stderr,"MWaitCondition::wakeOne: wake error: %s",strerror(ret));
+	if ( ret ) fprintf(stderr,"MWaitCondition::wakeOne: wake error: %s\n",strerror(ret));
 #endif
-
-	mutex.unlock();
     }
 
     void wakeAll()
     {
-	mutex.lock();
-
 #ifdef CHECK_RANGE
 	int ret =
 #endif
 	    cond_broadcast(& (cond) );
 
 #ifdef CHECK_RANGE
-	if( ret ) fprintf(stderr,"MWaitCondition::wakeAll: wake error: %s",strerror(ret));
+	if( ret ) fprintf(stderr,"MWaitCondition::wakeAll: wake error: %s\n",strerror(ret));
 #endif
-
-	mutex.unlock();
     }
 
     bool wait(unsigned long time)
     {
-	mutex.lock();
+	addref_lock();
 
 	int ret;
 	if (time != ULONG_MAX) {
@@ -458,10 +541,10 @@ public:
 	    ret = cond_wait ( &(cond), &(mutex.d->mutex) );
 	}
 
-	mutex.unlock();
+	release_unlock();
 
 #ifdef CHECK_RANGE
-	if( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s",strerror(ret));
+	if( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s\n",strerror(ret));
 #endif
 
 	return (ret == 0);
@@ -474,7 +557,7 @@ public:
 #ifdef CHECK_RANGE
 	if (mtx->d->type() == M_MUTEX_RECURSIVE)
 	    fprintf(stderr,"MWaitCondition::wait: warning - using recursive mutexes with\n"
-		     "                      conditions is undefined!");
+				       "                      conditions is undefined!\n");
 #endif
 
 #ifndef M_HAS_RECURSIVE_MUTEX
@@ -487,7 +570,7 @@ public:
 
 	    if (! rmp->count) {
 #  ifdef CHECK_RANGE
-		fprintf(stderr,"MWaitCondition::unlockAndWait: recursive mutex not locked!");
+		fprintf(stderr,"MWaitCondition::unlockAndWait: recursive mutex not locked!\n");
 #  endif
 
 		return FALSE;
@@ -503,6 +586,8 @@ public:
 	}
 #endif
 
+	adderf();
+
 	int ret;
 	if (time != ULONG_MAX) {
 	    timespec ti;
@@ -513,6 +598,8 @@ public:
 	} else {
 	    ret = cond_wait ( &(cond), &(mtx->d->mutex) );
 	}
+	
+	release();
 
 #ifndef M_HAS_RECURSIVE_MUTEX
 	if (mtx->d->type() == M_MUTEX_RECURSIVE) {
@@ -525,7 +612,7 @@ public:
 #endif
 
 #ifdef CHECK_RANGE
-	if ( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s",strerror(ret));
+	if ( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s\n",strerror(ret));
 #endif
 
 	return (ret == 0);
@@ -541,10 +628,9 @@ public:
 
 class MMutexPrivate {
 public:
-#if defined (_OS_SOLARIS_)
-    mutex_t mutex;
-#else
     pthread_mutex_t mutex;
+#if defined(M_MANUAL_LOCK_TRACKING)
+	int nLockCount;
 #endif
 
     MMutexPrivate(bool recursive = FALSE)
@@ -552,6 +638,10 @@ public:
 	pthread_mutexattr_t attr;
 	pthread_mutexattr_init(&attr);
 
+#if defined(M_MANUAL_LOCK_TRACKING)
+	nLockCount = 0;
+#endif
+
 #if defined(M_HAS_RECURSIVE_MUTEX)
 	if (recursive) {
 
@@ -587,20 +677,24 @@ public:
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: init failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: init failure: %s\n", strerror( ret ) );
 #endif
     }
 
     virtual ~MMutexPrivate()
     {
 #ifdef CHECK_RANGE
+#if defined(M_MANUAL_LOCK_TRACKING)
+	if (nLockCount)
+		fprintf(stderr, "MMutex::~MMutex: destroing locked mutex");
+#endif //M_MANUAL_LOCK_TRACKING
 	int ret =
 #endif
 	    pthread_mutex_destroy( &mutex );
 
 #ifdef CHECK_RANGE
 	if ( ret )
-	    fprintf(stderr, "MMutex::~MMutex: destroy failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::~MMutex: destroy failure: %s\n", strerror( ret ) );
 #endif
     }
 
@@ -611,14 +705,23 @@ public:
 #endif
 	    pthread_mutex_lock(&mutex);
 
+#if defined(M_MANUAL_LOCK_TRACKING)
+	++nLockCount;
+#endif //M_MANUAL_LOCK_TRACKING
+
 #ifdef CHECK_RANGE
 	if (ret)
-	    fprintf(stderr,"MMutex::lock: mutex lock failure: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::lock: mutex lock failure: %s\n", strerror(ret));
 #endif
     }
 
     virtual void unlock()
     {
+#if defined(M_MANUAL_LOCK_TRACKING)
+	--nLockCount;
+#endif //M_MANUAL_LOCK_TRACKING
+
+
 #ifdef CHECK_RANGE
 	int ret =
 #endif
@@ -626,7 +729,7 @@ public:
 
 #ifdef CHECK_RANGE
 	if (ret)
-	    fprintf(stderr,"MMutex::unlock: mutex unlock failure: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::unlock: mutex unlock failure: %s\n", strerror(ret));
 #endif
     }
 
@@ -634,17 +737,22 @@ public:
     {
 	int ret = pthread_mutex_trylock(&mutex);
 
-	if (ret == EBUSY) {
-	    return TRUE;
+	if (ret == EBUSY)
+	{
+		return TRUE;
 	} else if (ret) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr,"MMutex::locked: try lock failed: %s", strerror(ret));
+	    fprintf(stderr,"MMutex::locked: try lock failed: %s\n", strerror(ret));
 #endif
 	} else {
 	    pthread_mutex_unlock(&mutex);
 	}
 
+#if defined(M_MANUAL_LOCK_TRACKING)
+	return nLockCount;
+#else
 	return FALSE;
+#endif //M_MANUAL_LOCK_TRACKING
     }
 
 #if defined(CHECK_RANGE) || !defined(M_HAS_RECURSIVE_MUTEX)
@@ -670,7 +778,7 @@ public:
 
 #  ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: destroy failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: destroy failure: %s\n", strerror( ret ) );
 #  endif
 
     }
@@ -699,8 +807,8 @@ public:
 
 	if (owner != MThread::currentThread()) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr,"MMutex::unlock: unlock from different thread than locker");
-	    fprintf(stderr,"                was locked by %d, unlock attempt from %d",
+	    fprintf(stderr,"MMutex::unlock: unlock from different thread than locker\n");
+	    fprintf(stderr,"                was locked by %d, unlock attempt from %d\n",
 		     owner, MThread::currentThread());
 	    pthread_mutex_unlock(&mutex2);
 #endif
@@ -744,7 +852,7 @@ public:
 
 #  ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MMutex::MMutex: init failure: %s", strerror( ret ) );
+	    fprintf(stderr, "MMutex::MMutex: init failure: %s\n", strerror( ret ) );
 #  endif
 
 	count = 0;
@@ -760,16 +868,26 @@ public:
 class MThreadPrivate {
 public:
     pthread_t thread_id;
-    MWaitCondition thread_done;      // Used for MThread::wait()
-    bool finished, running;
+    MWaitCondition* thread_done;      // Used for MThread::wait()
+    bool finished, running, autodelete;
 
     MThreadPrivate()
 	: thread_id(0), finished(FALSE), running(FALSE)
     {
+	autodelete = false;
+	thread_done = NULL;
+    // this assumes that the first thread is created from
+    // yet single-threaded application
 	if (! dictMutex)
+	{
 		dictMutex = new MMutex;
+		ASSERT(dictMutex);
+	}
 	if (! thrDict)
+	{
 		thrDict = new MapHndlTrd;
+		ASSERT(thrDict);
+	}
     }
 
     ~MThreadPrivate()
@@ -786,6 +904,11 @@ public:
 		thread_id = 0;
     }
 
+    bool thread_wait(MMutex *mtx, unsigned long time){
+    	if (thread_done)
+    		return thread_done->wait(mtx, time);
+    	return true;
+    }
 
     void init(MThread *that)
     {
@@ -805,7 +928,7 @@ public:
     //cout << "I4" << flush;
 	if ( ret ) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr,"MThread::start: thread creation error: %s", strerror(ret));
+	    fprintf(stderr,"MThread::start: thread creation error: %s\n", strerror(ret));
 #endif
 		//cout << "I!" << flush;
 	    return;
@@ -818,37 +941,41 @@ public:
     //cout << "R" << flush;
 	dictMutex->lock();
 	thrDict->insert(MapHndlTrd::value_type(MThread::currentThread(), that));
+	// create "wait condition" object
+	ASSERT(that->d->thread_done == NULL);
+	that->d->thread_done = new MWaitCondition();
+	ASSERT(that->d->thread_done);
+	// if the thread calls "delete this" juct created MWaitCondition will join
+	// the army of memory leaks. Thread should call setSutoDelete() instead
+	// whereas I dont understand why cant I just remember this pinter and delete it
+	// in the end
 	dictMutex->unlock();
 	
 	that->run();
 
 	dictMutex->lock();
-
+	
 	MapHndlTrd::iterator it = thrDict->find(MThread::currentThread());
 	if ( it != thrDict->end() ) {
-        MThread *there = (*it).second;
-        assert(there);
-        MThreadPrivate* pd = there->d;
-		pd->running = FALSE;
-		pd->finished = TRUE;
+		MThread *there = (*it).second;
+		ASSERT(there);
+		there->d->running = FALSE;
+		there->d->finished = TRUE;
+		MWaitCondition* pWait = there->d->thread_done;
+		there->d->thread_done = NULL;
 		
-		if (!there->m_bAutoDelete)
+		if (there->d->autodelete)
 		{
-			//cout << "ND" << flush;
-			pd->thread_done.wakeAll();
-		}
-		else
-		{
-			// multistage safe delete
-			//cout << "AD" << flush;
+			thrDict->erase(it);
 			dictMutex->unlock();
-			pd->erase();
-			dictMutex->lock();
-			there->d = NULL;
+			// delete the object
 			delete there;
-			pd->thread_done.wakeAll();
-			delete pd;
+			pWait->wakeAll();
+			delete pWait;
+			return;
 		}
+		pWait->wakeAll();
+		delete pWait;
 	}
 
 	thrDict->erase(MThread::currentThread());
@@ -860,75 +987,113 @@ public:
 class MWaitConditionPrivate {
 public:
     pthread_cond_t cond;
-    MMutex mutex;
+    MMutex         mutex;
+    u_long         nrefs;
 
     MWaitConditionPrivate()
     {
+	nrefs = 1;
 	pthread_condattr_t cattr;
+#ifdef HAVE_PTHREAD_CONDATTR_INIT
 	pthread_condattr_init(&cattr);
+#else
+	memset(&cattr, 0, sizeof(cattr));
+#endif
 
 #ifdef CHECK_RANGE
 	int ret =
 #endif
 	    pthread_cond_init(&cond, &cattr);
 
+#ifdef HAVE_PTHREAD_CONDATTR_DESTROY
 	pthread_condattr_destroy(&cattr);
+#endif
 
 #ifdef CHECK_RANGE
 	if( ret )
-	    fprintf(stderr, "MWaitCondition::MWaitCondition: event init failure %s", strerror( ret ) );
+	    fprintf(stderr, "MWaitCondition::MWaitCondition: event init failure %s\n", strerror( ret ) );
 #endif
     }
 
+private:
     ~MWaitConditionPrivate()
     {
 	int ret = pthread_cond_destroy(&cond);
 	if( ret ) {
 #ifdef CHECK_RANGE
-	    fprintf(stderr, "MWaitCondition::~MWaitCondition: event destroy failure %s", strerror( ret ) );
+	    fprintf(stderr, "MWaitCondition::~MWaitCondition: event destroy failure %s\n", strerror( ret ) );
 #endif
 
 	    // seems we have threads waiting on us, lets wake them up
 	    pthread_cond_broadcast(&cond);
 	}
     }
-
+public:
+	// references
+	inline void addref(){
+	mutex.lock();
+	++nrefs;
+	mutex.unlock();
+	}
+	inline void addref_lock(){
+	mutex.lock();
+	++nrefs;
+	}
+	inline void release(){
+	mutex.lock();
+	ASSERT(nrefs>0);
+	if (--nrefs<=0)
+	{
+		mutex.unlock();
+		delete this;
+		return;
+	}
+	mutex.unlock();
+	}
+	inline void release_unlock(){
+	ASSERT(nrefs>0);
+	if (--nrefs<=0)
+	{
+		mutex.unlock();
+		delete this;
+		return;
+	}
+	mutex.unlock();
+	}
+	// functionality
     void wakeOne()
     {
-	mutex.lock();
-
 #ifdef CHECK_RANGE
 	int ret =
 #endif
 	    pthread_cond_signal( &(cond) );
 
 #ifdef CHECK_RANGE
-	if ( ret ) fprintf(stderr,"MWaitCondition::wakeOne: wake error: %s",strerror(ret));
+	if ( ret ) fprintf(stderr,"MWaitCondition::wakeOne: wake error: %s\n",strerror(ret));
 #endif
-
-	mutex.unlock();
     }
 
     void wakeAll()
     {
-	mutex.lock();
-
 #ifdef CHECK_RANGE
 	int ret =
 #endif
 	    pthread_cond_broadcast(&(cond));
 
 #ifdef CHECK_RANGE
-	if( ret ) fprintf(stderr,"MWaitCondition::wakeAll: wake error: %s",strerror(ret));
+	if( ret ) fprintf(stderr,"MWaitCondition::wakeAll: wake error: %s\n",strerror(ret));
 #endif
-
-	mutex.unlock();
     }
 
     bool wait(unsigned long time)
     {
-	mutex.lock();
-
+	addref_lock();
+	
+#if defined(M_MANUAL_LOCK_TRACKING)
+		// wait will unlock the mutex -- decrese the counter
+		--mutex.d->nLockCount;
+#endif
+	
 	int ret;
 	if (time != ULONG_MAX) {
 	    timespec ti;
@@ -940,10 +1105,14 @@ public:
 	    ret = pthread_cond_wait ( &(cond), &(mutex.d->mutex) );
 	}
 
-	mutex.unlock();
+#if defined(M_MANUAL_LOCK_TRACKING)
+		++mutex.d->nLockCount;
+#endif
+
+	release_unlock();
 
 #ifdef CHECK_RANGE
-	if( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s",strerror(ret));
+	if( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s\n",strerror(ret));
 #endif
 
 	return (ret == 0);
@@ -955,8 +1124,8 @@ public:
 
 #ifdef CHECK_RANGE
 	if (mtx->d->type() == M_MUTEX_RECURSIVE)
-	    fprintf(stderr,"MWaitCondition::unlockAndWait: warning - using recursive mutexes"
-		     " with\n                               conditions is undefined!");
+	    fprintf(stderr,"MWaitCondition::unlockAndWait: warning - using recursive mutexes with\n"
+				       "                               conditions is undefined!\n");
 #endif
 
 #ifndef M_HAS_RECURSIVE_MUTEX
@@ -969,7 +1138,7 @@ public:
 
 	    if (! rmp->count) {
 #  ifdef CHECK_RANGE
-		fprintf(stderr,"MWaitCondition::unlockAndWait: recursive mutex not locked!");
+		fprintf(stderr,"MWaitCondition::unlockAndWait: recursive mutex not locked!\n");
 #  endif
 
 		return FALSE;
@@ -985,6 +1154,13 @@ public:
 	}
 #endif
 
+	addref();
+
+#if defined(M_MANUAL_LOCK_TRACKING)
+	// wait will unlock the mutex -- decrese the counter
+	--mtx->d->nLockCount;
+#endif
+
 	int ret;
 	if (time != ULONG_MAX) {
 	    timespec ti;
@@ -996,6 +1172,13 @@ public:
 	    ret = pthread_cond_wait ( &(cond), &(mtx->d->mutex) );
 	}
 
+#if defined(M_MANUAL_LOCK_TRACKING)
+	// wait will unlock the mutex -- decrese the counter
+	++mtx->d->nLockCount;
+#endif
+
+	release();
+
 #ifndef M_HAS_RECURSIVE_MUTEX
 	if (mtx->d->type() == M_MUTEX_RECURSIVE) {
 	    MRMutexPrivate *rmp = (MRMutexPrivate *) mtx->d;
@@ -1007,7 +1190,7 @@ public:
 #endif
 
 #ifdef CHECK_RANGE
-	if ( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s",strerror(ret));
+	if ( ret ) fprintf(stderr,"MWaitCondition::wait: wait error:%s\n",strerror(ret));
 #endif
 
 	return (ret == 0);
