@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: mthread_unix.cpp,v 1.5 2002/01/18 21:05:50 maksik Exp $
+** $Id: mthread_unix.cpp,v 1.13 2002/07/06 18:20:55 maksik Exp $
 **
 ** MThread class for Unix
 **
@@ -46,6 +46,7 @@
 //   OS2	- OS/2
 //   OS2EMX	- XFree86 on OS/2 (not PM)
 //   WIN32	- Win32 (Windows 95/98 and Windows NT)
+//   CYGWIN - Cygnus unix simulator under win32
 //   SUN	- SunOS
 //   SOLARIS	- Sun Solaris
 //   HPUX	- HP-UX
@@ -94,6 +95,8 @@
 #define _OS_ULTRIX_
 #elif defined(reliantunix)
 #define _OS_RELIANTUNIX_
+#elif defined(__CYGWIN__)
+#define _OS_CYGWIN_
 #elif defined(linux) || defined(__linux) || defined(__linux__)
 #define _OS_LINUX_
 #elif defined(__FreeBSD__)
@@ -102,6 +105,8 @@
 #define _OS_NETBSD_
 #elif defined(__OpenBSD__)
 #define _OS_OPENBSD_
+#elif defined(__APPLE__)
+#define _OS_DARWIN_
 #elif defined(sgi) || defined(__sgi)
 #define _OS_IRIX_
 #elif defined(__osf__)
@@ -148,9 +153,19 @@
 #endif
 
 #include "mthread.h"
+#include "xsleep.h"
 
 #include <stdio.h>
 //#include <pthread.h>
+//#include <stdio.h>
+#include <sys/types.h>
+//#include <sys/socket.h>
+//#include <netinet/in.h>
+//#include <arpa/inet.h>
+//#include <netdb.h>
+//#include <errno.h>
+
+#include "defines.h"
 #include "mthread_p.h"
 
 
@@ -236,6 +251,7 @@ MMutex::MMutex(bool recursive)
 		d = new MRMutexPrivate();
     else
 		d = new MMutexPrivate();
+	ASSERT(d);
 }
 
 
@@ -465,7 +481,8 @@ bool MThread::isCurrent()
 */
 void MThread::sleep( unsigned long secs )
 {
-    ::sleep(secs);
+    // Chain 'em.  We just don't care.
+    msleep(secs*1000L);
 }
 
 
@@ -475,7 +492,8 @@ void MThread::sleep( unsigned long secs )
 */
 void MThread::msleep( unsigned long msecs )
 {
-    ::usleep( msecs * 1000);
+    // Chain 'em.  We just don't care.
+    usleep(msecs*1000L);
 }
 
 
@@ -485,9 +503,14 @@ void MThread::msleep( unsigned long msecs )
 */
 void MThread::usleep( unsigned long usecs )
 {
-    ::usleep( usecs );
-}
+    // Void functions will *not* tell you if they actually slept that long.
+    // Bad show, qT team.
+    struct timespec rqtp, rmtp;
+    rqtp.tv_nsec = usecs * 1000L;
+    rqtp.tv_sec = rmtp.tv_sec = rmtp.tv_nsec = 0;
 
+    safe_nanosleep(&rqtp, &rmtp);
+}
 
 /*!
   Constructs a new thread. The thread does not actually begin executing
@@ -495,8 +518,9 @@ void MThread::usleep( unsigned long usecs )
 */
 MThread::MThread(bool bAutoDelete /*=false*/)
 {
-    m_bAutoDelete = bAutoDelete;
     d = new MThreadPrivate;
+    ASSERT(d);
+    d->autodelete = bAutoDelete;
 }
 
 
@@ -516,40 +540,37 @@ MThread::~MThread()
 */
 void MThread::exit()
 {
-    dictMutex->lock();
-
-    MapHndlTrd::iterator it = thrDict->find(MThread::currentThread());
-	if ( it != thrDict->end() ) {
-        MThread *there = (*it).second;
-        assert(there);
-        MThreadPrivate* pd = there->d;
-		pd->running = FALSE;
-		pd->finished = TRUE;
-		
-		if (!there->m_bAutoDelete)
-		{
-			pd->thread_done.wakeAll();
-		}
-		else
-		{
-			// multistage safe delete
-			dictMutex->unlock();
-			pd->erase();
-			dictMutex->lock();
-			there->d = NULL;
-			delete there;
-			pd->thread_done.wakeAll();
-			delete pd;
-		}
-    }
-
-    dictMutex->unlock();
-
+	// we have to stop the thread object first
 #if defined(_OS_SOLARIS_)
-    thr_exit(0);
+	thr_exit(0);
 #else
-    pthread_exit(0);
+	pthread_exit(0);
 #endif
+	//
+	dictMutex->lock();
+	MapHndlTrd::iterator it = thrDict->find(MThread::currentThread());
+	if ( it != thrDict->end() ) {
+		MThread *there = (*it).second;
+		ASSERT(there);
+		there->d->running = FALSE;
+		there->d->finished = TRUE;
+		MWaitCondition* pWait = there->d->thread_done;
+		there->d->thread_done = NULL;
+		
+		if (there->d->autodelete)
+		{
+			thrDict->erase(it);
+			dictMutex->unlock();
+			// delete the object
+			delete there;
+			pWait->wakeAll();
+			delete pWait;
+			return;
+		}
+		pWait->wakeAll();
+		delete pWait; // this will destroy object while other threads can still be around "wait"
+	}
+	dictMutex->unlock();
 }
 
 /*!
@@ -568,18 +589,23 @@ void MThread::exit()
 */
 bool MThread::wait(unsigned long time)
 {
-    if (d->finished || ! d->running)
+	MLock lock(*dictMutex);
+	if (d->finished || ! d->running || !d->thread_done)
 		return TRUE;
-
-    return d->thread_done.wait(time);
+	return d->thread_wait(dictMutex, time);
 }
 
-bool MThread::wait( MMutex* mutex, unsigned long time = ULONG_MAX )
+bool MThread::wait( MMutex* mutex, unsigned long time /*=ULONG_MAX*/ )
 {
-    if (d->finished || ! d->running)
+	MLock lock(*dictMutex);
+	if (d->finished || ! d->running || !d->thread_done)
 		return TRUE;
-
-    return d->thread_done.wait(mutex, time);
+	if (mutex)
+		mutex->unlock();
+	bool bRes = d->thread_wait(dictMutex, time);
+	if (mutex)
+		mutex->lock();
+	return bRes;
 }
 
 /*!
@@ -603,6 +629,14 @@ void MThread::start()
     d->init(this);
 }
 
+/*!
+  Sets AutoDelete flag which allows automatic memory clean-up upon the completion
+  of the thread.
+*/
+void MThread::setAutoDelete()
+{
+	d->autodelete = true;
+}
 
 /*!
   Returns TRUE is the thread is finished.
@@ -725,6 +759,7 @@ bool MThread::running() const
 MWaitCondition::MWaitCondition()
 {
     d = new MWaitConditionPrivate;
+    ASSERT(d);
 }
 
 
@@ -733,7 +768,8 @@ MWaitCondition::MWaitCondition()
 */
 MWaitCondition::~MWaitCondition()
 {
-    delete d;
+	// MZ: added reference counting for wait()
+    d->release();
 }
 
 
@@ -871,6 +907,7 @@ MSemaphorePrivate::~MSemaphorePrivate()
 MSemaphore::MSemaphore(int maxcount)
 {
     d = new MSemaphorePrivate(maxcount);
+    ASSERT(d);
 }
 
 
@@ -1022,3 +1059,4 @@ int MSemaphore::total() const {
 //#include "qthread_unix.moc"
 
 //#endif
+
