@@ -56,6 +56,7 @@
 #include <set>
 #include <map>
 #include <algorithm>
+#include <iterator>
 
 // declare namespaces
 using std::min;
@@ -67,6 +68,7 @@ using std::set;
 using std::map;
 using std::vector;
 using std::list;
+using std::insert_iterator;
 
 #endif /*__cplusplus*/
 
@@ -83,6 +85,7 @@ using std::list;
 #include "tstring.h"
 #include "mthread.h"
 //#include "asyncsocket.h"
+#include "conversions.h"
 
 // TODO: move these defines somewhere
 #define MAX_TTL_ACCEPTED	15 // be very tolerant
@@ -95,8 +98,8 @@ using std::list;
 // TODO: this is not the right place for it, really
 // global variables
 extern MMutex g_LibcMutex;
-//extern MMutex g_GetHostByNameMutex;
-#define g_GetHostByNameMutex g_LibcMutex
+extern MMutex g_GetHostByNameMutex;
+//#define g_GetHostByNameMutex g_LibcMutex
 
 // application includes
 //#include "common.h"
