@@ -1,18 +1,31 @@
 /***************************************************************************
-                          packet.cpp  -  description
-                             -------------------
+ Mutella - A commandline/HTTP client for the Gnutella filesharing network.
+
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ packet.cpp  -  Node and packet definitions for gnutella protocol.
+ 
+ the original version of this file was taken from Gnucleus (http://gnucleus.sourceforge.net)
+
     begin                : Tue May 29 2001
     copyright            : (C) 2001 by
     email                : maksik@gmx.co.uk
  ***************************************************************************/
 
-// the original version of this file was taken from Gnucleus (http://gnucleus.sourceforge.net)
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "mutella.h"
+#include "basicstruct.h"
 #include "packet.h"
 
 
@@ -51,4 +64,5 @@ DWORD makeD(XWORD conv)
 	DWORD out = *in;
 
 	return out;
-}*/
\ No newline at end of file
+}*/
+
