@@ -30,7 +30,7 @@
 
 #pragma pack (1)
 
-struct packet_Header		// Size 23
+struct packet_Header			// Size 23
 {
 	GUID  Guid;					// 0  - 15
 	BYTE  Function;				// 16
@@ -40,7 +40,7 @@ struct packet_Header		// Size 23
 };
 
 
-struct packet_Ping			// Size 23
+struct packet_Ping				// Size 23
 {
 	packet_Header Header;		// 0  - 22	
 };
@@ -56,15 +56,23 @@ struct packet_Pong				// Size 37
 };
 
 
-struct packet_Query			// Size 26+
-{		
+struct packet_Bye				// Size 26+
+{
+	packet_Header Header;		// 0  - 22
+	LE(WORD)      le_Code;		// 23 - 24
+	// Message					// 25+
+};
+
+
+struct packet_Query				// Size 26+
+{
 	packet_Header Header;		// 0  - 22
 	LE(WORD)      le_Speed;		// 23 - 24
 	// Search					// 25+
 };
 
 
-struct packet_QueryHit		// Size 35+
+struct packet_QueryHit			// Size 35+
 {
 	packet_Header Header;		// 0  - 22
 	BYTE          TotalHits;	// 23
@@ -78,7 +86,7 @@ struct packet_QueryHit		// Size 35+
 	// ClientGuid				// Last 16 bytes
 };
 
-struct packet_QueryHitItem	// Size 9+
+struct packet_QueryHitItem			// Size 9+
 {
 	LE(DWORD) le_Index;				// 0  - 3
 	LE(DWORD) le_Size;				// 4  - 7
