@@ -72,12 +72,14 @@ public:
 	char m_szGWebCache1[MAX_URL];
 	char m_szGWebCache2[MAX_URL];
 	char m_szGWebCache3[MAX_URL];
+	bool m_bGWebCacheFollowRedirect;
 
 	// various filters and blacklists
 	bool m_bBlackListActive;
 	TPropertySet<IPNET> m_ipnetsetBlackList;
 	bool m_bWordFilterActive;
 	TPropertySet<CString> m_setWordFilter;
+	TPropertySet<IPNET> m_ipnetsetWhiteList;
 	
 	// Timeouts
 	DWORD m_dwConnectTimeout;
@@ -87,10 +89,6 @@ public:
 
 	// Search
 	char    m_szDownloadPath[MAX_PATH];
-	//bool	m_bGroupbySize;
-	//bool	m_bGroupbyName;
-	bool    m_bGroupbyFuzzyRules;
-	bool	m_bScreenBusy;
 	bool    m_bStrictSearch;
 	int     m_nMinFileKSize;
 	int     m_nMaxFileKSize;
@@ -99,6 +97,7 @@ public:
 	int     m_nMaxPerSearchResults;
 	bool    m_bSaveSearches;
 	//int     m_nMaxResultsDisplayed;
+	bool    m_bStrictSha1Grouping;
 
 	// Share
 	char m_szSharePath[MAX_PATH];
@@ -121,6 +120,7 @@ public:
 	double m_dBandwidthTotal;
 	double m_dBandwidthTransfer;
 	double m_dBandwidthConnects;
+	double m_dBandwidthDownl;
 	double m_dMinDownSpeed;
 	double m_dMinUpSpeed;
 
