@@ -133,6 +133,21 @@ CString MProperty::Format()
 	return "unsupported_type";
 }
 
+LPCSTR MProperty::FormatPropType()
+{
+	switch (GetPropType())
+	{
+		case PT_BOOL : return "bool";
+		case PT_INT : return "int";
+		case PT_DWORD : return "uint";
+		case PT_DOUBLE : return "real";
+		case PT_STRING : return "string";
+		case PT_IP : return "ip";
+		case PT_IPNET : return "net";
+	}
+	return "none";
+}
+
 bool MProperty::Read(MIniFile* pIniFile)
 {
 	// somewhat lasy way
@@ -140,7 +155,7 @@ bool MProperty::Read(MIniFile* pIniFile)
 	char tmp[1024]; // TODO: check if it sufficient
 	if (pIniFile->ReadStr(name, tmp, "", true)) // do trim
 	{
-		if (strlen(tmp))
+		if (strlen(tmp) || PT_STRING==GetPropType())
 			return SetFromStr(tmp);
 		if (PT_STRING==GetPropType())
 			return true; // its OK when string is set to ""
@@ -205,6 +220,18 @@ CString TPropertySet<CString>::Format()
 	return sTmp;
 }
 
+template<>
+void TPropertySet<CString>::CopyToStrSet(set<CString>& setStr)
+{
+	if (m_pMutexSet)
+		m_pMutexSet->lock();
+	
+	setStr = m_set;
+	
+	if (m_pMutexSet)
+		m_pMutexSet->unlock();
+}
+
 template<>
 bool TPropertySet<CString>::InsertStr(const CString& str)
 {
@@ -323,13 +350,32 @@ MPropertyContainer::~MPropertyContainer()
 	}
 }
 
-bool MPropertyContainer::Read(LPCSTR szName)
+bool MPropertyContainer::DoMigration(LPCSTR szOldVersion, LPCSTR szCurrentVersion, MIniFile* pInifile)
+{
+	return false; // nothing was modified
+}
+
+bool MPropertyContainer::Read(LPCSTR szName, LPCSTR szCurrentVersion /*=NULL*/)
 {
 	CString path = ExpandPath(szName);
 	MIniFile inifile;
 	if (!inifile.LoadFile(path.c_str()))
 		return false;
 	bool bOK = true;
+	// version control
+	if (szCurrentVersion)
+	{
+		char tmp[1024];
+		inifile.SetSection("Version");
+		if (!inifile.ReadStr("CurrentVersion", tmp, "", true))
+			bOK = false;
+		if (strcmp(tmp, szCurrentVersion)<0)
+			if (DoMigration(tmp, szCurrentVersion, &inifile))
+			{
+				inifile.FlushFile();
+			}
+	}
+	//
 	map<CString, MProperty*>map_copy = m_props;
 	for (sec_iterator its = sec_begin();its!=sec_end();its++)
 	{
@@ -353,12 +399,20 @@ bool MPropertyContainer::Read(LPCSTR szName)
 	return map_copy.empty();
 }
 
-bool MPropertyContainer::Write(LPCSTR szName)
+bool MPropertyContainer::Write(LPCSTR szName, LPCSTR szCurrentVersion /*=NULL*/)
 {
 	CString path = ExpandPath(szName);
 	MIniFile inifile(INI_FxALLOWxCREATE);
 	if (!inifile.LoadFile(path.c_str()))
 		return false;
+	bool bOK = true;
+	// version control
+	if (szCurrentVersion)
+	{
+		inifile.SetSection("Version");
+		if (!inifile.WriteStr("CurrentVersion", szCurrentVersion))
+			bOK = false;
+	}
 	//
 	map<CString, MProperty*>map_copy = m_props;
 	for (sec_iterator its = sec_begin();its!=sec_end();its++)
@@ -368,7 +422,6 @@ bool MPropertyContainer::Write(LPCSTR szName)
 			map_copy.erase(it->first);
 	}
 	//
-	bool bOK = true;
 	for (iterator it = map_copy.begin();it!=map_copy.end();it++)
 	{
 		if (it->second->IsPersistent() && !it->second->Write(&inifile))
@@ -454,7 +507,7 @@ MProperty* MPropertyContainer::AddNewProperty(LPCSTR szName, int nType, int nStr
 	return pP;
 }
 
-bool MPropertyContainer::AddSet(LPCSTR szName, MPropertySet* pSet)
+MProperty* MPropertyContainer::AddSet(LPCSTR szName, MPropertySet* pSet)
 {
 	if (FindProperty(szName))
 		return false;
@@ -463,7 +516,20 @@ bool MPropertyContainer::AddSet(LPCSTR szName, MPropertySet* pSet)
 	m_props[szName] = pP;
 	if (m_pCurSec)
 		m_pCurSec->m_props[szName] = pP;
-    return pP!=NULL;
+	return pP;
+}
+
+MProperty* MPropertyContainer::AddSet(LPCSTR szName, MPropertySet* pSet, LPCSTR szSetInit)
+{
+	if (FindProperty(szName))
+		return false;
+	MProperty* pP = new TProperty<MPropertySet,PT_SET>(szName, pSet);
+	ASSERT(pP);
+	m_props[szName] = pP;
+	if (m_pCurSec)
+		m_pCurSec->m_props[szName] = pP;
+	pSet->SetFromStr(szSetInit);
+	return pP;
 }
 
 MProperty* MPropertyContainer::AddProperty(LPCSTR szName, bool * pb)
