@@ -45,6 +45,8 @@ class MPropertySet;
 
 struct Property {
 	char name[PNAME_SIZE+1];
+	const char * description;
+	const char * description_full;
 	int  type;
 	int  size;
 	union {
@@ -73,12 +75,18 @@ public:
 	MProperty(LPCSTR szName){
 		strncpy(name,szName,PNAME_SIZE);
 		name[PNAME_SIZE] = '\0';
+		description = NULL;
+		description_full = NULL;
 		type = PT_NONE;
 		size = 0;
 		m_bPersistent = true;
 	}
 	int GetPropType(){return type & 0xFFFF;}
+	LPCSTR FormatPropType();
 	LPCSTR GetPropertyName(){return name;}
+	LPCSTR GetPropertyDescriptionShort() { return description; }
+	LPCSTR GetPropertyDescriptionFull() { return description_full; }
+	void   SetPropertyDescription(LPCSTR szDescrShort, LPCSTR szDescrFull) { description = szDescrShort; description_full = szDescrFull; }
 	bool   GetBoolValue(){ASSERT(type & PT_BOOL); return (type & PT_BYREF) ? *pbVal : bVal;}
 	void   SetBoolValue(bool b){ASSERT(type & PT_BOOL); if (type & PT_BYREF) *pbVal = b; else bVal = b; }
 	int    GetIntValue(){ASSERT(type & PT_INT); return (type & PT_BYREF) ? *pnVal : nVal;}
@@ -238,12 +246,13 @@ public:
 	MPropertySet() {}
 	virtual ~MPropertySet() {}
 	//
-	virtual bool InsertStr(const CString&)  = 0;
-	virtual bool RemoveStr(const CString&)  = 0;
-	virtual bool IsInStr(const CString&)    = 0;
+	virtual bool InsertStr(const CString&)   = 0;
+	virtual bool RemoveStr(const CString&)   = 0;
+	virtual bool IsInStr(const CString&)     = 0;
 	//
-	virtual bool SetFromStr(const CString&) = 0;
-	virtual CString Format()                = 0;
+	virtual bool SetFromStr(const CString&)  = 0;
+	virtual CString Format()                 = 0;
+	virtual void CopyToStrSet(set<CString>&) = 0;
 };
 
 //////////////////////////////////////////////////////////////////////////////////////////////
@@ -251,14 +260,15 @@ public:
 template<class t_type>
 class TPropertySet : public MPropertySet{
 public:
-	typedef set<t_type>::iterator iterator;
-	typedef set<t_type>::reverse_iterator reverse_iterator;
+	typedef typename set<t_type>::iterator iterator;
+	typedef typename set<t_type>::reverse_iterator reverse_iterator;
 	//
 	TPropertySet(MMutex* pMutex) : m_pMutexSet(pMutex) {}
 	//
-	virtual bool SetFromStr(const CString&); // this five functions will rely on
-	virtual CString Format();                // the implementation of SetFromStr()
-	virtual bool InsertStr(const CString&);  // and Format() of TProperty<t_type>
+	virtual bool SetFromStr(const CString&);  // this five functions will rely on
+	virtual CString Format();                 // the implementation of SetFromStr()
+	virtual bool InsertStr(const CString&);   // and Format() of TProperty<t_type>
+	virtual void CopyToStrSet(set<CString>&); //
 	virtual bool RemoveStr(const CString&);
 	virtual bool IsInStr(const CString&);
 	//
@@ -268,6 +278,21 @@ public:
 	void CopyTo(set<t_type>&);
 	template<class t_item_type>
 	bool IsIn(const t_item_type&);
+	//
+	template<typename _type, typename _func>
+	bool ForEach(const _type& param, _func func_ptr) {
+		if (m_pMutexSet)
+			m_pMutexSet->lock();
+		for (iterator it = m_set.begin(); it != m_set.end(); ++it)
+			if(!(*func_ptr)(param, *it)) {
+				if (m_pMutexSet)
+					m_pMutexSet->unlock();
+				return false;
+			}
+		if (m_pMutexSet)
+			m_pMutexSet->unlock();
+		return true;
+	}
 protected:
 	set<t_type> m_set;
 	MMutex* m_pMutexSet;
@@ -281,6 +306,8 @@ bool TPropertySet<CString>::SetFromStr(const CString& str);
 template<>
 CString TPropertySet<CString>::Format();
 template<>
+void TPropertySet<CString>::CopyToStrSet(set<CString>&);
+template<>
 bool TPropertySet<CString>::InsertStr(const CString& str);
 template<>
 bool TPropertySet<CString>::RemoveStr(const CString& str);
@@ -342,6 +369,20 @@ CString TPropertySet<t_type>::Format()
 	return sTmp;
 }
 
+template<class t_type>
+void TPropertySet<t_type>::CopyToStrSet(set<CString>& setStr)
+{
+	if (m_pMutexSet)
+		m_pMutexSet->lock();
+	for (iterator it=m_set.begin(); it!=m_set.end(); ++it)
+	{
+		TProperty<t_type> prop("tmp", const_cast<t_type*>(&(*it)));
+		setStr.insert(prop.Format());
+	}
+	if (m_pMutexSet)
+		m_pMutexSet->unlock();
+}
+
 template<class t_type>
 bool TPropertySet<t_type>::InsertStr(const CString& str)
 {
@@ -460,7 +501,7 @@ public:
     typedef prop_iterator iterator;
 
 	MPropertyContainer();
-	~MPropertyContainer();
+	virtual ~MPropertyContainer();
 	
 	int Transfer(MPropertyContainer* pPC);
 	inline int Count(){return m_props.size();}
@@ -481,7 +522,8 @@ public:
 	MProperty* AddProperty(LPCSTR szName, IPNET * pnet);
 	MProperty* AddProperty(LPCSTR szName, IPNET * pnet, const IPNET& netInit);
 	// set functions
-	bool AddSet(LPCSTR szName, MPropertySet* pSet);
+	MProperty* AddSet(LPCSTR szName, MPropertySet* pSet);
+	MProperty* AddSet(LPCSTR szName, MPropertySet* pSet, LPCSTR szSetInit);
 	
 	// general service	
 	MProperty* FindProperty(LPCSTR szName);
@@ -502,8 +544,10 @@ public:
 	inline sec_iterator sec_begin(){return m_secs.begin();}
 	inline sec_iterator sec_end(){return m_secs.end();}
 	// save/load
-	bool Read(LPCSTR szName);
-	bool Write(LPCSTR szName);
+	bool Read(LPCSTR szName, LPCSTR szCurrentVersion = NULL);
+	bool Write(LPCSTR szName, LPCSTR szCurrentVersion = NULL);
+
+	virtual bool DoMigration(LPCSTR szOldVersion, LPCSTR szCurrentVersion, MIniFile* pInifile);
 protected:
 	map<CString, MProperty*> m_props;
 	map<CString, MPropertySection*> m_secs;
