@@ -66,6 +66,13 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdlib.h>
+
+#ifdef HAVE_SYS_POLL_H
+#include <sys/poll.h>
+#else
+#include "fakepoll.h"
+#endif
+
 #include "term_help.h"
 #include "readline4fix.h"
 
@@ -181,12 +188,34 @@ char read_key(int wait)
       break;
     }
   }
+  //printf("c = %d\n", c);
 
   fcntl(STDIN_FILENO,F_SETFL,0);
   raw_mode(0);
   return c;
 }
 
+int wait_key(int wait)
+{
+  int nRet;
+  struct pollfd pf;
+  pf.fd = STDIN_FILENO;
+  pf.events = POLLIN;
+  pf.revents = 0;
+  nRet = 0;
+  raw_mode(1);
+  fcntl(STDIN_FILENO,F_SETFL,O_NONBLOCK);
+
+  if ( poll(&pf, 1, (wait==-1) ? -1 : (wait * 1000) ) >= 0 &&
+       ( pf.revents & (POLLERR | POLLHUP | POLLNVAL)) == 0 &&
+       ( pf.revents & POLLIN ))
+         nRet = 1;
+
+  fcntl(STDIN_FILENO,F_SETFL,0);
+  raw_mode(0);
+  
+  return nRet;
+}
 
 #ifdef USE_CURSES
 
