@@ -22,13 +22,51 @@
 #include "config.h"
 #endif
 
-//#include <iostream.h>
+#ifdef TSTRING_DEBUG
+#include "mutella.h"
+#endif //TSTRING_DEBUG
+
+#include "defines.h"
 #include "tstring.h"
 #include "string.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
+#ifdef TSTRING_DEBUG
+
+static MMutex* g_pMutexStringAlloc;
+static int g_nStringsSize;
+
+void on_alloc(int nSize)
+{
+	if (!nSize)
+		return;
+	g_pMutexStringAlloc->lock();
+	g_nStringsSize += nSize;
+	cout << "+ : " << g_nStringsSize <<  endl;
+	g_pMutexStringAlloc->unlock();
+}
+
+void on_free(int nSize)
+{
+	static bool bInit = true;
+	if (bInit)
+	{
+		g_pMutexStringAlloc = new MMutex;
+		g_nStringsSize = 0;
+		bInit = false;
+	}
+	if (!nSize)
+		return;
+	g_pMutexStringAlloc->lock();
+	g_nStringsSize -= nSize;
+	cout << "- : " << g_nStringsSize <<  endl;
+	g_pMutexStringAlloc->unlock();
+}
+
+#endif //TSTRING_DEBUG
+
 // partial specializations for "traits"
 template<>
 char TString<char>::_null = '\0';
