@@ -30,12 +30,61 @@
 
 #define CHECK_CONSIST check_consist();
 
+extern void on_alloc(int nSize);
+extern void on_free(int nSize);
+
+#define ON_ALLOC(size) on_alloc(size)
+#define ON_FREE(size) on_free(size)
+
 #else
 
 #define CHECK_CONSIST
+#define ON_ALLOC(size)
+#define ON_FREE(size)
 
 #endif // TSTRING_DEBUG
 
+template<class T> class TString;
+
+template <class T> bool operator== (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator== (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator== (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator== (const TString<T>& lhs,T cRhs);
+template <class T> bool operator== (T cLhs,const TString<T>& rhs);
+template <class T> bool operator!= (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator!= (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator!= (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator!= (const TString<T>& lhs,T cRhs);
+template <class T> bool operator!= (T cLhs,const TString<T>& rhs);
+template <class T> bool operator> (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator> (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator> (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator> (const TString<T>& lhs,T cRhs);
+template <class T> bool operator> (T cLhs,const TString<T>& rhs);
+template <class T> bool operator< (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator< (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator< (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator< (const TString<T>& lhs,T cRhs);
+template <class T> bool operator< (T cLhs,const TString<T>& rhs);
+template <class T> bool operator>= (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator>= (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator>= (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator>= (const TString<T>& lhs,T cRhs);
+template <class T> bool operator>= (T cLhs,const TString<T>& rhs);
+template <class T> bool operator<= (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> bool operator<= (const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+template <class T> bool operator<= (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> bool operator<= (const TString<T>& lhs,T cRhs);
+template <class T> bool operator<= (T cLhs,const TString<T>& rhs);
+template <class T> TString<T> operator+ (const TString<T>& lhs, const TString<T>& rhs);
+template <class T> TString<T> operator+ (const TString<T>& lhs,typename TString<T>::LPCT pszRhs);
+template <class T> TString<T> operator+ (typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+template <class T> TString<T> operator+ (const TString<T>& lhs, T cRhs);
+template <class T> TString<T> operator+ (T cLhs, const TString<T>& rhs);
+template <class T> std::ostream& operator<< (std::ostream& os, const TString<T>& st);
+template <class T> std::istream& operator>> (std::istream& is, const TString<T>& st);
+
+
 template<class T>
 class TString{
 public:
@@ -177,6 +226,7 @@ public:
 		//delete [] m_pBuffer;
 		free(m_pBuffer);
 		m_pBuffer = NULL;
+		ON_FREE(m_nBufferSize);
 		m_nBufferSize = 0;
 		m_nLength = 0;
 		CHECK_CONSIST
@@ -200,6 +250,7 @@ public:
 		LPT pOldBuf = m_pBuffer;
 		int nOldLen = m_nLength;
 		// fake clear()
+		ON_FREE(m_nBufferSize);
 		m_nBufferSize = 0;
 		m_nLength = 0;
 		m_pBuffer = NULL;
@@ -223,6 +274,7 @@ public:
 		LPT pOldBuf = m_pBuffer;
 		int nOldLen = m_nLength;
 		// fake clear()
+		ON_FREE(m_nBufferSize);
 		m_nBufferSize = 0;
 		m_nLength = 0;
 		m_pBuffer = NULL;
@@ -373,11 +425,27 @@ protected:
 	void _realloc(int nLength) { // allocate buffer to hold string of length nLength (no optimisation)
 #ifdef CHECK_RANGE
 		if (nLength < 0)
-			fprintf(stderr, "TString::_reallocNreplace nLength < 0\n");
+			fprintf(stderr, "TString::_realloc nLength < 0\n");
 #endif
 		// TODO: shrink buffer if the string is tooo short (check usage of this function before)
 		if (nLength+1>m_nBufferSize) {
-			m_nBufferSize = nLength+1; // realloc is exact function
+			ON_FREE(m_nBufferSize);
+			m_nBufferSize = nLength+1;
+			// round up the size to improve performance and reduce memory fragmentation
+			m_nBufferSize = (m_nBufferSize + 15) & 0x7FFFFFF0l; // sizes to 16 bytes
+			if (m_nBufferSize > 255)
+			{
+				// round up to 4 binary digits
+				m_nBufferSize >>= 4; // we have already aligned it to 16
+				register int n;
+				for(n = 4; (m_nBufferSize & 0xF) != m_nBufferSize; ++n)
+				{
+					m_nBufferSize += 1;
+					m_nBufferSize >>= 1;
+				}
+				m_nBufferSize <<= n;
+			}
+			ON_ALLOC(m_nBufferSize);
 			m_pBuffer = (T*) realloc(m_pBuffer, sizeof(T)*m_nBufferSize);
 			ASSERT(m_pBuffer);
 		}
@@ -387,8 +455,11 @@ protected:
 		if ( nAdd < 0 )
 			fprintf(stderr, "TString::_reallocNtail nAdd < 0\n");
 #endif
+		if (!nAdd)
+			return;
+		
 		if (m_nLength+1+nAdd>m_nBufferSize){
-			_realloc(m_nLength+nAdd+16); // TODO: optimize reallocations
+			_realloc(m_nLength+nAdd);
 		}
 		if (pTail) {
 #ifdef CHECK_RANGE
@@ -405,9 +476,16 @@ protected:
 		if (nLength < 0)
 			fprintf(stderr, "TString::_reallocNreplace nLength < 0\n");
 #endif
-		// TODO: shrink buffer if the string is tooo short (check usage of this function before)
+		if (!nLength)
+		{
+			if (m_pBuffer)
+				*m_pBuffer = '\0';
+			m_nLength = 0;
+			return;
+		}
+		
 		if (nLength+1>m_nBufferSize) {
-			_realloc(nLength+16); // TODO: optimize this +16
+			_realloc(nLength);
 		}
 		if (pContent) {
 #ifdef CHECK_RANGE
@@ -549,6 +627,16 @@ protected:
 	friend bool operator< <>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
 	friend bool operator< <>(const TString<T>& lhs,T cRhs);
 	friend bool operator< <>(T cLhs,const TString<T>& rhs);
+	friend bool operator>= <>(const TString<T>& lhs, const TString<T>& rhs);
+	friend bool operator>= <>(const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+	friend bool operator>= <>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+	friend bool operator>= <>(const TString<T>& lhs,T cRhs);
+	friend bool operator>= <>(T cLhs,const TString<T>& rhs);
+	friend bool operator<= <>(const TString<T>& lhs, const TString<T>& rhs);
+	friend bool operator<= <>(const TString<T>& lhs, typename TString<T>::LPCT pszRhs);
+	friend bool operator<= <>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
+	friend bool operator<= <>(const TString<T>& lhs,T cRhs);
+	friend bool operator<= <>(T cLhs,const TString<T>& rhs);
 	friend TString<T> operator+ <>(const TString<T>& lhs, const TString<T>& rhs);
 	friend TString<T> operator+ <>(const TString<T>& lhs,typename TString<T>::LPCT pszRhs);
 	friend TString<T> operator+ <>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
@@ -629,7 +717,7 @@ bool operator>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs) { return
 template<class T>
 bool operator>(const TString<T>& lhs, T cRhs) { if (!lhs.m_pBuffer) return true; return lhs.m_pBuffer[0] < cRhs; }
 template<class T>
-bool operator>(T CLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return false; return cLhs < rhs.m_pBuffer[0]; }
+bool operator>(T cLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return false; return cLhs < rhs.m_pBuffer[0]; }
 
 template<class T>
 bool operator<(const TString<T>& lhs, const TString<T>& rhs) { return 0 > TString<T>::_strcmp(lhs.m_pBuffer, rhs.m_pBuffer); }
@@ -640,7 +728,29 @@ bool operator<(typename TString<T>::LPCT pszLhs, const TString<T>& rhs) { return
 template<class T>
 bool operator<(const TString<T>& lhs, T cRhs) { if (!lhs.m_pBuffer) return false; return lhs.m_pBuffer[0] > cRhs; }
 template<class T>
-bool operator<(T CLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return true; return cLhs > rhs.m_pBuffer[0]; }
+bool operator<(T cLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return true; return cLhs > rhs.m_pBuffer[0]; }
+
+template<class T>
+bool operator>=(const TString<T>& lhs, const TString<T>& rhs) { return 0 <= TString<T>::_strcmp(lhs.m_pBuffer, rhs.m_pBuffer); }
+template<class T>
+bool operator>=(const TString<T>& lhs,typename TString<T>::LPCT pszRhs) { return 0 <= TString<T>::_strcmp(lhs.m_pBuffer, pszRhs); }
+template<class T>
+bool operator>=(typename TString<T>::LPCT pszLhs, const TString<T>& rhs) { return 0 <= TString<T>::_strcmp(pszLhs, rhs.m_pBuffer); }
+template<class T>
+bool operator>=(const TString<T>& lhs, T cRhs) { if (!lhs.m_pBuffer) return true; return lhs.m_pBuffer[0] <= cRhs; }
+template<class T>
+bool operator>=(T cLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return false; return cLhs <= rhs.m_pBuffer[0]; }
+
+template<class T>
+bool operator<=(const TString<T>& lhs, const TString<T>& rhs) { return 0 >= TString<T>::_strcmp(lhs.m_pBuffer, rhs.m_pBuffer); }
+template<class T>
+bool operator<=(const TString<T>& lhs,typename TString<T>::LPCT pszRhs) { return 0 >= TString<T>::_strcmp(lhs.m_pBuffer, pszRhs); }
+template<class T>
+bool operator<=(typename TString<T>::LPCT pszLhs, const TString<T>& rhs) { return 0 >= TString<T>::_strcmp(pszLhs, rhs.m_pBuffer); }
+template<class T>
+bool operator<=(const TString<T>& lhs, T cRhs) { if (!lhs.m_pBuffer) return false; return lhs.m_pBuffer[0] >= cRhs; }
+template<class T>
+bool operator<=(T cLhs, const TString<T>& rhs) { if (!rhs.m_pBuffer) return true; return cLhs >= rhs.m_pBuffer[0]; }
 
 template<class T>
 TString<T> operator+(const TString<T>& lhs, const TString<T>& rhs) {
@@ -699,6 +809,7 @@ std::ostream& operator<<(std::ostream& os, const TString<T>& st) {
 template<class T>
 std::istream& operator>>(std::istream& is, const TString<T>& st){
 	ASSERT(0);// not implemented properly
+	st.reserve(1024);
 	return is >> st.m_pBuffer;
 }
 
